/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * attrib_dlg
 *
 * Synopsis:
 *   Routines used to display the attributes dialog box, which is used
 *   to edit various attributes within the system.
 *
 * $Id: attrib_dlg.c,v 1.45 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#include <ctype.h>
#include <gnome.h>
#include <libpq-fe.h>
#include <stdlib.h>

#include <preps/gstr_utils.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "attrib_dlg_db.h"
#include "prepsmain.h"
#include "util.h"

/*
 *  Define some basic queries
 */
#define NAME_POS        0
#define RANK_POS        1
#define SEVR_POS        1
#define ACTIVE_POS      2
#define PK_POS          2
#define DESCR_POS       3
#define IS_CLOSED_POS   4
#define STATUS_QUERY "SELECT name, order_num, active, descr, is_closed_status \
FROM status WHERE status_num = %d"
#define SEVERITY_QUERY "SELECT name, order_num, active, descr \
FROM severity WHERE severity_num = %d"
#define TYPE_QUERY "SELECT name, severity_num, active, descr FROM \
problem_type WHERE type_num = %d"
#define SEVRS_QUERY "SELECT name, order_num, severity_num FROM \
severity ORDER BY order_num"

/*
 *  Object names
 */
#define DEF_SEVR_COMBO            "def_sevr_combo"
#define NAME_EDIT                 "name_edit"
#define DESCR_EDIT                "descr_edit"
#define ACTIVE_CHECK_BOX          "active_butt"
#define IS_CLOSED_CHECK_BOX       "is_closed_butt"
#define DEF_SEVR_LIST             "sevr_pk_name_list"
#define SEVR_CLIST                "sevr_clist"
#define STAT_CLIST                "stat_clist"
#define TYPE_CLIST                "type_clist"
#define NOTEBOOK                  "attrib_notebook"
#define SEVR_SHOW_INACTIVE        "sevr_shinact_cb"
#define STAT_SHOW_INACTIVE        "stat_shinact_cb"
#define TYPE_SHOW_INACTIVE        "type_shinact_cb"

/*
 * Used to store which table we are dealing with for a given edit
 * dialog.
 */
#define TABLE_NUM                 "table_num"
#define ED_STAT_TBL               1
#define ED_SEVR_TBL               2
#define ED_TYPE_TBL               3

#define SEVR_PAGE                 0
#define STAT_PAGE                 1
#define TYPE_PAGE                 2

static const gchar *my_child_list = "CHILD_LIST";

/*
 * This application can only have one attribute list dialog visible at
 * a time.  This is the one...
 */
static GtkWidget  *attrib_dlg = NULL;

typedef struct pk_order_num {
   gint pk;
   gint order_num;
} pk_order_num;


/*
 *  draw_xxxx_list
 *  
 *  These local functions are used to draw the clist boxes for the 
 *  attributes dialog.  They also display an error box upon an error
 *  occuring.
 *  
 *  These functions return TRUE upon success, and FALSE upon a 
 *  failure.
 */
static gboolean 
draw_sevr_list (GtkWindow *parent, PGconn *conn)
{
   char            *buffer[3] = {NULL, NULL, NULL};
   gint             ntuples;
   gint             i;
   pk_order_num    *pk_on;
   PGresult        *res;
   GtkCList        *lb_widget;  /* The list box we want to draw */
   GtkCheckButton  *cb_widget;  /* The "show inactive" check box */
   gboolean         suppress_inactive;
   gboolean         rtn_val = FALSE;
   
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (parent), SEVR_CLIST));
   g_assert (lb_widget != NULL);
   cb_widget = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (parent), 
            SEVR_SHOW_INACTIVE));
   g_assert (cb_widget != NULL);
   suppress_inactive = !gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (cb_widget));

   gtk_clist_clear (lb_widget);
   res = read_sevr_data (conn, suppress_inactive);
   
   if (chk_sql_error (res, "reading severity data"))
      {
      ntuples = PQntuples (res);
      
      for (i = 0; i < ntuples; i++)
         {
         buffer[0] = (char *)g_malloc (strlen (get_name (res, i)) + 1);
         strcpy (buffer[0], get_name (res, i));
         buffer[1] = (char *)g_malloc (strlen (get_active_yn (res, i)) + 1);
         strcpy (buffer[1], get_active_yn (res, i));
         buffer[2] = (char *)g_malloc (strlen (get_descr (res, i)) + 1);
         strcpy (buffer[2], get_descr (res, i));
         
         pk_on = g_malloc (sizeof (pk_order_num));
         pk_on->pk = atoi (get_pk_num (res, i));
         pk_on->order_num = atoi (get_rank (res, i));

         gtk_clist_append (lb_widget, buffer);
         gtk_clist_set_row_data_full (lb_widget, i, pk_on, g_free);
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         }

      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val;
}


static gboolean 
draw_stat_list (GtkWindow *parent, PGconn *conn)
{
   char            *buffer[3] = {NULL, NULL, NULL};
   gint             ntuples;
   gint             i;
   pk_order_num    *pk_on; 
   PGresult        *res;
   GtkCList        *lb_widget;
   GtkCheckButton  *cb_widget;
   gboolean         suppress_inactive;
   gboolean         rtn_val = FALSE;
   
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (parent), STAT_CLIST));
   g_assert (lb_widget != NULL);
   cb_widget = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (parent), 
                                             STAT_SHOW_INACTIVE));
   g_assert (cb_widget != NULL);
   suppress_inactive = !gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (cb_widget));
   
   gtk_clist_clear (lb_widget);
   res = read_stat_data (conn, suppress_inactive);
   
   if (chk_sql_error (res, "reading status data"))
      {
      ntuples = PQntuples (res);
      
      for (i = 0; i < ntuples; i++)
         {
         buffer[0] = (char *)g_malloc (strlen (get_name (res, i)) + 1);
         strcpy (buffer[0], get_name (res, i));
         buffer[1] = (char *)g_malloc (strlen (get_active_yn (res, i)) + 1);
         strcpy (buffer[1], get_active_yn (res, i));
         buffer[2] = (char *)g_malloc (strlen (get_descr (res, i)) + 1);
         strcpy (buffer[2], get_descr (res, i));
         
         pk_on = g_malloc (sizeof (pk_order_num));
         pk_on->pk = atoi (get_pk_num (res, i));
         pk_on->order_num = atoi (get_rank (res, i));

         gtk_clist_append (lb_widget, buffer);
         gtk_clist_set_row_data_full (lb_widget, i, pk_on, g_free);
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         }
      
      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val;
}


static gboolean 
draw_type_list (GtkWindow *parent, PGconn *conn)
{
   char            *buffer[4] = {NULL, NULL, NULL, NULL};
   gint             ntuples;
   gint             i;
   pk_order_num    *pk_on;
   PGresult        *res;
   GtkCList        *lb_widget;
   GtkCheckButton  *cb_widget;
   gboolean         suppress_inactive;
   gboolean         rtn_val = FALSE;
   
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (parent), TYPE_CLIST));
   g_assert (lb_widget != NULL);
   cb_widget = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (parent), 
                                             TYPE_SHOW_INACTIVE));
   g_assert (cb_widget != NULL);
   suppress_inactive = !gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (cb_widget));

   gtk_clist_clear (lb_widget);
   res = read_type_data (conn, suppress_inactive);
   
   if (chk_sql_error (res, "reading type data"))
      {
      ntuples = PQntuples (res);
      
      for (i = 0; i < ntuples; i++)
         {
         buffer[0] = (char *)g_malloc (strlen (get_name (res, i)) + 1);
         strcpy (buffer[0], get_name (res, i));
         buffer[1] = (char *)g_malloc (strlen (get_def_sevr (res, i)) + 1);
         strcpy (buffer[1], get_def_sevr (res, i));
         buffer[2] = (char *)g_malloc (strlen (get_active_yn (res, i)) + 1);
         strcpy (buffer[2], get_active_yn (res, i));
         buffer[3] = (char *)g_malloc (strlen (get_descr (res, i)) + 1);
         strcpy (buffer[3], get_descr (res, i));
         
         pk_on = g_malloc (sizeof (pk_order_num));
         pk_on->pk = atoi (get_pk_num (res, i));
         pk_on->order_num = atoi (get_rank (res, i));

         gtk_clist_append (lb_widget, buffer);
         gtk_clist_set_row_data_full (lb_widget, i, pk_on, g_free);
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         g_free (buffer[3]);
         }
      
      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val; 
}


/*
 * Create the title for the edit dialog based on the type of operation 
 * being performed, and the table it is being performed on.
 *
 * The string that is created should be freed by the calling function.
 */
static GString*
build_edit_dlg_title (gboolean is_update,
                      gint     table_num)
{
   GString *str;

   if (is_update)
      {
      str = g_string_new (_("Update "));
      }
   else
      {
      str = g_string_new (_("Add "));
      }

   switch (table_num)
      {
      case ED_STAT_TBL:
         str = g_string_append (str, _("Status"));
         break;
      case ED_SEVR_TBL:
         str = g_string_append (str, _("Severity"));
         break;
      case ED_TYPE_TBL:
         str = g_string_append (str, _("Problem Type"));
         break;
      default:
         g_assert_not_reached();
      }

   return str;
}


/*------------------------------------------------------------------------
 * edit_dlg signal handlers
 * 
 * These signal handlers are for the locally defined edit window
 * that is used to edit status and severity data.
 * -----------------------------------------------------------------------
 */
static void 
on_edit_dlg_show (GtkWidget  *widget,
                  gpointer    user_data)
{
   GString      *query;
   gboolean      query_error = FALSE;
   PGresult     *result;
   GtkWidget    *child;
   gboolean      is_update;
   PGconn       *conn;
   gint          pk_num;
   gint          table_num;

   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                  PK_NUM));
   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                      IS_UPDATE)) == 1);
   table_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                     TABLE_NUM));
   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   
   if (is_update)
      {
      query = g_string_new ("");
      switch (table_num)
         {
         case ED_STAT_TBL:
            g_string_sprintf (query, STATUS_QUERY, pk_num);
            break;
         case ED_SEVR_TBL:
            g_string_sprintf (query, SEVERITY_QUERY, pk_num);
            break;
         default:
            g_assert_not_reached ();
         }
      
      result = PQexec (conn, query->str);
      if (chk_sql_error (result, "getting data to edit"))
         {
         child = get_widget (widget, DESCR_EDIT);
         g_assert (child != NULL);
         gtk_text_insert (GTK_TEXT (child), NULL, NULL, NULL,
               PQgetvalue (result, 0, DESCR_POS), -1);
         child = get_widget (widget, NAME_EDIT);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child), 
               PQgetvalue (result, 0, NAME_POS));
         child = get_widget (widget, ACTIVE_CHECK_BOX);
         g_assert (child != NULL);
         if ((PQgetvalue (result, 0, ACTIVE_POS))[0] == 't')
            {
            gtk_toggle_button_set_active
               (GTK_TOGGLE_BUTTON (child), TRUE);
            }
         else
            {
            gtk_toggle_button_set_active
               (GTK_TOGGLE_BUTTON (child), FALSE);
            }
         if (table_num == ED_STAT_TBL)
            {
            child = get_widget (widget, IS_CLOSED_CHECK_BOX);
            g_assert (child != NULL);
            if (toupper((PQgetvalue (result, 0, IS_CLOSED_POS))[0]) == 'T')
               {
               gtk_toggle_button_set_active
                  (GTK_TOGGLE_BUTTON (child), TRUE);
               }
            else
               {
               gtk_toggle_button_set_active
                  (GTK_TOGGLE_BUTTON (child), FALSE);
               }
            }
         }
      else
         {
         query_error = TRUE;
         }
      PQclear (result);
      g_string_free (query, TRUE);
      }
   
   if (query_error)
      {
      gnome_dialog_close (GNOME_DIALOG (widget));
      }
   else
      {
      set_all_clean  (widget);
      }
}


static gboolean
validate (GtkWidget *dlg)
{
   GtkEntry  *name_entry;
   GString   *name_str;
   gboolean   rtn = TRUE;
   GtkWidget *msg_dlg;

   name_entry = GTK_ENTRY (get_widget (dlg, NAME_EDIT));
   name_str = g_string_new (gtk_entry_get_text (name_entry));
   name_str = g_string_strip (name_str);
   if (name_str->len == 0)
      {
      msg_dlg = gnome_message_box_new
         (_("Name canot be NULL."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      rtn = FALSE;
      }
   g_string_free (name_str, TRUE);

   return rtn;
}


static GString *
build_sql_string (GtkWidget *dlg)
{
   GString          *str = NULL;
   GString          *tmp_str;
   GString          *wv_str; /* the WHERE or VALUES clause... */
   GList            *def_sevr_list;
   gint              def_sevr_pk;
   gint              table_num;
   gboolean          is_update;
   gint               pk_num;
   GtkCombo          *def_sevr_combo = NULL;
   GtkEntry          *name_entry = NULL;
   GtkText           *descr_edit = NULL;
   GtkCheckButton    *active_cb = NULL;
   GtkCheckButton    *is_closed_cb = NULL;
   gboolean           first = TRUE;
   gchar             *char_buf;
   gchar              num_buf[NUM_BUFFER_LEN];

   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg), PK_NUM));
   table_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                     TABLE_NUM));
   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                      IS_UPDATE)) != 0);
   name_entry = GTK_ENTRY (get_widget (dlg, NAME_EDIT));
   g_assert (name_entry != NULL);
   descr_edit = GTK_TEXT (get_widget (dlg, DESCR_EDIT));
   g_assert (descr_edit != NULL);
   active_cb = GTK_CHECK_BUTTON (get_widget (dlg, ACTIVE_CHECK_BOX));
   g_assert (active_cb != NULL);

   if (is_update)
      {
      if (widgets_children_are_dirty (dlg))
         {
         str = g_string_new ("UPDATE ");
         wv_str = g_string_new (" WHERE ");
         switch (table_num)
            {
            case ED_STAT_TBL:
               str = g_string_append (str, " STATUS SET ");
               is_closed_cb = GTK_CHECK_BUTTON
                  (get_widget (dlg, IS_CLOSED_CHECK_BOX));
               if (widget_is_dirty (GTK_WIDGET (is_closed_cb)))
                  {
                  if (!first)
                     {
                     str = g_string_append (str, ", ");
                     }
                  first = FALSE;
                  str = g_string_append (str, "is_closed_status = ");
                  if (gtk_toggle_button_get_active
                        (GTK_TOGGLE_BUTTON (is_closed_cb)))
                     {
                     str = g_string_append (str, "TRUE");
                     }
                  else
                     {
                     str = g_string_append (str, "FALSE");
                     }
                  }
               wv_str = g_string_append (wv_str, "status_num = ");
               break;
            case ED_SEVR_TBL:
               str = g_string_append (str, " SEVERITY SET ");
               wv_str = g_string_append (wv_str, "severity_num = ");
               break;
            case ED_TYPE_TBL:
               str = g_string_append (str, " PROBLEM_TYPE SET ");
               def_sevr_combo = GTK_COMBO (get_widget (dlg, DEF_SEVR_COMBO));
               g_assert (def_sevr_combo != NULL);
               if (widget_is_dirty (GTK_WIDGET (def_sevr_combo->entry)))
                  {
                  def_sevr_list = gtk_object_get_data
                     (GTK_OBJECT (def_sevr_combo), DEF_SEVR_LIST);
                  def_sevr_pk = find_pk_from_name 
                     (def_sevr_list,
                      gtk_entry_get_text (GTK_ENTRY (def_sevr_combo->entry)));
                  g_assert (def_sevr_pk >= 0);
                  sprintf (num_buf, "%d", def_sevr_pk);
                  if (!first)
                     {
                     str = g_string_append (str, ", ");
                     }
                  first = FALSE;
                  str = g_string_append (str, "severity_num = ");
                  str = g_string_append (str, num_buf);
                  }
               wv_str = g_string_append (wv_str, "type_num = ");
               break;
            default:
               g_assert_not_reached ();
            }
         if (widget_is_dirty (GTK_WIDGET (name_entry)))
            {
            if (!first)
               {
               str = g_string_append (str, ", ");
               }
            first = FALSE;
            tmp_str = g_string_new (gtk_entry_get_text (name_entry));
            g_string_prepare_db_instr (tmp_str);
            str = g_string_append (str, "name = ");
            str = g_string_append (str, tmp_str->str);
            g_string_free (tmp_str, TRUE);
            }
         if (widget_is_dirty (GTK_WIDGET (descr_edit)))
            {
            if (!first)
               {
               str = g_string_append (str, ", ");
               }
            first = FALSE;
            char_buf = gtk_editable_get_chars
               (GTK_EDITABLE (descr_edit), 0, -1);
            tmp_str = g_string_new (char_buf);
            g_free (char_buf);
            g_string_prepare_db_instr (tmp_str);
            str = g_string_append (str, "descr = ");
            str = g_string_append (str, tmp_str->str);
            g_string_free (tmp_str, TRUE);
            }
         if (widget_is_dirty (GTK_WIDGET (active_cb)))
            {
            if (!first)
               {
               str = g_string_append (str, ", ");
               }
            first = FALSE;
            str = g_string_append (str, "active = ");
            if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (active_cb)))
               {
               str = g_string_append (str, "TRUE");
               }
            else
               {
               str = g_string_append (str, "FALSE");
               }
            }
         g_assert (first == FALSE);
         sprintf (num_buf, "%d", pk_num);
         wv_str = g_string_append (wv_str, num_buf);
         str = g_string_append (str, wv_str->str);
         g_string_free (wv_str, TRUE);
         } /* end if widgets dirty */
      } /* end is_update */
   else
      {
      str = g_string_new ("INSERT INTO ");
      wv_str = g_string_new (" VALUES (");
      switch (table_num)
         {
         case ED_SEVR_TBL:
            str = g_string_append (str, "SEVERITY (sevrity_num, ");
            wv_str = g_string_append (wv_str, "nextval ('severity_seq'), ");
            break;
         case ED_STAT_TBL:
            str = g_string_append
               (str, "STATUS (status_num, is_closed_status, ");
            wv_str = g_string_append (wv_str, "nextval ('status_seq'), ");
            is_closed_cb = GTK_CHECK_BUTTON
               (get_widget (dlg, IS_CLOSED_CHECK_BOX));
            g_assert (is_closed_cb != NULL);
            if (gtk_toggle_button_get_active
                  (GTK_TOGGLE_BUTTON (is_closed_cb)))
               {
               wv_str = g_string_append (wv_str, "TRUE, ");
               }
            else
               {
               wv_str = g_string_append (wv_str, "FALSE, ");
               }

            break;
         case ED_TYPE_TBL:
            str = g_string_append
               (str, "PROBLEM_TYPE (type_num, severity_num, ");
            wv_str = g_string_append (wv_str, "nextval ('problem_type_seq'), ");
            def_sevr_combo = GTK_COMBO (get_widget (dlg, DEF_SEVR_COMBO));
            g_assert (def_sevr_combo != NULL);
            def_sevr_list = gtk_object_get_data
               (GTK_OBJECT (def_sevr_combo), DEF_SEVR_LIST);
            def_sevr_pk = find_pk_from_name 
               (def_sevr_list,
                gtk_entry_get_text (GTK_ENTRY (def_sevr_combo->entry)));
            g_assert (def_sevr_pk >= 0);
            sprintf (num_buf, "%d", def_sevr_pk);
            wv_str = g_string_append (wv_str, num_buf);
            wv_str = g_string_append (wv_str, ", ");
            break;
         default:
            g_assert_not_reached ();
         }
      str = g_string_append (str, "active, name, descr)");
      /* active */
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (active_cb)))
         {
         wv_str = g_string_append (wv_str, "TRUE, ");
         }
      else
         {
         wv_str = g_string_append (wv_str, "FALSE, ");
         }
      /* name */
      tmp_str = g_string_new (gtk_entry_get_text (name_entry));
      g_string_prepare_db_instr (tmp_str);
      wv_str = g_string_append (wv_str, tmp_str->str);
      wv_str = g_string_append (wv_str, ", ");
      /* descr */
      char_buf = gtk_editable_get_chars (GTK_EDITABLE (descr_edit), 0, -1);
      tmp_str = g_string_assign (tmp_str, char_buf);
      g_free (char_buf);
      g_string_prepare_db_instr (tmp_str);
      wv_str = g_string_append (wv_str, tmp_str->str);
      wv_str = g_string_append (wv_str, ") ");
      str = g_string_append (str, wv_str->str);
      g_string_free (wv_str, TRUE);
      }

   return str;
}


static void
on_ok_butt_clicked (GnomeDialog *dlg)
{
   GString   *sql_str;
   PGconn    *conn;
   PGresult  *res;
   gint       table_num;
   GtkWidget *parent;


   if (validate (GTK_WIDGET (dlg)))
      {
      sql_str = build_sql_string (GTK_WIDGET (dlg));
      if (sql_str == NULL)
         {
         gnome_dialog_close (dlg);
         }
      else
         {
         conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
         g_assert (conn != NULL);
         res = PQexec (conn, sql_str->str);
         g_string_free (sql_str, TRUE);
         if (chk_sql_error (res, "attribute SQL"))
            {
            parent = get_widget (GTK_WIDGET (dlg), MY_PARENT);
            g_assert (parent != NULL);
            table_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                              TABLE_NUM));
            switch (table_num)
               {
               case ED_SEVR_TBL:
                  draw_sevr_list (GTK_WINDOW (parent), conn);
                  break;
               case ED_STAT_TBL:
                  draw_stat_list (GTK_WINDOW (parent), conn);
                  break;
               case ED_TYPE_TBL:
                  draw_type_list (GTK_WINDOW (parent), conn);
                  break;
               default:
                  g_assert_not_reached ();
               }
            gnome_dialog_close (dlg);
            }
         PQclear (res);
         }
      }
}


static void 
on_can_butt_clicked (GnomeDialog *dlg)
{
   gnome_dialog_close (dlg);
}


static void 
on_edit_dlg_destroy (GtkObject  *object, gpointer  user_data)
{
   destroy_dirty_list (GTK_WIDGET (object));
}


static void
on_edit_dlg_clicked (GnomeDialog    *dlg,
                     gint            btn_cd,
                     gpointer        user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_ok_butt_clicked (dlg);
         break;
      case 1:
         on_can_butt_clicked (dlg);
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


/*
 *  create_edit_dlg
 *  
 *  Create the edit dialog used for statuses and severities.
 */
static GtkWidget* 
create_edit_dlg (gboolean    is_update,
                 long        pk_num,
                 int         table,
                 PGconn     *conn, 
                 GtkWidget  *parent,
                 gboolean    user_is_admin)
{
  GtkWidget *edit_dlg;
  GtkWidget *dialog_vbox2;
  GtkWidget *vbox9;
  GtkWidget *ed_name_lbl;
  GtkWidget *ed_name_edit;
  GtkWidget *ed_descr_lbl;
  GtkWidget *scrolledwindow1;
  GtkWidget *ed_descr_edit;
  GtkWidget *hbox12;
  GtkWidget *ed_active_but;
  GtkWidget *ed_is_closed_stat_but;
  GtkWidget *popup;
  GString   *title;

  title = build_edit_dlg_title (is_update, table);
  edit_dlg = gnome_dialog_new (title->str,
                               GNOME_STOCK_BUTTON_OK,
                               GNOME_STOCK_BUTTON_CANCEL,
                               NULL);
  g_string_free (title, TRUE);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "edit_dlg", edit_dlg);
  gtk_container_border_width (GTK_CONTAINER (edit_dlg), 3);
  gtk_signal_connect (GTK_OBJECT (edit_dlg), "show",
                      GTK_SIGNAL_FUNC (on_edit_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (edit_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_edit_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (edit_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_edit_dlg_clicked),
                      NULL);
  gnome_dialog_set_close (GNOME_DIALOG (edit_dlg), FALSE);
  gnome_dialog_set_parent (GNOME_DIALOG (edit_dlg), GTK_WINDOW (parent));
  gtk_window_set_policy (GTK_WINDOW (edit_dlg), FALSE, FALSE, TRUE);
  if (user_is_admin)
     {
     gnome_dialog_set_default (GNOME_DIALOG (edit_dlg), 0);
     }
  else
     {
     gnome_dialog_set_default (GNOME_DIALOG (edit_dlg), 1);
     gnome_dialog_set_sensitive (GNOME_DIALOG (edit_dlg), 0, FALSE); 
     }

  gtk_object_set_data (GTK_OBJECT (edit_dlg), IS_UPDATE, 
                       GINT_TO_POINTER (is_update ? 1 : 0));
  gtk_object_set_data (GTK_OBJECT (edit_dlg), PK_NUM,
                       GINT_TO_POINTER (pk_num));
  gtk_object_set_data (GTK_OBJECT (edit_dlg), TABLE_NUM,
                       GINT_TO_POINTER (table));
  gtk_object_set_data (GTK_OBJECT (edit_dlg), TOP_DOG, edit_dlg);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), MY_DB_CONN, conn);

  dialog_vbox2 = GNOME_DIALOG (edit_dlg)->vbox;
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "dialog_vbox2", dialog_vbox2);
  gtk_box_set_spacing (GTK_BOX (dialog_vbox2), 5);
  gtk_widget_show (dialog_vbox2);

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "vbox9", vbox9);
  gtk_widget_show (vbox9);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), vbox9, TRUE, TRUE, 0);

  ed_name_lbl = gtk_label_new (_("Name:"));
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "ed_name_lbl", ed_name_lbl);
  gtk_widget_show (ed_name_lbl);
  gtk_box_pack_start (GTK_BOX (vbox9), ed_name_lbl, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (ed_name_lbl), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (ed_name_lbl), 0, 0.5);

  ed_name_edit = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (edit_dlg), NAME_EDIT, ed_name_edit);
  gtk_widget_show (ed_name_edit);
  gtk_box_pack_start (GTK_BOX (vbox9), ed_name_edit, FALSE, TRUE, 0);
  gtk_widget_set_sensitive (ed_name_edit, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (ed_name_edit), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (edit_dlg, ed_name_edit);
  popup = attach_generic_edit_popup_menu (ed_name_edit);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (edit_dlg), "popup1", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  ed_descr_lbl = gtk_label_new (_("Description:"));
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "ed_descr_lbl", ed_descr_lbl);
  gtk_widget_show (ed_descr_lbl);
  gtk_box_pack_start (GTK_BOX (vbox9), ed_descr_lbl, FALSE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (ed_descr_lbl), 0, 0.5);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "scrolledwindow1", 
                       scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox9), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  ed_descr_edit = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), 
                       DESCR_EDIT, 
                       ed_descr_edit);
  gtk_widget_show (ed_descr_edit);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), ed_descr_edit);
  gtk_widget_set_usize (ed_descr_edit, 300, -2);
  gtk_text_set_editable (GTK_TEXT (ed_descr_edit), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (ed_descr_edit), TRUE);
  gtk_widget_set_sensitive (ed_descr_edit, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (ed_descr_edit), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (edit_dlg, ed_descr_edit);
  popup = attach_generic_edit_popup_menu (ed_descr_edit);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (edit_dlg), "popup2", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (edit_dlg), "hbox12", hbox12);
  gtk_widget_show (hbox12);
  gtk_box_pack_start (GTK_BOX (vbox9), hbox12, FALSE, TRUE, 3);

  ed_active_but = gtk_check_button_new_with_label (_("Active"));
  gtk_object_set_data (GTK_OBJECT (edit_dlg), ACTIVE_CHECK_BOX,
                       ed_active_but);
  gtk_widget_show (ed_active_but);
  gtk_box_pack_start (GTK_BOX (hbox12), ed_active_but, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (ed_active_but, user_is_admin);
  gtk_signal_connect (GTK_OBJECT (ed_active_but), "toggled",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (edit_dlg, ed_active_but);


  if (table == ED_STAT_TBL)
     {
     ed_is_closed_stat_but = gtk_check_button_new_with_label 
        (_("Is Closed Status"));
     gtk_widget_ref (ed_is_closed_stat_but);
     gtk_object_set_data_full 
        (GTK_OBJECT (edit_dlg), IS_CLOSED_CHECK_BOX,
         ed_is_closed_stat_but, (GtkDestroyNotify) gtk_widget_unref); 
     gtk_widget_show (ed_is_closed_stat_but);
     gtk_box_pack_start (GTK_BOX (hbox12), ed_is_closed_stat_but,
                         TRUE, TRUE, 0);
     gtk_widget_set_sensitive (ed_is_closed_stat_but, user_is_admin);
     gtk_signal_connect (GTK_OBJECT (ed_is_closed_stat_but), "toggled",
                         GTK_SIGNAL_FUNC (set_dirty), NULL);
     add_widget_to_dirty_list (edit_dlg, ed_is_closed_stat_but);
     }

  return edit_dlg;
}


/*------------------------------------------------------------------------
 * Type Edit dialog box signal handlers.
 *------------------------------------------------------------------------
 */
static void 
on_type_edit_dlg_show (GtkWidget  *widget, gpointer  user_data)
{
   GString     *query;
   gboolean     query_error = FALSE;
   PGresult    *result;
   GtkWidget   *child;
   GList       *def_sevr_list;
   gboolean     is_update;
   gint         pk_num;
   PGconn      *conn;

   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                      IS_UPDATE)) == 1);
   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                  PK_NUM));
   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   
   if (is_update)
      {
      query = g_string_new ("");
      g_string_sprintf (query, TYPE_QUERY, pk_num);
      result = PQexec (conn, query->str);
      if (chk_sql_error (result, "getting type data to edit"))
         {
         child = get_widget (widget, NAME_EDIT);
         g_assert (child != NULL);
         gtk_entry_set_text (GTK_ENTRY (child),
               PQgetvalue (result, 0, NAME_POS));

         child = get_widget (widget, DESCR_EDIT);
         g_assert (child != NULL);
         gtk_text_insert (GTK_TEXT (child), NULL, NULL, NULL,
               PQgetvalue (result, 0, DESCR_POS), -1);
         
         child = get_widget (widget, DEF_SEVR_COMBO);
         g_assert (child != NULL);
         def_sevr_list = gtk_object_get_data (GTK_OBJECT (child),
                                              DEF_SEVR_LIST);
         gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (child)->entry), 
               find_name_from_pk (def_sevr_list,
                                  atoi (PQgetvalue (result, 0, SEVR_POS)))); 

         child = get_widget (widget, ACTIVE_CHECK_BOX);
         g_assert (child != NULL);
         if ((PQgetvalue (result, 0, ACTIVE_POS))[0] == 't')
            {
            gtk_toggle_button_set_active
               (GTK_TOGGLE_BUTTON (child), TRUE);
            }
         else
            {
            gtk_toggle_button_set_active
               (GTK_TOGGLE_BUTTON (child), FALSE);
            }
         }
      else
         {
         query_error = TRUE;
         }
      g_string_free (query, TRUE);
      PQclear (result);
      }
   
   if (query_error)
      {
      gnome_dialog_close (GNOME_DIALOG (widget));
      }
   else
      {
      set_all_clean (widget);
      }
}

static void 
on_type_edit_dlg_destroy (GtkObject *object, gpointer user_data)
{
   GList    *temp_list;
   GtkCombo *combo;
   
   combo = GTK_COMBO (get_widget (GTK_WIDGET (object), DEF_SEVR_COMBO));
   temp_list = gtk_object_get_data (GTK_OBJECT (combo), DEF_SEVR_LIST);
   g_assert (combo != NULL);
   g_assert (temp_list != NULL);
   /*
    *  Before we free the list itself, we need to be sure to go
    *  through and free all of the stuff in the list.
    */
   while (temp_list)
      {
      g_free (((name_pk_struct *)temp_list->data)->name);
      g_free (temp_list->data);
      temp_list = temp_list->next;
      }
   g_list_free (temp_list);
   destroy_dirty_list (GTK_WIDGET (object));
}


static void
on_type_ed_dlg_clicked (GnomeDialog    *dlg,
                        gint            btn_cd,
                        gpointer        user_data)
{
   switch (btn_cd)
      {
      case 0:
         /*on_type_ed_ok_butt_clicked (dlg);*/
         on_ok_butt_clicked (dlg);
         break;
      case 1:
         on_can_butt_clicked (dlg);
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


/*
 *  create_type_edit_dlg
 *  
 *  Create the edit dialog used for problem types.
 */
static GtkWidget *
create_type_edit_dlg (gboolean   is_update,
                      long       pk_num,
                      PGconn    *conn,
                      GtkWidget *parent,
                      gboolean   user_is_admin)
{
   GtkWidget          *type_edit_dlg;
   GtkWidget          *dialog_vbox3;
   GtkWidget          *vbox11;
   GtkWidget          *label4;
   GtkWidget          *type_ed_name_edit;
   GtkWidget          *label5;
   GtkWidget          *scrolledwindow2;
   GtkWidget          *type_ed_descr_ed;
   GtkWidget          *hbox14;
   GtkWidget          *vbox12;
   GtkWidget          *label6;
   GtkWidget          *type_ed_def_sevr;
   GtkWidget          *type_ed_active_butt;
   GList              *name_list = NULL;
   GList              *full_list = NULL;
   name_pk_struct     *curr_sevr;
   gint                i;
   gint                n;
   PGresult           *result;
   GString            *title;
   GtkWidget          *popup;

   title = build_edit_dlg_title (is_update, ED_TYPE_TBL);
   type_edit_dlg = gnome_dialog_new (title->str,
                                     GNOME_STOCK_BUTTON_OK,
                                     GNOME_STOCK_BUTTON_CANCEL,
                                     NULL);
   g_string_free (title, TRUE);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), 
                        "type_edit_dlg", 
                        type_edit_dlg);
   gtk_container_border_width (GTK_CONTAINER (type_edit_dlg), 3);
   gtk_signal_connect (GTK_OBJECT (type_edit_dlg), "show",
                       GTK_SIGNAL_FUNC (on_type_edit_dlg_show),
                       NULL);
   gtk_signal_connect (GTK_OBJECT (type_edit_dlg), "destroy",
                       GTK_SIGNAL_FUNC (on_type_edit_dlg_destroy),
                       NULL);
   gtk_signal_connect (GTK_OBJECT (type_edit_dlg), "clicked",
                       GTK_SIGNAL_FUNC (on_type_ed_dlg_clicked),
                       NULL);
   gnome_dialog_set_parent (GNOME_DIALOG (type_edit_dlg),
                            GTK_WINDOW (parent));
   gnome_dialog_set_close (GNOME_DIALOG (type_edit_dlg), FALSE);
   gtk_window_set_policy (GTK_WINDOW (type_edit_dlg), FALSE, FALSE, TRUE);
   if (user_is_admin)
      {
      gnome_dialog_set_default (GNOME_DIALOG (type_edit_dlg), 0);
      }
   else
      {
      gnome_dialog_set_default (GNOME_DIALOG (type_edit_dlg), 1);
      gnome_dialog_set_sensitive (GNOME_DIALOG (type_edit_dlg), 0, FALSE); 
      }
            
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), IS_UPDATE, 
                        GINT_TO_POINTER (is_update ? 1 : 0));
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), PK_NUM,
                        GINT_TO_POINTER (pk_num));
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), TOP_DOG, type_edit_dlg);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), MY_PARENT, parent);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), MY_DB_CONN, conn);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), TABLE_NUM,
                        GINT_TO_POINTER (ED_TYPE_TBL));

   dialog_vbox3 = GNOME_DIALOG (type_edit_dlg)->vbox;
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), 
                        "dialog_vbox3", dialog_vbox3);
   gtk_box_set_spacing (GTK_BOX (dialog_vbox3), 5);
   gtk_widget_show (dialog_vbox3);
                  
   vbox11 = gtk_vbox_new (FALSE, 0);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), "vbox11", vbox11);
   gtk_widget_show (vbox11);
   gtk_box_pack_start (GTK_BOX (dialog_vbox3), vbox11, TRUE, TRUE, 0);
                          
   label4 = gtk_label_new (_("Name:"));
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), "label4", label4);
   gtk_widget_show (label4);
   gtk_box_pack_start (GTK_BOX (vbox11), label4, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);
        
   type_ed_name_edit = gtk_entry_new ();
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), 
                        NAME_EDIT,
                        type_ed_name_edit);
   gtk_widget_show (type_ed_name_edit);
   gtk_box_pack_start (GTK_BOX (vbox11), type_ed_name_edit, TRUE, TRUE, 0);
   gtk_widget_set_sensitive (type_ed_name_edit, user_is_admin);
   gtk_signal_connect (GTK_OBJECT (type_ed_name_edit), "changed",
                       GTK_SIGNAL_FUNC (set_dirty), NULL);
   add_widget_to_dirty_list (type_edit_dlg, type_ed_name_edit);
   popup = attach_generic_edit_popup_menu (type_ed_name_edit);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (type_edit_dlg), "popup1", popup,
                             (GtkDestroyNotify) gtk_widget_unref);
                
   label5 = gtk_label_new (_("Description:"));
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), "label5", label5);
   gtk_widget_show (label5);
   gtk_box_pack_start (GTK_BOX (vbox11), label5, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);
                         
   scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), 
                        "scrolledwindow2", 
                        scrolledwindow2);
   gtk_widget_show (scrolledwindow2);
   gtk_box_pack_start (GTK_BOX (vbox11), scrolledwindow2, TRUE, TRUE, 0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), 
                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   
   type_ed_descr_ed = gtk_text_new (NULL, NULL);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), 
                        DESCR_EDIT,
                        type_ed_descr_ed);
   gtk_widget_show (type_ed_descr_ed);
   gtk_container_add (GTK_CONTAINER (scrolledwindow2), type_ed_descr_ed);
   gtk_widget_set_usize (type_ed_descr_ed, 300, -2);
   gtk_text_set_editable (GTK_TEXT (type_ed_descr_ed), TRUE);
   gtk_text_set_word_wrap (GTK_TEXT (type_ed_descr_ed), TRUE);
   gtk_widget_set_sensitive (type_ed_descr_ed, user_is_admin);
   gtk_signal_connect (GTK_OBJECT (type_ed_descr_ed), "changed",
                       GTK_SIGNAL_FUNC (set_dirty), NULL);
   add_widget_to_dirty_list (type_edit_dlg, type_ed_descr_ed);
   popup = attach_generic_edit_popup_menu (type_ed_descr_ed);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (type_edit_dlg), "popup2", popup,
                             (GtkDestroyNotify) gtk_widget_unref);
               
   hbox14 = gtk_hbox_new (FALSE, 0);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), "hbox14", hbox14);
   gtk_widget_show (hbox14);
   gtk_box_pack_start (GTK_BOX (vbox11), hbox14, TRUE, TRUE, 0);
                       
   vbox12 = gtk_vbox_new (FALSE, 0);
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), "vbox12", vbox12);
   gtk_widget_show (vbox12);
   gtk_box_pack_start (GTK_BOX (hbox14), vbox12, TRUE, TRUE, 0);
                               
   label6 = gtk_label_new (_("Default Severity:"));
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), "label6", label6);
   gtk_widget_show (label6);
   gtk_box_pack_start (GTK_BOX (vbox12), label6, TRUE, TRUE, 0);
   gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);
    
   type_ed_def_sevr = gtk_combo_new ();
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg),
                        DEF_SEVR_COMBO,
                        type_ed_def_sevr);
   /* Store the severities and the PK's with the combo box */
   result = PQexec (conn, SEVRS_QUERY);
   if (chk_sql_error (result, "getting severities"))
      {
      n = PQntuples (result);
      for (i = 0; i < n; i++)
         {
         curr_sevr = (name_pk_struct *)g_malloc (sizeof (name_pk_struct));
         curr_sevr->pk = atoi (PQgetvalue (result, i, PK_POS));
         curr_sevr->name  = (gchar *)g_malloc (strlen 
               (PQgetvalue (result, i, NAME_POS)));
         strcpy (curr_sevr->name, PQgetvalue (result, i, NAME_POS));
         name_list = g_list_append (name_list, curr_sevr->name);
         full_list = g_list_append (full_list, curr_sevr);
         }
      gtk_combo_set_popdown_strings (GTK_COMBO (type_ed_def_sevr),
                                     name_list);
      gtk_object_set_data (GTK_OBJECT (type_ed_def_sevr),
                           DEF_SEVR_LIST,
                           full_list);
      g_list_free (name_list);
      }
   PQclear (result);
   gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (type_ed_def_sevr)->entry),
                           FALSE);
   gtk_widget_show (type_ed_def_sevr);
   gtk_box_pack_start (GTK_BOX (vbox12), type_ed_def_sevr, TRUE, TRUE, 3);
   gtk_widget_set_sensitive (type_ed_def_sevr, user_is_admin);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (type_ed_def_sevr)->entry),
                       "changed", GTK_SIGNAL_FUNC (set_dirty), NULL);
   add_widget_to_dirty_list (type_edit_dlg,
                             GTK_COMBO (type_ed_def_sevr)->entry);
                
   type_ed_active_butt = gtk_check_button_new_with_label (_("Active"));
   gtk_object_set_data (GTK_OBJECT (type_edit_dlg), 
                        ACTIVE_CHECK_BOX,
                        type_ed_active_butt);
   gtk_widget_show (type_ed_active_butt);
   gtk_box_pack_start (GTK_BOX (hbox14), type_ed_active_butt, TRUE, TRUE, 0);
   gtk_container_border_width (GTK_CONTAINER (type_ed_active_butt), 5);
   gtk_widget_set_sensitive (type_ed_active_butt, user_is_admin);
   gtk_signal_connect (GTK_OBJECT (type_ed_active_butt), "toggled",
                       GTK_SIGNAL_FUNC (set_dirty), NULL);
   add_widget_to_dirty_list (type_edit_dlg, type_ed_active_butt);
                          
   return type_edit_dlg;
}

                                          
static void
raise_or_create_edit_dlg (GtkWidget  *parent,
                          gboolean    is_update,
                          long        pk_num,
                          int         table,
                          PGconn     *conn,
                          gboolean    user_is_admin)
{
   GtkWidget  *dlg;
   GList      *child_list;
   GList      *list_iter;

   /*
    * Go through the existing list of children first.  If we have a
    * child that already uses this item's PK and is editing the same
    * type of object, just bring that one to the top.
    */
   child_list = gtk_object_get_data (GTK_OBJECT (parent), my_child_list);

   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      if (table == GPOINTER_TO_INT
                      (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                            TABLE_NUM)))
         {
         if (pk_num == GPOINTER_TO_INT
                          (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                                PK_NUM)))
            {
            gdk_window_show (GTK_WIDGET (list_iter->data)->window);
            gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

            return;
            }
         }
      list_iter = list_iter->next;
      }

   /*
    * Didn't find a child alredy displaying this object, so we need to
    * create one.
    */
   if (table == ED_TYPE_TBL)
      {
      dlg = create_type_edit_dlg (is_update, pk_num, conn,
                                  parent, user_is_admin);
      }
   else
      {
      dlg = create_edit_dlg (is_update, pk_num, table,
                             conn, parent, user_is_admin);
      }
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * list when it dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);

   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (parent), my_child_list, child_list);
}


/*------------------------------------------------------------------------
 * attributes dialog signal handlers
 * 
 * Signal handler functions for the attributes dialog box.
 *------------------------------------------------------------------------
 */
static void 
on_row_move (GtkCList *clist,
             gint      start,
             gint      end,
             gpointer  user_data)
{
   PGconn        *conn;
   PGresult      *res;
   GString       *sql_buffer;
   GString       *sql_template;
   pk_order_num  *pk_on;
   gint           i, n;
   gboolean       in_error = FALSE;

   /*
    * If we are not an admin user, just destroy the window and 
    * return.  If they were able to move anything, they aren't
    * allowed to modify the tables in question, so attempting to
    * do so would be bad...
    */
   if (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data), ADMIN_USER)) != 1)
      {
      return;
      }
   /*
    * Determine which clist something is being moved in, then build the
    * update statement for that list.
    */
   if (clist == GTK_CLIST (get_widget (GTK_WIDGET (user_data), SEVR_CLIST)))
      {
      sql_template = g_string_new
         ("UPDATE severity SET order_num = %d WHERE severity_num = %d");
      }
   else
   if (clist == GTK_CLIST (get_widget (GTK_WIDGET (user_data), STAT_CLIST)))
      {
      sql_template = g_string_new
         ("UPDATE status SET order_num = %d WHERE status_num = %d");
      }
   else
   if (clist == GTK_CLIST (get_widget (GTK_WIDGET (user_data), TYPE_CLIST)))
      {
      sql_template = g_string_new
         ("UPDATE problem_type SET order_num = %d WHERE type_num = %d");
      }
   else
      {
      g_assert_not_reached ();
      return;
      }

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   sql_buffer = g_string_new ("");

   START_TRANSACTION (conn);
   n = clist->rows;
   for (i = 0; i < n; i++)
      {
      pk_on = gtk_clist_get_row_data (clist, i);
      g_assert (pk_on != NULL);
      if (pk_on->order_num != i)
         {
         g_string_sprintf
            (sql_buffer, sql_template->str, i, pk_on->pk);
         res = PQexec (conn, sql_buffer->str);
         if (!chk_sql_error (res, "Updating order_num"))
            {
            PQclear (res);
            in_error = TRUE;
            break;
            }
         PQclear (res);
         }
      }

   if (in_error)
      {
      ROLLBACK (conn);
      }
   else
      {
      COMMIT (conn);
      }

   g_string_free (sql_buffer, TRUE);
   g_string_free (sql_template, TRUE);

   return;
}


static void 
on_attributes_dlg_show (GtkWidget  *widget, gpointer  user_data)
{
   gboolean         in_error = TRUE;  /* Set FALSE if we do not encounter 
                                         an error at any step. */
   PGconn          *conn = (PGconn *)user_data;
   GtkToggleButton *cb;
   GtkNotebook     *notebk;
   GString         *config_str;
   gboolean         is_def;
   gint             page;
   gboolean         flag;

   config_str = g_string_new ("");

   notebk = GTK_NOTEBOOK (get_widget (widget, NOTEBOOK));
   g_assert (notebk != NULL);
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, ACTIVE_TAB, "0");
   page = gnome_config_get_int_with_default (config_str->str, &is_def);
   gtk_notebook_set_page (notebk, page);

   cb = GTK_TOGGLE_BUTTON (get_widget (widget, SEVR_SHOW_INACTIVE));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, SEVR_SH_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   gtk_toggle_button_set_active (cb, flag);

   cb = GTK_TOGGLE_BUTTON (get_widget (widget, STAT_SHOW_INACTIVE));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, STAT_SH_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   gtk_toggle_button_set_active (cb, flag);

   cb = GTK_TOGGLE_BUTTON (get_widget (widget, TYPE_SHOW_INACTIVE));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, TYPE_SH_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   gtk_toggle_button_set_active (cb, flag);

   g_string_free (config_str, TRUE);
   
   if (draw_sevr_list (GTK_WINDOW (widget), conn))
      {
      if (draw_stat_list (GTK_WINDOW (widget), conn))
         {
         if (draw_type_list (GTK_WINDOW (widget), conn))
            {
            in_error = FALSE;
            }
         }
      }

   if (in_error)
      {
      gtk_widget_destroy (GTK_WIDGET (widget));
      }
   
      
   return;
}

 
static void 
on_attributes_dlg_destroy (GtkObject *object, gpointer   user_data)
{
   GList           *child_list;
   GList           *list_iter;
   GtkNotebook     *notebook;
   GtkToggleButton *cb;
   GString         *config_str;

   /*
    * Save the state of the dialog for the next time we open it.
    */
   config_str = g_string_new ("");
   notebook = GTK_NOTEBOOK (get_widget (GTK_WIDGET (object), NOTEBOOK));
   g_assert (notebook != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, ACTIVE_TAB);
   gnome_config_set_int (config_str->str, 
                         gtk_notebook_get_current_page (notebook));

   cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (object),
                                       SEVR_SHOW_INACTIVE));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, SEVR_SH_INACT);
   gnome_config_set_bool (config_str->str,
                          gtk_toggle_button_get_active (cb));

   cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (object),
                                       STAT_SHOW_INACTIVE));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, STAT_SH_INACT);
   gnome_config_set_bool (config_str->str,
                          gtk_toggle_button_get_active (cb));

   cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (object),
                                       TYPE_SHOW_INACTIVE));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     ATTR_DLG_SEC, TYPE_SH_INACT);
   gnome_config_set_bool (config_str->str,
                          gtk_toggle_button_get_active (cb));

   g_string_sprintf (config_str, CONF_FILE, PACKAGE, DLG_INI_FILE);
   gnome_config_sync_file (config_str->str);
   g_string_free (config_str, TRUE);
   

   /*
    * Kill all of the children
    */
   child_list = gtk_object_get_data (object, my_child_list);
   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                        child_dead,
                                        (gpointer)my_child_list);
      gtk_widget_destroy (GTK_WIDGET (list_iter->data));
      list_iter = list_iter->next;
      }
   g_list_free (child_list);
}


static void
on_attributes_dlg_clicked (GnomeDialog *dlg,
                           gint         btn,
                           gpointer     user_data)
{
   /*
    * It does not really matter what the button code is (there is
    * only one, the close button).  Just save the size.
    */
   save_dlg_size (GTK_WIDGET (dlg), ATTR_DLG_SEC, FALSE);
}



static void 
on_sevr_add (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *ed_dlg;
   PGconn    *conn;
   gboolean   admin_user;
   GList     *child_list;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);
   
   ed_dlg = create_edit_dlg (FALSE,
                             0,
                             ED_SEVR_TBL,
                             conn,
                             GTK_WIDGET (user_data),
                             admin_user);
   gtk_widget_show (ed_dlg);

   /*
    * Setup the child_dead() callback to remove the child from the 
    * child_list when it dies.
    */
   gtk_signal_connect (GTK_OBJECT (ed_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);
 
   child_list = gtk_object_get_data (GTK_OBJECT (user_data), my_child_list);
   child_list = g_list_append (child_list, ed_dlg);
   gtk_object_set_data (GTK_OBJECT (user_data), my_child_list, child_list);
}


static void 
on_sevr_mod (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   gint       row_idx; 
   gint       pk_num;
   PGconn    *conn;
   gboolean   admin_user;
   GtkWidget *msg_dlg; 

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), SEVR_CLIST));
   g_assert (lb_widget != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);
   
   if (lb_widget->selection)
      {
      row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
      pk_num = 
         ((pk_order_num *)gtk_clist_get_row_data (lb_widget, row_idx))->pk; 
      raise_or_create_edit_dlg (GTK_WIDGET (user_data),
                                TRUE,
                                pk_num,
                                ED_SEVR_TBL,
                                conn,
                                admin_user);
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To modify a severity, you must select a severity to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void
on_sevr_sel_row (GtkWidget      *widget,
                 gint            row,
                 gint            column,
                 GdkEventButton *event,
                 gpointer        user_data)
{
   GtkCList  *clist = GTK_CLIST (widget);
   gint       pk_num;
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            pk_num =
               ((pk_order_num *)gtk_clist_get_row_data (clist, row))->pk;
            raise_or_create_edit_dlg (GTK_WIDGET (user_data), TRUE,
                                      pk_num, ED_SEVR_TBL,
                                      conn, admin_user);
            break;

         default:
            break;
         }
      }
}


static void 
on_sevr_del (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   PGresult  *res;
   gint       row_idx; 
   gint       pk_num;
   PGconn    *conn;
   GtkWidget *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), SEVR_CLIST));
   g_assert (lb_widget != NULL);
   
   if (lb_widget->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("Deleting a severity is not reversable.\nAre you sure you want to continue?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0)
         {
         row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
         pk_num = GPOINTER_TO_INT 
            (gtk_clist_get_row_data (lb_widget, row_idx)); 
         START_TRANSACTION (conn);
         res = delete_severity (conn, pk_num);
         if (chk_sql_error (res, "Severity Delete Failed"))
            {
            COMMIT (conn);
            }
         else
            {
            ROLLBACK (conn);
            }
         draw_sevr_list (GTK_WINDOW (user_data), conn);
         PQclear (res);
         }
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To delete a severity, you must select a severity to delete."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void 
on_sevr_show_inactive_toggled (GtkToggleButton *togglebutton,
                               gpointer         user_data)
{
   PGconn    *conn;
   /*
    *  Just redraw the stuff in the box.   The draw routine will
    *  take care it the rest.
    */
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   draw_sevr_list (GTK_WINDOW (user_data), conn); 
}


static void 
on_stat_add (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *ed_dlg;
   PGconn    *conn;
   gboolean   admin_user;
   GList     *child_list;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   ed_dlg = create_edit_dlg (FALSE,
                             0,
                             ED_STAT_TBL,
                             conn,
                             GTK_WIDGET (user_data),
                             admin_user);
   gtk_widget_show (ed_dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when it dies.
    */
   gtk_signal_connect (GTK_OBJECT (ed_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);
 
   child_list = gtk_object_get_data (GTK_OBJECT (user_data), my_child_list);
   child_list = g_list_append (child_list, ed_dlg);
   gtk_object_set_data (GTK_OBJECT (user_data), my_child_list, child_list);
}


static void 
on_stat_mod (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   gint       row_idx; 
   gint       pk_num;
   PGconn    *conn;
   gboolean   admin_user;
   GtkWidget *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), STAT_CLIST));
   g_assert (lb_widget != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);
   
   if (lb_widget->selection)
      {
      row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
      pk_num =
         ((pk_order_num *)gtk_clist_get_row_data (lb_widget, row_idx))->pk; 
      raise_or_create_edit_dlg (GTK_WIDGET (user_data),
                                TRUE,
                                pk_num,
                                ED_STAT_TBL,
                                conn,
                                admin_user);
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To modify a status, you must select a status to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void
on_stat_sel_row (GtkWidget      *widget,
                 gint            row,
                 gint            column,
                 GdkEventButton *event,
                 gpointer        user_data)
{
   GtkCList  *clist = GTK_CLIST (widget);
   gint       pk_num;
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            pk_num = 
               ((pk_order_num *)gtk_clist_get_row_data (clist, row))->pk;
            raise_or_create_edit_dlg (GTK_WIDGET (user_data), TRUE,
                                      pk_num, ED_STAT_TBL,
                                      conn, admin_user);
            break;

         default:
            break;
         }
      }
}


static void 
on_stat_del (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   PGresult  *res;
   gint       row_idx;
   gint       pk_num;
   PGconn    *conn;
   GtkWidget *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), STAT_CLIST));
  g_assert (lb_widget != NULL);
   
   if (lb_widget->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("Deleting a status is not reversable.\nAre you sure you want to continue?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0)
         {
         row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
         pk_num = GPOINTER_TO_INT 
            (gtk_clist_get_row_data (lb_widget, row_idx)); 
         START_TRANSACTION (conn);
         res = delete_status (conn, pk_num);
         if (chk_sql_error (res, "Status Delete Failed"))
            {
            COMMIT (conn);
            }
         else
            {
            ROLLBACK (conn);
            }
         draw_stat_list (GTK_WINDOW (user_data), conn);
         PQclear (res);
         }
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To deleta a status, you must select a status to delete."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void 
on_stat_show_inactive_toggled (GtkToggleButton *togglebutton, 
                               gpointer         user_data)
{
   PGconn    *conn;
   /*
    *  Just redraw the stuff in the box.   The draw routine will
    *  take care it the rest.
    */
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   draw_stat_list (GTK_WINDOW (user_data), conn); 
}


static void 
on_type_add (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *ed_dlg;
   PGconn    *conn;
   gboolean   admin_user;
   GList     *child_list;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   ed_dlg = create_type_edit_dlg (FALSE,
                                  0,
                                  conn,
                                  GTK_WIDGET (user_data),
                                  admin_user);
   gtk_widget_show (ed_dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when it dies.
    */
   gtk_signal_connect (GTK_OBJECT (ed_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);
 
   child_list = gtk_object_get_data (GTK_OBJECT (user_data), my_child_list);
   child_list = g_list_append (child_list, ed_dlg);
   gtk_object_set_data (GTK_OBJECT (user_data), my_child_list, child_list);
}


static void 
on_type_mod (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   gint       row_idx;
   gint       pk_num;
   PGconn    *conn;
   gboolean   admin_user;
   GtkWidget *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), TYPE_CLIST));
   g_assert (lb_widget != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);
   
   if (lb_widget->selection)
      {
      row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
      pk_num = 
         ((pk_order_num *)gtk_clist_get_row_data (lb_widget, row_idx))->pk;
      raise_or_create_edit_dlg (GTK_WIDGET (user_data), 
                                TRUE,
                                pk_num,
                                ED_TYPE_TBL,
                                conn,
                                admin_user);
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To modify a type, you must select a type to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void
on_type_sel_row (GtkWidget      *widget,
                 gint            row,
                 gint            column,
                 GdkEventButton *event,
                 gpointer        user_data)
{
   GtkCList  *clist = GTK_CLIST (widget);
   gint       pk_num;
   PGconn    *conn;
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (user_data),
                               ADMIN_USER)) == 1);

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            pk_num = 
               ((pk_order_num *)gtk_clist_get_row_data (clist, row))->pk;
            raise_or_create_edit_dlg (GTK_WIDGET (user_data), TRUE,
                                      pk_num, ED_TYPE_TBL,
                                      conn, admin_user);
            break;

         default:
            break;
         }
      }
}



static void 
on_type_del (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   PGresult  *res;
   gint       row_idx;
   gint       pk_num;
   PGconn    *conn;
   GtkWidget *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (user_data), TYPE_CLIST));
   g_assert (lb_widget != NULL);
         
   if (lb_widget->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("Deleting a problm type is not reverable.\nAre you sure you want to continue?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0)
         {
         row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
         pk_num = GPOINTER_TO_INT 
            (gtk_clist_get_row_data (lb_widget, row_idx)); 
         START_TRANSACTION (conn);
         res = delete_type (conn, pk_num);
         if (chk_sql_error (res, "Type Delete Failed"))
            {
            COMMIT (conn);
            }
         else
            {
            ROLLBACK (conn);
            }
         draw_type_list (GTK_WINDOW (user_data), conn);
         PQclear (res);
         }
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To delete a type, you must select a type to delete."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void 
on_type_show_inactive_toggled (GtkToggleButton *togglebutton,
                               gpointer         user_data)
{
   PGconn    *conn;
   /*
    *  Just redraw the stuff in the box.   The draw routine will
    *  take care it the rest.
    */
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   draw_type_list (GTK_WINDOW (user_data), conn); 
}


static gboolean
on_button_press (GtkWidget      *widget,
                 GdkEventButton *event,
                 gpointer        user_data)
{
   gint         row;
   gint         col;
   gint         val;
   GtkWidget   *popup;
   GtkWidget   *main_win = GTK_WIDGET (user_data);
   gboolean     admin_user;
   GtkNotebook *nb;
   gint         page;

   static GnomeUIInfo popup_menu_sevr[] =
      {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Severity"),
                                 N_("Create a new severity"),
                                 on_sevr_add, NULL),
      GNOMEUIINFO_ITEM_NONE (N_("View/_Modify Severity"),
                             N_("View or modify the selected severity"),
                             on_sevr_mod),
      GNOMEUIINFO_ITEM_NONE (N_("Delete Status"),
                             N_("Delete the selected severity"),
                             on_sevr_del),
#if 0
      GNOMEUIINFO_TOGGLEITEM (N_("Severity Active"),
                              N_("Specify if the severity is active or not"),
                              on_sevr_toggled, NULL),
#endif
      GNOMEUIINFO_END
      };

   static GnomeUIInfo popup_menu_stat[] =
      {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Status"),
                                 N_("Create a new status"),
                                 on_stat_add, NULL),
      GNOMEUIINFO_ITEM_NONE (N_("View/_Modify Status"),
                             N_("View or modify the selected status"),
                             on_stat_mod),
      GNOMEUIINFO_ITEM_NONE (N_("Delete Status"),
                             N_("Delete the selected status"),
                             on_stat_del),
#if 0
      GNOMEUIINFO_TOGGLEITEM (N_("Status Active"),
                              N_("Specify if the status is active or not"),
                              on_stat_toggled, NULL),
#endif
      GNOMEUIINFO_END
      };

   static GnomeUIInfo popup_menu_type[] =
      {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Problem Type"),
                                 N_("Create a new problem type"),
                                 on_type_add, NULL),
      GNOMEUIINFO_ITEM_NONE (N_("View/_Modify Problem Type"),
                             N_("View or modify the selected problem type"),
                             on_type_mod),
      GNOMEUIINFO_ITEM_NONE (N_("Delete Problem Type"),
                             N_("Delete the selected problem type"),
                             on_type_del),
#if 0
      GNOMEUIINFO_TOGGLEITEM 
         (N_("Problem Type Active"),
          N_("Specify if the problem type is active or not"),
          on_type_toggled, NULL),
#endif
      GNOMEUIINFO_END
      };

   GnomeUIInfo *popup_menu = NULL;  /* The actual popup menu to use.  It
                                       will get set to one of the menus
                                       defined above based on the current
                                       page in the notebook. */


   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_win),
                                                       ADMIN_USER)) == 1);

   nb = GTK_NOTEBOOK (get_widget (main_win, NOTEBOOK));
   g_assert (nb != NULL);
   page = gtk_notebook_get_current_page (nb);
   switch (page)
      {
      case SEVR_PAGE:
         popup_menu = popup_menu_sevr;
         break;

      case STAT_PAGE:
         popup_menu = popup_menu_stat;
         break;
         
      case TYPE_PAGE:
         popup_menu = popup_menu_type;
         break;

      default:
         g_assert_not_reached();
         break;
      }

   if (event != NULL)
      {
      switch (event->button)
         {
         case 1:
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  val = gtk_clist_get_selection_info (GTK_CLIST (widget),
                                                      event->x,
                                                      event->y,
                                                      &row,
                                                      &col);
                  popup = gnome_popup_menu_new (popup_menu);
                  if (val)
                     {
                     gtk_clist_select_row (GTK_CLIST (widget), row, col);
                     }
                  else
                     {
                     gtk_widget_set_sensitive (popup_menu[1].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     if (GTK_CLIST (widget)->selection != NULL)
                        {
                        gtk_clist_unselect_row
                           (GTK_CLIST (widget),
                            GPOINTER_TO_INT
                            (GTK_CLIST (widget)->selection->data), 0);
                        }
                     }
                  if (!admin_user)
                     {
                     gtk_widget_set_sensitive (popup_menu[0].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     }
                  gnome_popup_menu_do_popup_modal (popup,
                                                   NULL, 
                                                   NULL,
                                                   event,
                                                   main_win);
                  gtk_widget_destroy (popup);
                  break;
               default:
                  break;
               }
            break;
         default:
            break;
         }
      }

   return FALSE;
}


static GtkWidget * 
create_attrib_dlg (GtkWidget *parent, PGconn *conn, gboolean user_is_admin)
{
  GtkWidget *attributes_dlg;
  GtkWidget *vbox3;
  GtkWidget *notebook1;
  GtkWidget *vbox4;
  GtkWidget *sevr_scroll;
  GtkWidget *sevr_list;
  GtkWidget *sev_name_lbl;
  GtkWidget *sev_active_lbl;
  GtkWidget *sev_descr_lbl;
  GtkWidget *hbox6;
  GtkWidget *sevr_add_butt;
  GtkWidget *sevr_mod_butt;
  GtkWidget *sevr_del_butt;
  GtkWidget *sevr_show_inactive;
  GtkWidget *vbox5;
  GtkWidget *stat_scroll;
  GtkWidget *stat_list;
  GtkWidget *stat_name_lbl;
  GtkWidget *stat_active_lbl;
  GtkWidget *stat_descr_lbl;
  GtkWidget *hbox4;
  GtkWidget *stat_add_butt;
  GtkWidget *stat_mod_butt;
  GtkWidget *stat_del_butt;
  GtkWidget *stat_show_inactive;
  GtkWidget *vbox7;
  GtkWidget *type_scroll;
  GtkWidget *type_list;
  GtkWidget *type_name_lbl;
  GtkWidget *type_def_sevr_lbl;
  GtkWidget *type_active_lbl;
  GtkWidget *type_descr_lbl;
  GtkWidget *hbox7;
  GtkWidget *type_add_butt;
  GtkWidget *type_mod_butt;
  GtkWidget *type_del_butt;
  GtkWidget *type_show_inactive;
  GtkWidget *sevr_label;
  GtkWidget *stat_label;
  GtkWidget *type_label;
  GtkTooltips *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  attributes_dlg = gnome_dialog_new (_("Problem Report Attributes"),
                                     GNOME_STOCK_BUTTON_CLOSE,
                                     NULL);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "attributes_dlg",
                       attributes_dlg);
  gtk_signal_connect (GTK_OBJECT (attributes_dlg), "show",
                      GTK_SIGNAL_FUNC (on_attributes_dlg_show),
                      conn);
  gtk_signal_connect (GTK_OBJECT (attributes_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_attributes_dlg_clicked),
                      NULL);
  gtk_window_set_policy (GTK_WINDOW (attributes_dlg), FALSE, TRUE, FALSE);
  gtk_signal_connect (GTK_OBJECT (attributes_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_attributes_dlg_destroy),
                      NULL);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), ADMIN_USER,
        GINT_TO_POINTER (user_is_admin ? 1 : 0));
  gnome_dialog_set_close (GNOME_DIALOG (attributes_dlg), TRUE);
  gnome_dialog_set_default (GNOME_DIALOG (attributes_dlg), 0);
  gnome_dialog_set_parent (GNOME_DIALOG (attributes_dlg),
                           GTK_WINDOW (parent));
  set_dlg_size (attributes_dlg, ATTR_DLG_SEC, 525, 300);

  vbox3 = GNOME_DIALOG (attributes_dlg)->vbox;
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "vbox3", vbox3);
  gtk_widget_show (vbox3);

  notebook1 = gtk_notebook_new ();
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), NOTEBOOK, notebook1);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox3), notebook1, TRUE, TRUE, 0);

  vbox4 = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "vbox4", vbox4);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox4);
  gtk_container_border_width (GTK_CONTAINER (vbox4), 5);

  sevr_scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg),
                       "sevr_scroll", sevr_scroll);
  gtk_widget_show (sevr_scroll);
  gtk_box_pack_start (GTK_BOX (vbox4), sevr_scroll, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sevr_scroll), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);

  sevr_list = gtk_clist_new (3);
  gtk_clist_set_reorderable (GTK_CLIST (sevr_list), user_is_admin);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), SEVR_CLIST, sevr_list);
  gtk_widget_show (sevr_list);
  gtk_container_add (GTK_CONTAINER (sevr_scroll), sevr_list);
  gtk_clist_set_column_width (GTK_CLIST (sevr_list), 0, 100);
  gtk_clist_set_column_width (GTK_CLIST (sevr_list), 1, 50);
  gtk_clist_set_column_width (GTK_CLIST (sevr_list), 2, 300);
  gtk_clist_column_titles_show (GTK_CLIST (sevr_list));
  gtk_signal_connect (GTK_OBJECT (sevr_list), "select-row",
                      GTK_SIGNAL_FUNC (on_sevr_sel_row),
                      attributes_dlg);
  gtk_signal_connect_after (GTK_OBJECT (sevr_list), "row-move",
                            GTK_SIGNAL_FUNC (on_row_move),
                            attributes_dlg);
  gtk_signal_connect (GTK_OBJECT (sevr_list), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press),
                      attributes_dlg);

  sev_name_lbl = gtk_label_new ("Name");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "sev_name_lbl", 
                       sev_name_lbl);
  gtk_widget_show (sev_name_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (sevr_list), 0, sev_name_lbl);
  gtk_widget_set_usize (sev_name_lbl, 32, -2);

  sev_active_lbl = gtk_label_new ("Active");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "sev_active_lbl", 
                       sev_active_lbl);
  gtk_widget_show (sev_active_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (sevr_list), 1, sev_active_lbl);
  gtk_widget_set_usize (sev_active_lbl, 36, -2);

  sev_descr_lbl = gtk_label_new ("Description");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "sev_descr_lbl", 
                       sev_descr_lbl);
  gtk_widget_show (sev_descr_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (sevr_list), 2, sev_descr_lbl);
  gtk_widget_set_usize (sev_descr_lbl, 100, -2);

  hbox6 = gtk_hbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "hbox6", hbox6);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox6, FALSE, FALSE, 0);

  sevr_add_butt = gtk_button_new_with_label ("Add");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "sevr_add_butt", 
                       sevr_add_butt);
  gtk_widget_show (sevr_add_butt);
  gtk_box_pack_start (GTK_BOX (hbox6), sevr_add_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, sevr_add_butt, 
                        "Add a severity to the list.", NULL);
  gtk_signal_connect (GTK_OBJECT (sevr_add_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_sevr_add),
                      attributes_dlg);
  gtk_widget_set_sensitive (sevr_add_butt, user_is_admin);

  sevr_mod_butt = gtk_button_new_with_label ("View/Modify");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "sevr_mod_butt", 
                       sevr_mod_butt);
  gtk_widget_show (sevr_mod_butt);
  gtk_box_pack_start (GTK_BOX (hbox6), sevr_mod_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, sevr_mod_butt, 
                        "Modify the currently selected severity.", NULL);
  gtk_signal_connect (GTK_OBJECT (sevr_mod_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_sevr_mod),
                      attributes_dlg);

  sevr_del_butt = gtk_button_new_with_label ("Delete");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg),
                       "sevr_del_butt",
                       sevr_del_butt);
  gtk_widget_show (sevr_del_butt);
  gtk_box_pack_start (GTK_BOX (hbox6), sevr_del_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, sevr_del_butt, "Delete the currently selected severity.  Note that this will only work if the severity is not used.  If the severity is used, you may deactivate it using the \"Modify\" button.", NULL);
  gtk_signal_connect (GTK_OBJECT (sevr_del_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_sevr_del),
                      attributes_dlg);
  gtk_widget_set_sensitive (sevr_del_butt, user_is_admin);

  sevr_show_inactive = gtk_check_button_new_with_label 
                          ("Show Inactive Severities");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       SEVR_SHOW_INACTIVE, 
                       sevr_show_inactive);
  gtk_widget_show (sevr_show_inactive);
  gtk_box_pack_start (GTK_BOX (hbox6), sevr_show_inactive, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, sevr_show_inactive,
                        "Toggle the display of severities that are not active",
                        NULL);
  gtk_signal_connect (GTK_OBJECT (sevr_show_inactive), "toggled",
                      GTK_SIGNAL_FUNC (on_sevr_show_inactive_toggled),
                      attributes_dlg);

  vbox5 = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "vbox5", vbox5);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox5);
  gtk_container_border_width (GTK_CONTAINER (vbox5), 5);

  stat_scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "stat_scroll", stat_scroll);
  gtk_widget_show (stat_scroll);
  gtk_box_pack_start (GTK_BOX (vbox5), stat_scroll, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (stat_scroll), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);

  stat_list = gtk_clist_new (3);
  gtk_clist_set_reorderable (GTK_CLIST (stat_list), user_is_admin);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), STAT_CLIST, stat_list);
  gtk_widget_show (stat_list);
  gtk_container_add (GTK_CONTAINER (stat_scroll), stat_list);
  gtk_clist_set_column_width (GTK_CLIST (stat_list), 0, 100);
  gtk_clist_set_column_width (GTK_CLIST (stat_list), 1, 50);
  gtk_clist_set_column_width (GTK_CLIST (stat_list), 2, 300);
  gtk_clist_column_titles_show (GTK_CLIST (stat_list));
  gtk_signal_connect (GTK_OBJECT (stat_list), "select_row",
                      GTK_SIGNAL_FUNC (on_stat_sel_row),
                      attributes_dlg);
  gtk_signal_connect_after (GTK_OBJECT (stat_list), "row-move",
                            GTK_SIGNAL_FUNC (on_row_move),
                            attributes_dlg);
  gtk_signal_connect (GTK_OBJECT (stat_list), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press),
                      attributes_dlg);

  stat_name_lbl = gtk_label_new ("Name");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "stat_name_lbl", 
                       stat_name_lbl);
  gtk_widget_show (stat_name_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (stat_list), 0, stat_name_lbl);

  stat_active_lbl = gtk_label_new ("Active");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "stat_active_lbl", 
                       stat_active_lbl);
  gtk_widget_show (stat_active_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (stat_list), 1, stat_active_lbl);

  stat_descr_lbl = gtk_label_new ("Description");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg),
                       "stat_descr_lbl", 
                       stat_descr_lbl);
  gtk_widget_show (stat_descr_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (stat_list), 2, stat_descr_lbl);

  hbox4 = gtk_hbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "hbox4", hbox4);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox4, FALSE, FALSE, 0);

  stat_add_butt = gtk_button_new_with_label ("Add");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "stat_add_butt", 
                       stat_add_butt);
  gtk_widget_show (stat_add_butt);
  gtk_box_pack_start (GTK_BOX (hbox4), stat_add_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        stat_add_butt, 
                        "Add a status to the list.", 
                        NULL);
  gtk_signal_connect (GTK_OBJECT (stat_add_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_stat_add),
                      attributes_dlg);
  gtk_widget_set_sensitive (stat_add_butt, user_is_admin);

  stat_mod_butt = gtk_button_new_with_label ("View/Modify");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "stat_mod_butt", 
                       stat_mod_butt);
  gtk_widget_show (stat_mod_butt);
  gtk_box_pack_start (GTK_BOX (hbox4), stat_mod_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        stat_mod_butt, 
                        "Modify the currently selected status.", 
                        NULL);
  gtk_signal_connect (GTK_OBJECT (stat_mod_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_stat_mod),
                      attributes_dlg);

  stat_del_butt = gtk_button_new_with_label ("Delete");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "stat_del_butt", 
                       stat_del_butt);
  gtk_widget_show (stat_del_butt);
  gtk_box_pack_start (GTK_BOX (hbox4), stat_del_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, stat_del_butt, "Delete the currently selected status.  Note that this will only work if the status is not used.  If the status is used, you may deactivate it using the \"Modify\" button.", NULL);
  gtk_signal_connect (GTK_OBJECT (stat_del_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_stat_del),
                      attributes_dlg);
  gtk_widget_set_sensitive (stat_del_butt, user_is_admin);

  stat_show_inactive = gtk_check_button_new_with_label 
                          ("Show Inactive Statuses");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       STAT_SHOW_INACTIVE, 
                       stat_show_inactive);
  gtk_widget_show (stat_show_inactive);
  gtk_box_pack_start (GTK_BOX (hbox4), stat_show_inactive, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (stat_show_inactive), "toggled",
                      GTK_SIGNAL_FUNC (on_stat_show_inactive_toggled),
                      attributes_dlg);

  vbox7 = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "vbox7", vbox7);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox7);
  gtk_container_border_width (GTK_CONTAINER (vbox7), 5);

  type_scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg),
                       "type_scroll", type_scroll);
  gtk_widget_show (type_scroll);
  gtk_box_pack_start (GTK_BOX (vbox7), type_scroll, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (type_scroll), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);

  type_list = gtk_clist_new (4);
  gtk_clist_set_reorderable (GTK_CLIST (type_list), user_is_admin);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), TYPE_CLIST, type_list);
  gtk_widget_show (type_list);
  gtk_container_add (GTK_CONTAINER (type_scroll), type_list);
  gtk_clist_set_column_width (GTK_CLIST (type_list), 0, 100);
  gtk_clist_set_column_width (GTK_CLIST (type_list), 1, 50);
  gtk_clist_set_column_width (GTK_CLIST (type_list), 2, 50);
  gtk_clist_set_column_width (GTK_CLIST (type_list), 3, 250);
  gtk_clist_column_titles_show (GTK_CLIST (type_list));
  gtk_signal_connect (GTK_OBJECT (type_list), "select_row",
                      GTK_SIGNAL_FUNC (on_type_sel_row),
                      attributes_dlg);
  gtk_signal_connect_after (GTK_OBJECT (type_list), "row-move",
                            GTK_SIGNAL_FUNC (on_row_move),
                            attributes_dlg);
  gtk_signal_connect (GTK_OBJECT (type_list), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press),
                      attributes_dlg);

  type_name_lbl = gtk_label_new ("Name");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_name_lbl", 
                       type_name_lbl);
  gtk_widget_show (type_name_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (type_list), 0, type_name_lbl);

  type_def_sevr_lbl = gtk_label_new ("Def Sevr");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_def_sevr_lbl", 
                       type_def_sevr_lbl);
  gtk_widget_show (type_def_sevr_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (type_list), 1, type_def_sevr_lbl);

  type_active_lbl = gtk_label_new ("Active");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_active_lbl", 
                       type_active_lbl);
  gtk_widget_show (type_active_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (type_list), 2, type_active_lbl);

  type_descr_lbl = gtk_label_new ("Description");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_descr_lbl", 
                       type_descr_lbl);
  gtk_widget_show (type_descr_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (type_list), 3, type_descr_lbl);

  hbox7 = gtk_hbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "hbox7", hbox7);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox7, FALSE, FALSE, 0);

  type_add_butt = gtk_button_new_with_label ("Add");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_add_butt", 
                       type_add_butt);
  gtk_widget_show (type_add_butt);
  gtk_box_pack_start (GTK_BOX (hbox7), type_add_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        type_add_butt, 
                        "Add a type to the list.", 
                        NULL);
  gtk_signal_connect (GTK_OBJECT (type_add_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_type_add),
                      attributes_dlg);
  gtk_widget_set_sensitive (type_add_butt, user_is_admin);

  type_mod_butt = gtk_button_new_with_label ("View/Modify");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_mod_butt", 
                       type_mod_butt);
  gtk_widget_show (type_mod_butt);
  gtk_box_pack_start (GTK_BOX (hbox7), type_mod_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        type_mod_butt, 
                        "Modify the currently selected type.", 
                        NULL);
  gtk_signal_connect (GTK_OBJECT (type_mod_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_type_mod),
                      attributes_dlg);

  type_del_butt = gtk_button_new_with_label ("Delete");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       "type_del_butt", 
                       type_del_butt);
  gtk_widget_show (type_del_butt);
  gtk_box_pack_start (GTK_BOX (hbox7), type_del_butt, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, type_del_butt, "Delete the currently selected type.  Note that this will only work if the type is not used.  If the type is used, you may deactivate it using the \"Modify\" button.", NULL);
  gtk_signal_connect (GTK_OBJECT (type_del_butt), "clicked",
                      GTK_SIGNAL_FUNC (on_type_del),
                      attributes_dlg);
  gtk_widget_set_sensitive (type_del_butt, user_is_admin);

  type_show_inactive = gtk_check_button_new_with_label ("Show Inactive Types");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), 
                       TYPE_SHOW_INACTIVE, 
                       type_show_inactive);
  gtk_widget_show (type_show_inactive);
  gtk_box_pack_start (GTK_BOX (hbox7), type_show_inactive, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (type_show_inactive), "toggled",
                      GTK_SIGNAL_FUNC (on_type_show_inactive_toggled),
                      attributes_dlg);

  sevr_label = gtk_label_new ("Severity");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "sevr_label", sevr_label);
  gtk_widget_show (sevr_label);
  set_notebook_tab (notebook1, 0, sevr_label);

  stat_label = gtk_label_new ("Status");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "stat_label", stat_label);
  gtk_widget_show (stat_label);
  set_notebook_tab (notebook1, 1, stat_label);

  type_label = gtk_label_new ("Type");
  gtk_object_set_data (GTK_OBJECT (attributes_dlg), "type_label", type_label);
  gtk_widget_show (type_label);
  set_notebook_tab (notebook1, 2, type_label);

  gtk_object_set_data (GTK_OBJECT (attributes_dlg), TOOLTIPS, tooltips);

  return attributes_dlg;
}


void
destroy_attrib_dlg (void)
{
   if (attrib_dlg != NULL)
      {
      gtk_widget_destroy (attrib_dlg);
      attrib_dlg = NULL;
      }
}


void
show_attrib_dlg (GtkWidget *parent, 
                 PGconn    *conn,
                 gboolean   user_is_admin)
{
   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (attrib_dlg != NULL)
      {
      gdk_window_show (attrib_dlg->window);
      gdk_window_raise (attrib_dlg->window);
      }
   else
      {
      attrib_dlg = create_attrib_dlg (parent, conn, user_is_admin);

      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(),
       * which will set the "user_data" (&attrib_dlg in this case) to
       * NULL.
       */
      gtk_signal_connect (GTK_OBJECT (attrib_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (gtk_widget_destroyed),
                          &attrib_dlg);
      gtk_widget_show (attrib_dlg);
      }
}


gboolean
attrib_dlg_dirty (void)
{
   if (attrib_dlg == NULL)
      {
      return FALSE;
      }

   return list_dlg_has_dirty_child (attrib_dlg, my_child_list);
}
