/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2001 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * gtkutils
 *
 * Synopsis:
 *   Some of these utility functions are copied from the code 
 *   created by Glade.  This code is being placed here so that 
 *   I can create UI using Glade, but then split the code out 
 *   (something Glade does not currently fully support).
 *   
 *   Other functions and structures are used for passing data
 *   around within a set of widgets.
 *
 * $Id: gtkutils.c,v 1.15 2001/12/29 17:57:43 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */

#include <assert.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gtkutils.h"

#define DIRTY_FLAG     "i_am_dirty" 
#define DIRTY_CHK_LIST "dirty_check_list"
#define DIRTY          1
#define CLEAN          0

static GList *pixmaps_directories = NULL;

void
gtkutils_cleanup (void)
{
   GList  *list_iter;

   for (list_iter = g_list_first (pixmaps_directories);
        list_iter != NULL;
        list_iter = list_iter->next)
      {
      g_free (list_iter->data);
      }

   g_list_free (pixmaps_directories);
   pixmaps_directories = NULL;
}


void
set_dirty (GtkWidget *widget, gpointer user_data)
{
   gtk_object_set_data (GTK_OBJECT (widget), DIRTY_FLAG,
                        GINT_TO_POINTER (DIRTY));
}


void
set_clean (GtkWidget *widget, gpointer user_data)
{
   gtk_object_set_data (GTK_OBJECT (widget), DIRTY_FLAG,
                        GINT_TO_POINTER (CLEAN));
}


void
set_all_clean (GtkWidget *parent)
{
   GList    *child_list;
   GList    *list_iter;

   child_list = gtk_object_get_data (GTK_OBJECT (parent), DIRTY_CHK_LIST);
   list_iter = g_list_first (child_list);
   while (list_iter != NULL)
      {
      set_clean (GTK_WIDGET (list_iter->data), NULL);
      list_iter = list_iter->next;
      }
}


void 
add_widget_to_dirty_list (GtkWidget *parent, GtkWidget *widget)
{
   GList  *dirty_list;

   dirty_list = gtk_object_get_data (GTK_OBJECT (parent), DIRTY_CHK_LIST);
   dirty_list = g_list_append (dirty_list, widget);
   gtk_object_set_data (GTK_OBJECT (parent), DIRTY_CHK_LIST, dirty_list);
}


void
destroy_dirty_list (GtkWidget *parent)
{
   GList *dirty_list;

   dirty_list = gtk_object_get_data (GTK_OBJECT (parent), DIRTY_CHK_LIST);
   if (dirty_list != NULL)
      {
      g_list_free (dirty_list);
      gtk_object_remove_data (GTK_OBJECT (parent), DIRTY_CHK_LIST);
      }
}


gboolean
widget_is_dirty (GtkWidget *widget)
{
   /*
    * Ya know, if we _really_ wanted to be anal, we could get the top
    * dog for this widget, then determine if it even had a dirty list,
    * then determine if this widget was even in it, but in the interest
    * economy, I will not....
    */
   return (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                 DIRTY_FLAG)) == DIRTY);
}


gboolean
widgets_children_are_dirty (GtkWidget *parent)
{
   GList    *child_list;
   GList    *list_iter;
   gboolean  is_dirty;

   child_list = gtk_object_get_data (GTK_OBJECT (parent), DIRTY_CHK_LIST);
   list_iter = g_list_first (child_list);
   while (list_iter != NULL)
      {
      is_dirty = 
         (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                                DIRTY_FLAG)) == DIRTY);
      if (is_dirty)
         {
         return TRUE;
         }
      list_iter = list_iter->next;
      }

   return FALSE;
}


gboolean
list_dlg_has_dirty_child (GtkWidget *dlg, const gchar *child_list_key)
{
   GList  *list;

   g_assert (dlg != NULL);
   g_assert (child_list_key != NULL);

   list = gtk_object_get_data (GTK_OBJECT (dlg), child_list_key);
   list = g_list_first (list);
   while (list != NULL)
      {
      if (widgets_children_are_dirty (GTK_WIDGET (list->data)))
         {
         return TRUE;
         }
      list = list->next;
      }

   return FALSE;
}


/*
 * Note that this child list is totally a different concept than the
 * DIRTY_CHK_LIST, which is technically a list of child widgets.  Rather,
 * these "child" lists are used to setup a parent/child relationship 
 * between top level widgets.
 */
void
child_dead (GtkObject *child_widget, gchar *list_name)
{
   GList     *child_list;
   GtkWidget *parent;

   parent = GTK_WIDGET (gtk_object_get_data (child_widget, MY_PARENT));
   g_assert (parent != NULL);

   child_list = gtk_object_get_data (GTK_OBJECT (parent), list_name);
   g_assert (child_list != NULL);

   child_list = g_list_remove (child_list, child_widget);
   gtk_object_set_data (GTK_OBJECT (parent), list_name, child_list);
}

void
add_pixmap_directory                   (const gchar *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, 
                                        g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
static gchar*
check_file_exists                      (const gchar    *directory,
                                        const gchar    *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        const gchar     *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("pixmaps", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}


GtkWidget*
get_widget                             (const GtkWidget       *widget,
                                        const gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (const GtkWidget *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, 
                                        page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}


/* Create an Arrow widget with the specified parameters
 * and pack it into a button */
GtkWidget *create_arrow_button (GtkArrowType  arrow_type,
                                GtkShadowType shadow_type)
{
   GtkWidget *button;
   GtkWidget *arrow;

   button = gtk_button_new ();
   arrow = gtk_arrow_new (arrow_type, shadow_type);

   gtk_container_add (GTK_CONTAINER (button), arrow);
                                                           
   gtk_widget_show (arrow);

   return (button);
}
