/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * login_dlg
 *
 * Synopsis:
 *   Database login dialog box.
 *
 * $Id: login_dlg.c,v 1.11 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <preps/gstr_utils.h>

#include <assert.h>
#include <string.h>
#include <syslog.h>

#include "db_utils.h"
#include "gtkutils.h"
#include "util.h"

#define USERNAME_ENTRY       "username_entry"
#define PASSWORD_ENTRY       "password_entry"
#define DB_NAME_COMBO        "db_name_combo"
#define DB_NAME_ENTRY        "db_n_c_entry"
#define HOST_ENTRY           "host_entry"
#define PORT_ENTRY           "port_entry"
#define OK_BTN               "ok_button"
#define CANCEL_BTN           "cancel_button"
#define LOGIN_DATA           "login_data"

/* 
 * this is the return value that is used when we execute as 
 * a modal dialog.
 */
static gint return_value;

static void 
on_db_name_changed (GtkWidget *widget, gpointer user_data)
{
   GtkEntry     *username_entry;
   GtkEntry     *password_entry;
   GtkEntry     *host_entry;
   GtkEntry     *port_entry;
   GtkWidget    *parent = GTK_WIDGET (user_data);
   DbLoginData  *data;
   GString      *tmp_str;

   username_entry = GTK_ENTRY (get_widget (parent, USERNAME_ENTRY));
   password_entry = GTK_ENTRY (get_widget (parent, PASSWORD_ENTRY));
   host_entry = GTK_ENTRY (get_widget (parent, HOST_ENTRY));
   port_entry = GTK_ENTRY (get_widget (parent, PORT_ENTRY));
   data = (DbLoginData *)gtk_object_get_data (GTK_OBJECT (parent), 
                                              LOGIN_DATA);
   assert (username_entry);
   assert (password_entry);
   assert (host_entry);
   assert (port_entry);
   assert (data);

   /*
    * The name DB name has changed.  Reflect that in the db_data.
    */
   tmp_str = g_string_new (gtk_entry_get_text (GTK_ENTRY (widget)));
   tmp_str = g_string_strip (tmp_str);
   set_db_name (data, tmp_str->str, TRUE);
   g_string_free (tmp_str, TRUE);

   /*
    * set_db_name may have changed the username, host, and port
    * settings.  Reflect that back to the user.
    */
   gtk_entry_set_text (username_entry, 
                       (data->username) ? data->username->str : "");
   gtk_entry_set_text (host_entry, 
                       (data->host) ? data->host->str : "");
   gtk_entry_set_text (port_entry, 
                       (data->port) ? data->port->str : "");

   /*
    * Since the DB name has changed, we really should clear the password.
    */
   gtk_entry_set_text (password_entry, "");
   clear_db_password (data);
}


static void
on_connect_dlg_destroy (GtkWidget *widget, gpointer user_data)
{
   if (GTK_WINDOW (user_data)->modal)
      {
      gtk_main_quit ();
      }
}


static void
on_connect_dlg_show (GtkWidget *widget, gpointer user_data)
{
   GtkEntry     *username_entry;
   GtkEntry     *password_entry;
   GtkEntry     *db_name_entry;
   GtkEntry     *host_entry;
   GtkEntry     *port_entry;
   GtkWidget    *parent = GTK_WIDGET (user_data); 
   DbLoginData  *data;
   GtkCombo     *db_name_combo;

   
   /*
    * The pointers to the entry widgets, and a pointer to the 
    * login data structure are all stored in the parent
    * dlg widget.  Get them.
    */
   username_entry = GTK_ENTRY (get_widget (parent, USERNAME_ENTRY));
   password_entry = GTK_ENTRY (get_widget (parent, PASSWORD_ENTRY));
   db_name_entry = GTK_ENTRY (get_widget (parent, DB_NAME_ENTRY));
   host_entry = GTK_ENTRY (get_widget (parent, HOST_ENTRY));
   port_entry = GTK_ENTRY (get_widget (parent, PORT_ENTRY));
   data = (DbLoginData *)gtk_object_get_data (GTK_OBJECT (parent), 
                                              LOGIN_DATA);
   db_name_combo = GTK_COMBO (get_widget (parent, DB_NAME_COMBO));
   assert (db_name_combo);
   assert (username_entry);
   assert (password_entry);
   assert (db_name_entry);
   assert (host_entry);
   assert (port_entry);
   assert (data);

   if (data->db_name_list)
      {
      gtk_combo_set_popdown_strings (db_name_combo, data->db_name_list);
      }

   if (data->db_name)
      {
      gtk_entry_set_text (db_name_entry, data->db_name->str);
      }
   if (data->username)
      {
      gtk_entry_set_text (username_entry, data->username->str);
      }
   if (data->password)
      {
      gtk_entry_set_text (password_entry, data->password->str);
      }
   if (data->host)
      {
      gtk_entry_set_text (host_entry, data->host->str);
      }
   if (data->port)
      {
      gtk_entry_set_text (port_entry, data->port->str);
      }


   /*
    * Connect this signal only after all of the initial "changes"
    * have been made.
    */
   gtk_signal_connect (GTK_OBJECT (db_name_entry),
                       "changed",
                       GTK_SIGNAL_FUNC (on_db_name_changed),
                       parent);
}


static void
on_ok_btn_clicked (gpointer user_data)
{
   GtkEntry     *username_entry;
   GtkEntry     *password_entry;
   GtkEntry     *db_name_entry;
   GtkEntry     *host_entry;
   GtkEntry     *port_entry;
   GtkWidget    *parent = GTK_WIDGET (user_data); 
   DbLoginData  *data;
   GString      *tmp_str = g_string_new ("");

   
   /*
    * The pointers to the entry widgets, and a pointer to the 
    * login data structure are all stored in the parent
    * dlg widget.  Get them.
    */
   username_entry = GTK_ENTRY (get_widget (parent, USERNAME_ENTRY));
   password_entry = GTK_ENTRY (get_widget (parent, PASSWORD_ENTRY));
   db_name_entry = GTK_ENTRY (get_widget (parent, DB_NAME_ENTRY));
   host_entry = GTK_ENTRY (get_widget (parent, HOST_ENTRY));
   port_entry = GTK_ENTRY (get_widget (parent, PORT_ENTRY));
   data = (DbLoginData *)gtk_object_get_data (GTK_OBJECT (parent), 
                                              LOGIN_DATA);
   assert (username_entry);
   assert (password_entry);
   assert (db_name_entry);
   assert (host_entry);
   assert (port_entry);
   assert (data);

   /*
    * For each entry, get the text (if there is any text there),
    * and store it in the login data struct.  The set_xxx functions
    * will handle the memory allocation/deallocation, and NULLing
    * of fields with no text.
    */
   tmp_str = g_string_assign (tmp_str, gtk_entry_get_text (username_entry));
   tmp_str = g_string_strip (tmp_str);
   set_db_username (data, tmp_str->str);
   
   tmp_str = g_string_assign (tmp_str, gtk_entry_get_text (password_entry));
   tmp_str = g_string_strip (tmp_str);
   set_db_password (data, gtk_entry_get_text (password_entry));
   
   tmp_str = g_string_assign (tmp_str, gtk_entry_get_text (db_name_entry));
   tmp_str = g_string_strip (tmp_str);
   set_db_name (data, gtk_entry_get_text (db_name_entry), FALSE);

   tmp_str = g_string_assign (tmp_str, gtk_entry_get_text (host_entry));
   tmp_str = g_string_strip (tmp_str);
   set_db_host (data, gtk_entry_get_text (host_entry));

   tmp_str = g_string_assign (tmp_str, gtk_entry_get_text (port_entry));
   tmp_str = g_string_strip (tmp_str);
   set_db_port (data, gtk_entry_get_text (port_entry));

   return_value = ID_OK;
   g_string_free (tmp_str, TRUE);
   gtk_widget_destroy (GTK_WIDGET (user_data));
}


static void
on_cancel_btn_clicked (gpointer user_data)
{
   return_value = ID_CANCEL;
   gtk_widget_destroy (GTK_WIDGET (user_data));
}


static void
on_connect_dlg_clicked (GnomeDialog *dlg,
                        gint         btn_cd,
                        gpointer     user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_ok_btn_clicked (dlg);
         break;
      case 1:
         on_cancel_btn_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached ();
      }
}


static GtkWidget*
create_login_dlg (GtkWidget    *parent,
                  DbLoginData  *data)
{
   GtkWidget *connect_dlg;
   GtkWidget *dialog_vbox1;
   GtkWidget *table1;
   GtkWidget *label1;
   GtkWidget *label2;
   GtkWidget *label3;
   GtkWidget *label4;
   GtkWidget *label5;
   GtkWidget *username_entry;
   GtkWidget *password_entry;
   GtkWidget *host_entry;
   GtkWidget *port_entry;
   GtkWidget *popup;

   GtkWidget *db_name_combo;
   GtkWidget *db_name_entry; /* entry part of db_name_combo if Gnome is
                                supported.  Otherwise, just an entry. */

   GtkTooltips *tooltips;
 
   tooltips = gtk_tooltips_new ();
 
   connect_dlg = gnome_dialog_new (_("Connect to Database"),
                                   GNOME_STOCK_BUTTON_OK,
                                   GNOME_STOCK_BUTTON_CANCEL,
                                   NULL);
   gtk_object_set_data (GTK_OBJECT (connect_dlg), 
                        "connect_dlg", 
                        connect_dlg);
   gtk_object_set_data (GTK_OBJECT (connect_dlg),
                        LOGIN_DATA,
                        data);
   gtk_window_set_policy (GTK_WINDOW (connect_dlg), FALSE, FALSE, TRUE);

   gtk_signal_connect (GTK_OBJECT (connect_dlg), "show",
                       GTK_SIGNAL_FUNC (on_connect_dlg_show),
                       connect_dlg);
   gtk_signal_connect (GTK_OBJECT (connect_dlg), "destroy",
                       GTK_SIGNAL_FUNC (on_connect_dlg_destroy),
                       connect_dlg);
   gtk_signal_connect (GTK_OBJECT (connect_dlg), "clicked",
                       GTK_SIGNAL_FUNC (on_connect_dlg_clicked),
                       NULL);
   if (parent != NULL)
      {
      gnome_dialog_set_parent (GNOME_DIALOG (connect_dlg),
                               GTK_WINDOW (parent));
      }
   else
      {
      gtk_window_position (GTK_WINDOW (connect_dlg), GTK_WIN_POS_CENTER);
      }

   dialog_vbox1 = GNOME_DIALOG (connect_dlg)->vbox;
   gtk_widget_show (dialog_vbox1);

   table1 = gtk_table_new (5, 2, FALSE);
   gtk_widget_ref (table1);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "table1", table1,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (table1);
   gtk_box_pack_start (GTK_BOX (dialog_vbox1), table1, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
   gtk_table_set_row_spacings (GTK_TABLE (table1), 5);
   gtk_table_set_col_spacings (GTK_TABLE (table1), 2);

   label3 = gtk_label_new ("Database:");
   gtk_widget_ref (label3);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "label3", label3,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label3);
   gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 0, 1,
                     (GtkAttachOptions) (0),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);

   label1 = gtk_label_new ("User Name:");
   gtk_widget_ref (label1);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "label1", label1,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label1);
   gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_RIGHT);

   label2 = gtk_label_new ("Password:");
   gtk_widget_ref (label2);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "label2", label2,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label2);
   gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 2, 3,
                     (GtkAttachOptions) (0),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);

   label4 = gtk_label_new ("Host:");
   gtk_widget_ref (label4);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "label4", label4,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label4);
   gtk_table_attach (GTK_TABLE (table1), label4, 0, 1, 3, 4,
                     (GtkAttachOptions) (0),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_RIGHT);
   gtk_misc_set_alignment (GTK_MISC (label4), 1, 0.5);

   label5 = gtk_label_new ("Port:");
   gtk_widget_ref (label5);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "label5", label5,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (label5);
   gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 4, 5,
                     (GtkAttachOptions) (0),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_RIGHT);

   /*
    * If Gnome is supported, make this a combo.  Otherwise it is 
    * just an entry.
    */
   db_name_combo = gtk_combo_new ();
   gtk_widget_ref (db_name_combo);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), 
                             DB_NAME_COMBO, 
                             db_name_combo,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (db_name_combo);
   gtk_table_attach (GTK_TABLE (table1), db_name_combo, 1, 2, 0, 1,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   db_name_entry = GTK_COMBO (db_name_combo)->entry;
   gtk_widget_ref (db_name_entry);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), 
                             DB_NAME_ENTRY, 
                             db_name_entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (db_name_entry);
   gtk_tooltips_set_tip (tooltips, 
                         db_name_entry, 
                         "The name of the database to connect to.", 
                         NULL);

   username_entry = gtk_entry_new ();
   gtk_widget_ref (username_entry);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), 
                             USERNAME_ENTRY, 
                             username_entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (username_entry);
   gtk_table_attach (GTK_TABLE (table1), username_entry, 1, 2, 1, 2,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   popup = attach_generic_edit_popup_menu (username_entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "popup1", popup,
                             (GtkDestroyNotify) gtk_widget_unref);

   password_entry = gtk_entry_new ();
   gtk_widget_ref (password_entry);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), 
                             PASSWORD_ENTRY, 
                             password_entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_entry_set_visibility (GTK_ENTRY (password_entry), FALSE);
   gtk_widget_show (password_entry);
   gtk_table_attach (GTK_TABLE (table1), password_entry, 1, 2, 2, 3,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   popup = attach_generic_edit_popup_menu (password_entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "popup2", popup,
                             (GtkDestroyNotify) gtk_widget_unref);

   host_entry = gtk_entry_new ();
   gtk_widget_ref (host_entry);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), 
                             HOST_ENTRY, 
                             host_entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (host_entry);
   gtk_table_attach (GTK_TABLE (table1), host_entry, 1, 2, 3, 4,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   popup = attach_generic_edit_popup_menu (host_entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "popup3", popup,
                             (GtkDestroyNotify) gtk_widget_unref);

   port_entry = gtk_entry_new ();
   gtk_widget_ref (port_entry);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), 
                             PORT_ENTRY, 
                             port_entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (port_entry);
   gtk_table_attach (GTK_TABLE (table1), port_entry, 1, 2, 4, 5,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   popup = attach_generic_edit_popup_menu (port_entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (connect_dlg), "popup4", popup,
                             (GtkDestroyNotify) gtk_widget_unref);

   gtk_object_set_data (GTK_OBJECT (connect_dlg), "tooltips", tooltips);

   return connect_dlg;
}


gint
execute_login_dlg (GtkWidget    *parent,
                   DbLoginData  *data)
{
   GtkWidget  *dlg;

   dlg = create_login_dlg (parent, data);

   gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
   gtk_widget_show (dlg);

   gtk_main ();

   return return_value;
}
