/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * probentry_dlg
 *
 * Synopsis:
 *   Dialog for entry of problem reports.
 *
 * $Id: probentry_dlg.c,v 1.30 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gnome.h>
#include <libpq-fe.h>

#include <preps/gstr_utils.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "util.h"

/*
 *  These control the minimum number of characters that a user must
 *  enter in the subject and descr fields.  If you want your users to
 *  be forced to enter lengthier stuff, bump these numbers up.
 */
#define MIN_SUBJECT_LEN         1
#define MIN_DESCR_LEN           1 

/*
 *  Define the names of some of the key widgets in the dialog.  
 */
#define PROJECT_CLIST           "project_clist"
#define PROBLEM_TYPE_CLIST      "problem_type_clist"
#define SEVERITY_CLIST          "severity_clist"
#define SUBJECT_ENTRY           "subject_entry"
#define VERSION_COMBO           "version_combo"
#define VERSION_ENTRY           "version_entry"
#define DESCR_TEXT              "descr_text"

/*
 * The VERSION_NAME_PK_LIST stores the name/pk combinations that are
 * used in the VERSION_COMBO.
 */
#define VERSION_NAME_PK_LIST    "version_name_pk_list"

/*
 * 'Null' version display data (for the combo boxes).
 */
#define NULL_NAME           _("<None>")
#define NULL_NAME_SIZE      6

/*
 *  These queries are used to fill in the selection lists for 
 *  project, problem type, and severity.
 */
#define PK_NUM_POS   0
#define NAME_POS     1
#define DEF_SEVR_POS 2
#define GET_PROJECTS "SELECT p.project_num, p.name \
                        FROM project p, submitter s \
                       WHERE p.active = TRUE \
                         AND p.project_num = s.project_num \
                         AND s.login_id = '%s'"
#define GET_PROBLEM_TYPES "SELECT type_num, name, severity_num \
                             FROM problem_type \
                            WHERE active = TRUE \
                            ORDER BY order_num "
#define GET_SEVERITIES "SELECT severity_num, name, order_num \
                          FROM severity \
                         WHERE active = TRUE \
                         ORDER BY order_num"
#define GET_LOW_STATUS "SELECT status_num \
                          FROM status \
                         WHERE active = TRUE \
                           AND order_num in \
                                (SELECT MIN (order_num) \
                                   FROM status \
                                  WHERE active = TRUE)" 
                                
#define INSERT_PROBLEM "INSERT INTO problem_report (problem_num, project_num, \
                                    severity_num, type_num, status_num, \
                                    submitter_id, title, descr, open_version) \
                        VALUES (nextval ('problem_report_seq'), %d, %d, \
                                %d, %d, getpgusername(), %s, %s, "

#define VERSION_QUERY "SELECT version_num, version_text \
                         FROM project_version \
                        WHERE project_num = %d \
                          AND active = TRUE \
                        ORDER BY order_num "
/*
 * This data structure couples the problem type with it's 
 * default severity.
 */
typedef struct type_sevr_struct {
   gint   pk_type;
   gint   pk_sevr;
} type_sevr_struct;


/*
 *  on_problem_entry_dlg_destroy
 *  
 *  We are done, free any memory that the dialog was using. 
 */
static void 
on_problem_entry_dlg_destroy (GtkObject *object,
                              gpointer   user_data)
{
   GList    *version_name_pk_list;

   /*
    * Get the name/pk list associated with the version combo, and
    * destroy it.
    */
   version_name_pk_list = gtk_object_get_data (object, VERSION_NAME_PK_LIST);
   destroy_name_pk_list (version_name_pk_list);

   destroy_dirty_list (GTK_WIDGET (object));

   /*
    * If this is a modal dialog, we need to shut down it's message
    * processing loop.
    */
   if (GTK_WINDOW(object)->modal)
      {
      gtk_main_quit ();
      }
}


/*
 *  on_problem_entry_dlg_show
 *  
 *  First showing the dialog box.  Initialize the project, problem type,
 *  and severity lists.  For the project list, limit the projects to 
 *  just those that the user has access to.
 */
static void 
on_problem_entry_dlg_show (GtkWidget  *widget,
                           gpointer    user_data)
{
   GtkCList          *project_clist;
   GtkCList          *type_clist;
   GtkCList          *sevr_clist;
   GtkCombo          *version_combo;
   GList             *full_list = NULL;
   GList             *name_list = NULL;
   gboolean           in_error = FALSE;
   gint               i;
   gint               n;
   gint               pk_num;
   GString           *buffer;
   type_sevr_struct  *tmp_ts;
   name_pk_struct    *curr_name_pk;
   PGconn            *conn;   
   PGresult          *res;
   char              *list_buf[1];
   
   project_clist = GTK_CLIST (get_widget (widget, PROJECT_CLIST));
   type_clist = GTK_CLIST (get_widget (widget, PROBLEM_TYPE_CLIST));
   sevr_clist = GTK_CLIST (get_widget (widget, SEVERITY_CLIST));
   version_combo = GTK_COMBO (get_widget (widget, VERSION_COMBO));
   g_assert (project_clist != NULL);
   g_assert (type_clist != NULL);
   g_assert (sevr_clist != NULL);
   g_assert (version_combo != NULL);

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   
   /*
    * Fill the project list with all active projects that this user 
    * has submitter access to.
    */
   buffer = g_string_new ("");
   g_string_sprintf (buffer, GET_PROJECTS, PQuser (conn));
   res = PQexec (conn, buffer->str);
   if (chk_sql_error (res, _("filling the project list")))
      {
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         list_buf[0] = g_malloc (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
         gtk_clist_append (project_clist, list_buf);
         pk_num = atoi (PQgetvalue (res, i, PK_NUM_POS));
         gtk_clist_set_row_data (project_clist, i, GINT_TO_POINTER (pk_num));
         g_free (list_buf[0]);
         }
      }
   else
      {
      in_error = TRUE;
      }
   PQclear (res);
   g_string_free (buffer, TRUE);
   
   /*
    *  If we are not in an error state, fill in the problem type list
    *  with all active problem types.
    */
   if (!in_error)
      {
      res = PQexec (conn, GET_PROBLEM_TYPES);
      if (chk_sql_error (res, _("filling the problem types list")))
         {
         n = PQntuples (res);
         for (i = 0; i < n; i++)
            {
            list_buf[0] = g_malloc 
               (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
            strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
            gtk_clist_append (type_clist, list_buf);
            /*
             *  Note that we are going to let the clist take care of
             *  deleting the tmp_ts data whenever it is deleted (see
             *  usage of gtk_clist_set_row_data_full).
             */
            tmp_ts = (type_sevr_struct *)g_malloc 
               (sizeof (type_sevr_struct));
            tmp_ts->pk_type = 
               atoi (PQgetvalue (res, i, PK_NUM_POS));
            tmp_ts->pk_sevr =
               atoi (PQgetvalue (res, i, DEF_SEVR_POS));
            gtk_clist_set_row_data_full (type_clist, i, tmp_ts, g_free);
            g_free (list_buf[0]);
            }
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }
   
   /*
    *  If we are not in an error state, fill in the severity list with
    *  all active severities.
    */
   if (!in_error)
      {
      res = PQexec (conn, GET_SEVERITIES);
      if (chk_sql_error (res, _("filling the severities list")))
         {
         n = PQntuples (res);
         for (i = 0; i < n; i++)
            {
            list_buf[0] = g_malloc
               (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
            strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
            gtk_clist_append (sevr_clist, list_buf);
            pk_num = atoi (PQgetvalue (res, i, PK_NUM_POS));
            gtk_clist_set_row_data (sevr_clist, i, GINT_TO_POINTER (pk_num));
            g_free (list_buf[0]);
            }
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * We do not know what project the user has chosen yet, so just 
    * initialize with the <None> value.
    */
   curr_name_pk = g_malloc (sizeof (name_pk_struct));
   curr_name_pk->pk = INVALID_PK;
   curr_name_pk->name = g_malloc (NULL_NAME_SIZE + 1);
   strcpy (curr_name_pk->name, NULL_NAME);
   name_list = g_list_append (name_list, curr_name_pk->name);
   full_list = g_list_append (full_list, curr_name_pk);
   gtk_combo_set_popdown_strings (version_combo, name_list);
   g_list_free (name_list);
   gtk_object_set_data (GTK_OBJECT (widget),
                        VERSION_NAME_PK_LIST,
                        full_list);
   
   if (in_error)
      {
      gtk_widget_destroy (widget);
      }
   else
      {
      set_all_clean (widget);
      }
}


/*
 *  on_ok_button_clicked
 *  
 *  clicked signal handler for the OK button
 *  
 *  On OK, we want to insert the problem report into the database.  For
 *  the status, we will use the lowest active status.
 */
static void 
on_ok_button_clicked (GnomeDialog *dlg)
{
   GtkWidget         *parent;  /* application main window */
   GtkCList          *proj_clist;
   GtkCList          *type_clist;
   GtkCList          *sevr_clist;
   GtkEntry          *version_entry;
   GtkEntry          *subject_entry;
   GtkText           *descr_text;
   gboolean           ok_to_process = TRUE;
   GString           *subject_str;
   GString           *descr_str;
   GString           *msg_str;
   GString           *buffer;
   gchar              num_buffer[NUM_BUFFER_LEN + 1]; /* xtra char for ')' */
   GList             *ver_list;
   gint               row_idx;
   gint               pk_proj;
   type_sevr_struct  *ts;
   gint               pk_sevr;
   gint               pk_stat;
   gint               pk_ver;
   gchar             *char_buf;
   PGconn            *conn;
   PGresult          *res;
   GtkWidget         *msg_dlg;

   proj_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg),
                                       PROJECT_CLIST));
   type_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg),
                                       PROBLEM_TYPE_CLIST));
   sevr_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg),
                                       SEVERITY_CLIST));
   version_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg),
                                          VERSION_ENTRY));
   ver_list = gtk_object_get_data (GTK_OBJECT (dlg),
                                   VERSION_NAME_PK_LIST);
   subject_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg),
                                          SUBJECT_ENTRY));
   descr_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg),
                                      DESCR_TEXT));
   parent = get_widget (GTK_WIDGET (dlg), MY_PARENT);
   g_assert (proj_clist    != NULL);
   g_assert (type_clist    != NULL);
   g_assert (sevr_clist    != NULL);
   g_assert (version_entry != NULL);
   g_assert (ver_list      != NULL);
   g_assert (subject_entry != NULL);
   g_assert (descr_text    != NULL);
   g_assert (parent        != NULL);

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   
   subject_str = g_string_new (gtk_entry_get_text (subject_entry));
   g_string_prepare_db_instr (subject_str);
   char_buf = gtk_editable_get_chars (GTK_EDITABLE (descr_text), 0, -1);
   descr_str = g_string_new (char_buf);
   g_free (char_buf);
   g_string_prepare_db_instr (descr_str);
   /*
    *  Do a top down check of the data that the user entered. 
    *  
    *  The user needs to have selected a:
    *    project
    *    problem type
    *    severity
    *    
    *  In addition, the user needs to have entered at least the
    *  minimum number of characters in the subject and descr.
    */
   if (!proj_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a project."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      ok_to_process = FALSE;
      }
   
   if (ok_to_process && !type_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a problem type."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      ok_to_process = FALSE;
      }
   
   if (ok_to_process && !sevr_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a severity."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      ok_to_process = FALSE;
      }

   /*
    * For this one, we must subtract 2 for the extra single quotes we
    * inserted when preparing the string.  Also check for "NULL" if
    * the MIN_SUBJECT_LEN is greater than zero.
    */
   if (ok_to_process && 
         (subject_str->len < MIN_SUBJECT_LEN - 2 || 
          (MIN_SUBJECT_LEN > 0 && !strcmp (subject_str->str, "NULL"))))
      {
      msg_str = g_string_new ("");
      g_string_sprintf 
         (msg_str, 
          _("You must enter at least %d character(s) in the subject"),
          MIN_SUBJECT_LEN);
      msg_dlg = gnome_message_box_new (msg_str->str,
                                       GNOME_MESSAGE_BOX_ERROR,
                                       GNOME_STOCK_BUTTON_OK,
                                       NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      ok_to_process = FALSE;
      g_string_free (msg_str, TRUE);
      }
   
   /*
    * For this one, we must subtract 2 for the extra single quotes we
    * inserted when preparing the string.  Also check for "NULL" if
    * the MIN_DESCR_LEN is greater than zero.
    */
   if (ok_to_process && 
         (descr_str->len < MIN_DESCR_LEN ||
          (MIN_DESCR_LEN > 0 && !strcmp (descr_str->str, "NULL"))))
      {
      msg_str = g_string_new ("");
      g_string_sprintf
         (msg_str, 
          _("You must enter at least %d character(s) in the description"),
          MIN_DESCR_LEN);
      msg_dlg = gnome_message_box_new (msg_str->str,
                                       GNOME_MESSAGE_BOX_ERROR,
                                       GNOME_STOCK_BUTTON_OK,
                                       NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      ok_to_process = FALSE;
      g_string_free (msg_str, TRUE);
      }
   
   /*
    *  If we are OK to process at this point, then let's do the
    *  insert.
    */
   if (ok_to_process)
      {
      row_idx = GPOINTER_TO_INT (proj_clist->selection->data);
      pk_proj = GPOINTER_TO_INT (gtk_clist_get_row_data (proj_clist, 
                                                         row_idx));

      row_idx = GPOINTER_TO_INT (type_clist->selection->data);
      ts = gtk_clist_get_row_data (type_clist, row_idx);

      row_idx = GPOINTER_TO_INT (sevr_clist->selection->data);
      pk_sevr = GPOINTER_TO_INT (gtk_clist_get_row_data (sevr_clist,
                                                         row_idx));

      START_TRANSACTION (conn);
      buffer = g_string_new (GET_LOW_STATUS);
      res = PQexec (conn, buffer->str);
      chk_sql_error (res, _("getting low status"));
      pk_stat = atoi (PQgetvalue (res, 0, 0));
      PQclear (res);

      g_string_sprintf (buffer, 
                        INSERT_PROBLEM, 
                        pk_proj, pk_sevr, ts->pk_type, pk_stat,
                        subject_str->str, descr_str->str); 

      pk_ver = find_pk_from_name (ver_list, 
                                  gtk_entry_get_text (version_entry));
      if (pk_ver == INVALID_PK)
         {
         buffer = g_string_append (buffer, "NULL)");
         }
      else
         {
         sprintf (num_buffer, "%d)", pk_ver);
         buffer = g_string_append (buffer, num_buffer);
         }
      res = PQexec (conn, buffer->str);
      if (chk_sql_error (res, _("inserting problem report")))
         {
         COMMIT (conn);
         save_dlg_size (GTK_WIDGET (dlg), PROB_ENT_SEC, FALSE);
         gnome_dialog_close (dlg);
         refresh_pr_list (parent);
         }
      else
         {
         ROLLBACK (conn);
         }
      PQclear (res);
      g_string_free (buffer, TRUE);
      }

   g_string_free (subject_str, TRUE);
   g_string_free (descr_str, TRUE);
}


/*
 *  on_cancel_button_clicked
 *  
 *  clickec signal handler for the Cancel button
 *  
 *  On Cancel, we just want to close the dialog box.   No insert into
 *  the database.
 */
static void 
on_cancel_button_clicked (GnomeDialog *dlg)
{
   save_dlg_size (GTK_WIDGET (dlg), PROB_ENT_SEC, FALSE);
   gnome_dialog_close (dlg);
} 


static void
on_problem_entry_dlg_clicked (GnomeDialog *dlg,
                             gint         btn_cd,
                             gpointer     user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_ok_button_clicked (dlg);
         break;
      case 1:
         on_cancel_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


/*
 *  on_project_clist_select_row
 *
 *  select_row signal handler
 *
 *  When the user selects a problem type, we need to load the version
 *  combo with the version information for the selected project.
 */
static void
on_project_clist_select_row (GtkCList   *clist,
                             gint        row,
                             gint        column,
                             GdkEvent   *event,
                             gpointer    user_data)
{
   GtkCombo        *version_combo;
   GList           *full_list;
   GList           *name_list = NULL;
   gint             pk_proj;
   PGresult        *res;
   name_pk_struct  *cur_ver;
   GString         *sql_buffer;
   GString         *def_ver;
   gint             def_ver_pk;
   gint             i, n;
   GtkWidget       *main_widget = GTK_WIDGET (user_data);
   PGconn          *conn;

   conn = gtk_object_get_data (GTK_OBJECT (main_widget), MY_DB_CONN);
   g_assert (conn != NULL);
   version_combo = GTK_COMBO (get_widget (main_widget, VERSION_COMBO));
   
   /*
    * Get th previous list and destroy it.
    */
   full_list = gtk_object_get_data (GTK_OBJECT (main_widget), 
                                    VERSION_NAME_PK_LIST);
   destroy_name_pk_list (full_list);
   full_list = NULL;
   gtk_object_remove_data (GTK_OBJECT (main_widget), VERSION_NAME_PK_LIST);

   /*
    * Get the default version for this project.
    */
   pk_proj = GPOINTER_TO_INT (gtk_clist_get_row_data (clist, row));
   sql_buffer = g_string_new ("");
   g_string_sprintf 
      (sql_buffer,
       _("select open_version from project where project_num = %d"),
       pk_proj);
   res = PQexec (conn, sql_buffer->str);
   g_assert (PQntuples (res) == 1);
   def_ver = g_string_new (NULL_NAME);
   def_ver_pk = INVALID_PK;
   if (chk_sql_error (res, _("Getting default open version")))
      {
      if (!PQgetisnull (res, 0, 0))
         {
         def_ver_pk = atoi (PQgetvalue (res, 0, 0));
         }
      }
   PQclear (res);

   /*
    * Always start with <None>
    */
   cur_ver = g_malloc (sizeof (name_pk_struct));
   cur_ver->pk = INVALID_PK;
   cur_ver->name = g_malloc (NULL_NAME_SIZE + 1);
   strcpy (cur_ver->name, NULL_NAME);
   name_list = g_list_append (name_list, cur_ver->name);
   full_list = g_list_append (full_list, cur_ver);
   /*
    * Determine the project PK.  Build the list of versions for this
    * project.
    */
   g_string_sprintf (sql_buffer, VERSION_QUERY, pk_proj);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Getting project versions")))
      {
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         cur_ver = g_malloc (sizeof (name_pk_struct));
         cur_ver->pk = atoi (PQgetvalue (res, i, PK_NUM_POS));
         cur_ver->name = 
            g_malloc (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (cur_ver->name, PQgetvalue (res, i, NAME_POS));
         if (cur_ver->pk == def_ver_pk)
            {
            def_ver = g_string_assign (def_ver, cur_ver->name);
            }
         name_list = g_list_append (name_list, cur_ver->name);
         full_list = g_list_append (full_list, cur_ver);
         }
      }

   /*
    * Even if the SQL failed, we at least have a list of one item
    * to install.
    */
   gtk_combo_set_popdown_strings (version_combo, name_list);
   gtk_entry_set_text (GTK_ENTRY (version_combo->entry), def_ver->str);
   gtk_object_set_data (GTK_OBJECT (main_widget), 
                        VERSION_NAME_PK_LIST,
                        full_list);

   /*
    * Clean up.
    */
   g_string_free (sql_buffer, TRUE);
   g_string_free (def_ver, TRUE);
   g_list_free (name_list);
   PQclear (res);
}

/*
 *  on_problem_type_clist_select_row
 *  
 *  select_row signal handler
 *  
 *  When the user selects a problem type, we auto-select the default
 *  severity for that problem type.
 */
static void 
on_problem_type_clist_select_row (GtkCList  *clist,
                                  gint       row,
                                  gint       column,
                                  GdkEvent  *event,
                                  gpointer   user_data)
{
   GtkCList          *sevr_list;
   type_sevr_struct  *ts;
   gint               sevr_row;
   
   sevr_list = GTK_CLIST (get_widget (GTK_WIDGET (user_data),
                                      SEVERITY_CLIST)); 
   g_assert (sevr_list != NULL);
   ts = gtk_clist_get_row_data (clist, row);
   
   gtk_clist_freeze (sevr_list);
   sevr_row = gtk_clist_find_row_from_data 
      (sevr_list, GINT_TO_POINTER (ts->pk_sevr));
   
   gtk_clist_select_row (sevr_list, sevr_row, 0);
   gtk_clist_moveto (sevr_list, sevr_row, 0, 0.5, 0.0);
   gtk_clist_thaw (sevr_list);
}


GtkWidget* 
create_problem_entry_dlg (GtkWidget *parent,
                          PGconn    *conn)
{
  GtkWidget        *problem_entry_dlg;
  GtkWidget        *dialog_vbox16;
  GtkWidget        *vbox30;
  GtkWidget        *table1;
  GtkWidget        *label19;
  GtkWidget        *scrolledwindow8;
  GtkWidget        *project_clist;
  GtkWidget        *label24;
  GtkWidget        *label20;
  GtkWidget        *scrolledwindow9;
  GtkWidget        *problem_type_clist;
  GtkWidget        *label25;
  GtkWidget        *label21;
  GtkWidget        *scrolledwindow10;
  GtkWidget        *severity_clist;
  GtkWidget        *label26;
  GtkWidget        *label27;
  GtkWidget        *label23;
  GtkWidget        *subject_entry;
  GtkWidget        *version_combo;
  GtkWidget        *version_entry;
  GtkWidget        *label22;
  GtkWidget        *scrolledwindow7;
  GtkWidget        *descr_text;
  GtkWidget        *popup;

  problem_entry_dlg = gnome_dialog_new (_("Enter Problem Report"),
                                        GNOME_STOCK_BUTTON_OK,
                                        GNOME_STOCK_BUTTON_CANCEL,
                                        NULL);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       "problem_entry_dlg", 
                       problem_entry_dlg);
  gtk_container_border_width (GTK_CONTAINER (problem_entry_dlg), 5);
  GTK_WINDOW (problem_entry_dlg)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_policy (GTK_WINDOW (problem_entry_dlg), FALSE, TRUE, FALSE);
  gtk_signal_connect (GTK_OBJECT (problem_entry_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_problem_entry_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (problem_entry_dlg), "show",
                      GTK_SIGNAL_FUNC (on_problem_entry_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (problem_entry_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_problem_entry_dlg_clicked),
                      NULL);
  gnome_dialog_set_default (GNOME_DIALOG (problem_entry_dlg), 0);
  gnome_dialog_set_close (GNOME_DIALOG (problem_entry_dlg), FALSE);
  gnome_dialog_set_parent (GNOME_DIALOG (problem_entry_dlg),
                           GTK_WINDOW (parent));
  set_dlg_size (problem_entry_dlg, PROB_ENT_SEC, 550, 360);

  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), TOP_DOG, 
                       problem_entry_dlg);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), MY_DB_CONN, conn);

  dialog_vbox16 = GNOME_DIALOG (problem_entry_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox16), 5);
  gtk_widget_show (dialog_vbox16);

  vbox30 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "vbox30", vbox30);
  gtk_widget_show (vbox30);
  gtk_box_pack_start (GTK_BOX (dialog_vbox16), vbox30, TRUE, TRUE, 0);

  table1 = gtk_table_new (4, 3, FALSE);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "table1", table1);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox30), table1, FALSE, FALSE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 5);
  
  label19 = gtk_label_new (_("Project"));
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label19", label19);
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table1), label19, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  scrolledwindow8 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       "scrolledwindow8", 
                       scrolledwindow8);
  gtk_widget_show (scrolledwindow8);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow8, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow8), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  project_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       PROJECT_CLIST, 
                       project_clist);
  gtk_widget_show (project_clist);
  gtk_signal_connect (GTK_OBJECT (project_clist), "select_row",
                      GTK_SIGNAL_FUNC (on_project_clist_select_row),
                      problem_entry_dlg);
  gtk_container_add (GTK_CONTAINER (scrolledwindow8), project_clist);
  gtk_clist_set_column_width (GTK_CLIST (project_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (project_clist));
  gtk_signal_connect (GTK_OBJECT (project_clist), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (problem_entry_dlg, project_clist);

  label24 = gtk_label_new ("label24");
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label24", label24);
  gtk_clist_set_column_widget (GTK_CLIST (project_clist), 0, label24);
  gtk_widget_set_sensitive (label24, FALSE);

  label20 = gtk_label_new (_("Problem Type"));
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label20", label20);
  gtk_widget_show (label20);
  gtk_table_attach (GTK_TABLE (table1), label20, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);

  gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

  scrolledwindow9 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       "scrolledwindow9", 
                       scrolledwindow9);
  gtk_widget_show (scrolledwindow9);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow9, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow9), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  problem_type_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       PROBLEM_TYPE_CLIST, 
                       problem_type_clist);
  gtk_widget_show (problem_type_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow9), problem_type_clist);
  gtk_signal_connect (GTK_OBJECT (problem_type_clist), "select_row",
                      GTK_SIGNAL_FUNC (on_problem_type_clist_select_row),
                      problem_entry_dlg);
  gtk_clist_set_column_width (GTK_CLIST (problem_type_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (problem_type_clist));
  gtk_signal_connect (GTK_OBJECT (problem_type_clist), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (problem_entry_dlg, problem_type_clist);

  label25 = gtk_label_new ("label25");
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label25", label25);
  gtk_clist_set_column_widget (GTK_CLIST (problem_type_clist), 0, label25);
  gtk_widget_set_sensitive (label25, FALSE);

  label21 = gtk_label_new (_("Severity"));
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label21", label21);
  gtk_widget_show (label21);
  gtk_table_attach (GTK_TABLE (table1), label21, 2, 3, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    0, 0);
  gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

  scrolledwindow10 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       "scrolledwindow10", 
                       scrolledwindow10);
  gtk_widget_show (scrolledwindow10);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow10, 2, 3, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow10), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  severity_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       SEVERITY_CLIST, 
                       severity_clist);
  gtk_widget_show (severity_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow10), severity_clist);
  gtk_clist_set_column_width (GTK_CLIST (severity_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (severity_clist));
  gtk_signal_connect (GTK_OBJECT (severity_clist), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (problem_entry_dlg, severity_clist);

  label26 = gtk_label_new ("label26");
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label26", label26);
  gtk_clist_set_column_widget (GTK_CLIST (severity_clist), 0, label26);
  gtk_widget_set_sensitive (label26, FALSE);

  label23 = gtk_label_new (_("Subject"));
  gtk_widget_ref (label23);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg), 
                            "label23", 
                            label23,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label23);
  gtk_table_attach (GTK_TABLE (table1), label23, 0, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);

  label27 = gtk_label_new (_("Version"));
  gtk_widget_ref (label27);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg),
                            "label27",
                            label27,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label27);
  gtk_table_attach (GTK_TABLE (table1), label27, 2, 3, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND |GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_misc_set_alignment (GTK_MISC (label27), 0, 0.5);

  subject_entry = gtk_entry_new ();
  gtk_widget_ref (subject_entry);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg), 
                            SUBJECT_ENTRY, 
                            subject_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (subject_entry);
  gtk_table_attach (GTK_TABLE (table1), subject_entry, 0, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    0, 0);
  gtk_signal_connect (GTK_OBJECT (subject_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (problem_entry_dlg, subject_entry);
  popup = attach_generic_edit_popup_menu (subject_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg), "popup1",
                            popup, (GtkDestroyNotify) gtk_widget_unref);

  version_combo = gtk_combo_new ();
  gtk_widget_ref (version_combo);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg),
                            VERSION_COMBO,
                            version_combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (version_combo);
  gtk_table_attach (GTK_TABLE (table1), version_combo, 2, 3, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND |GTK_FILL,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
                    0, 0);
  version_entry = GTK_COMBO (version_combo)->entry;
  gtk_widget_ref (version_entry);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg),
                            VERSION_ENTRY,
                            version_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (version_entry);
  gtk_entry_set_editable (GTK_ENTRY (version_entry), FALSE);
  gtk_signal_connect (GTK_OBJECT (version_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (problem_entry_dlg, version_entry);

  label22 = gtk_label_new (_("Description"));
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), "label22", label22);
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (vbox30), label22, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label22), 0, 0.5);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       "scrolledwindow7", 
                       scrolledwindow7);
  gtk_widget_show (scrolledwindow7);
  gtk_box_pack_start (GTK_BOX (vbox30), scrolledwindow7, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  descr_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (problem_entry_dlg), 
                       DESCR_TEXT, 
                       descr_text);
  gtk_widget_show (descr_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), descr_text);
  gtk_text_set_editable (GTK_TEXT (descr_text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (descr_text), TRUE);
  gtk_signal_connect (GTK_OBJECT (descr_text), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (problem_entry_dlg, descr_text);
  popup = attach_generic_edit_popup_menu (descr_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (problem_entry_dlg), "popup2",
                            popup, (GtkDestroyNotify) gtk_widget_unref);

  return problem_entry_dlg;
}
