/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * probupdate_dlg
 *
 * Synopsis:
 *   Handle updates to, and viewing of existing problem reports.
 *
 * $Id: probupdate_dlg.c,v 1.46 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gnome.h>
#include <libpq-fe.h>

#include <preps/audit_trail.h>
#include <preps/gstr_utils.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "util.h"

/*
 * Define the names of some of the key widgets.
 */
#define PROJECT_ENTRY          "project_entry"
#define SUBMITTER_ENTRY        "submitter_entry"
#define RESPONSIBLE_COMBO      "responsible_combo"
#define SUBJECT_ENTRY          "subject_entry"
#define STATUS_CLIST           "status_clist"
#define PROBLEM_TYPE_ENTRY     "problem_type_entry"
#define SEVERITY_CLIST         "severity_clist"
#define DESCR_EDIT             "descr_edit"
#define FIX_DESCR_EDIT         "fix_descr_edit"
#define AUDIT_TRAIL_EDIT       "aud_tr_edit"
#define RESPONSIBLE_LIST       "responsible_list"
#define OPEN_VER_ENTRY         "open_ver_entry"
#define FIX_VER_COMBO          "fix_ver_combo"
#define FIX_VER_ENTRY          "fix_ver_entry"
#define CHANGE_LOG_TEXT        "change_log_text"
#define FIX_TAB_LBL            "fix_tab_label"
#define FIX_TAB_CHAR_CNT       "fix_tab_char_cnt"

#define FIX_DESCR              _("Fix Description")
#define FIX_DESCR_STAR         _("Fix Description*")

#define RESP_NOBODY_NAME       _("<None>")
#define RESP_NOBODY_ID         "" 

#define CLIST_START   7
#define CLIST_END     10 

/*
 * If the position of any of the buttons change, change it here to 
 * affect pretty much everything else...
 */
#define OK_BUTTON_POS      0
#define APPLY_BUTTON_POS   1
#define CLOSE_BUTTON_POS   2

/*
 * This "locking" is voluntary, but it seems to work much nicer than the
 * current levels of locking available from PostgreSQL.  The basic idea
 * here is to run the CREATE_LOCK query, then check and make sure that 
 * the locker is you.  If the locker is someone else, they had it first.
 *
 * REMOVE_LOCK must be run before the dialog exits.
 */
#define CREATE_LOCK "\
UPDATE problem_report \
   SET locker_id = getpgusername () \
 WHERE locker_id is NULL \
   AND problem_num = %d "
#define SELECT_LOCKER "\
SELECT pr.locker_id, \
       p.last_name || ', ' || p.first_name || ' (' || pr.locker_id || ')' \
  FROM problem_report pr, person p \
 WHERE p.login_id = pr.locker_id \
   AND problem_num = %d "
#define REMOVE_LOCK "\
UPDATE problem_report \
   SET locker_id = NULL \
 WHERE locker_id = getpgusername () \
   AND problem_num = %d "

/*
 * Get all of the information that we need for the problem report.
 */
#define PRJ_NUM_POS          0
#define PRJ_NAME_POS         1 
#define SUBMITTER_ID_POS     2 
#define RESPONSIBLE_ID_POS   3
#define TITLE_POS            4
#define DESCR_POS            5
#define FIX_POS              6
#define SEVERITY_NUM_POS     7
#define STATUS_NUM_POS       8
#define PROB_TYPE_POS        9
#define OPEN_VER_POS         10
#define FIX_VER_POS          11
#define CHANGE_LOG_TEXT_POS  12
#define SELECT_PR_DATA "\
SELECT pr.project_num, prj.name, \
       p.last_name||', '|| p.first_name||' ('||pr.submitter_id||')', \
       pr.responsible_id, pr.title, pr.descr, pr.fix, pr.severity_num, \
       pr.status_num, pt.name, pr.open_version, pr.close_version, \
       pr.change_log_text \
  FROM problem_report pr, project prj, person p, problem_type pt \
 WHERE pt.type_num = pr.type_num \
   AND p.login_id = pr.submitter_id \
   AND prj.project_num = pr.project_num \
   AND pr.problem_num = %d "

#define FULL_NAME_POS        4
#define SELECT_RESPONSIBLE "\
SELECT 0, 1, 2, r.login_id, \
       p.last_name||', '||p.first_name||' ('||r.login_id||')' \
  FROM responsible r, person p, problem_report pr \
 WHERE p.login_id = r.login_id \
   AND r.project_num = pr.project_num \
   AND pr.problem_num = %d \
 ORDER BY p.last_name, p.first_name, r.login_id  "

#define NAME_POS           0
#define PK_NUM_POS         1
#define ORDER_POS          2
#define SELECT_STATUSES "\
SELECT s.name, s.status_num, s.order_num \
  FROM status s, problem_report pr \
 WHERE (s.status_num = pr.status_num \
        OR s.active = TRUE) \
   AND pr.problem_num = %d \
 ORDER BY s.order_num "

#define SELECT_SEVERITIES "\
SELECT s.name, s.severity_num, s.order_num \
  FROM severity s, problem_report pr \
 WHERE (s.severity_num = pr.severity_num \
        OR s.active = TRUE) \
   AND pr.problem_num = %d \
 ORDER BY s.order_num "

#define VERSION_NUM_POS      1 
#define VERSION_TXT_POS      2 
#define SELECT_VERSIONS "\
SELECT pv.order_num, pv.version_num, pv.version_text \
  FROM project_version pv, problem_report pr \
 WHERE pv.active = TRUE \
   AND pv.project_num = pr.project_num \
   AND pr.problem_num = %d \
 UNION \
SELECT pv.order_num, pv.version_num, pv.version_text \
  FROM project_version pv, problem_report pr \
 WHERE pv. project_num = pr.project_num \
   AND pv.version_num = pr.close_version \
   AND pr.close_version is not NULL \
   AND pr.problem_num = %d \
 ORDER BY 1  "

#define SELECT_VER_TEXT "\
SELECT version_text, version_num \
  FROM project_version \
 WHERE version_num = %d \
   AND project_num = %d "

/*
 * The VERSION_NAME_PK_LIST stores the name/pk combinations that are
 * used in the FIX_VER_COMBO.
 */
#define VERSION_NAME_PK_LIST    "version_name_pk_list"

/*
 * 'Null' version display data (for the version related stuff).
 */
#define NULL_NAME           "<None>"
#define NULL_NAME_SIZE      6


static void 
draw_audit_trail (GtkText  *widget, 
                  PGconn   *conn,
                  gint      pr_pk)
{
   GString   *sql_buffer;
   PGresult  *res;
   gint       i, n;

   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, PR_SUBMIT_DATE_QUERY, pr_pk);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, "Getting open line"))
      {
      g_assert (PQntuples (res) == 1);
      gtk_text_insert (widget, NULL, NULL, NULL,
                       PQgetvalue (res, 0, 0), -1);
      gtk_text_insert (widget, NULL, NULL, NULL, "\n", -1);
      }
   PQclear (res);

   g_string_sprintf (sql_buffer, SELECT_AUDIT_TRAIL, pr_pk, pr_pk);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, "Getting Audit Trail Data"))
      {
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         gtk_text_insert (widget, NULL, NULL, NULL,
                          PQgetvalue (res, i, 0), -1);
         gtk_text_insert (widget, NULL, NULL, NULL, "\n", -1);
         }
      }

   g_string_free (sql_buffer, TRUE);
   PQclear (res);
}


static void 
on_pr_update_dlg_show (GtkWidget *widget, gpointer  user_data)
{
   GtkEntry         *project_entry;
   GtkEntry         *submitter_entry;
   GtkEntry         *open_ver_entry;
   GtkEntry         *fix_ver_entry;
   GtkCombo         *responsible_combo;
   GtkCombo         *fix_ver_combo;
   GtkEntry         *subject_entry;
   GtkCList         *status_clist;
   GtkEntry         *pr_type_entry;
   GtkCList         *severity_clist;
   GtkText          *descr_text;
   GtkText          *fix_text;
   GtkText          *change_log_text;
   GtkText          *au_tr_text;
   GtkLabel         *lbl;
   GString          *sql_buffer;
   GString          *resp_id;
   PGresult         *res;
   PGresult         *tmp_res;
   name_id_struct   *curr_resp;
   name_pk_struct   *curr_ver;
   GList            *name_list = NULL;
   GList            *full_list = NULL;
   GList            *list_iter = NULL;
   gboolean          in_error = FALSE;
   gint              i, n;
   gint              curr_pk;
   gint              curr_row;
   gint              prj_pk;
   gint              ver_pk;
   gint              txt_len;
   gchar            *list_buf[1];
   PGconn           *conn;
   gint              pk_num;
   GtkWidget        *msg_dlg;
   GString          *msg_str;

   /*
    * Get pointers to the widgets that we need to fill.
    */
   project_entry = GTK_ENTRY (get_widget (widget, PROJECT_ENTRY));
   submitter_entry = GTK_ENTRY (get_widget (widget, SUBMITTER_ENTRY));
   open_ver_entry = GTK_ENTRY (get_widget (widget, OPEN_VER_ENTRY));
   fix_ver_entry = GTK_ENTRY (get_widget (widget, FIX_VER_ENTRY));
   responsible_combo = GTK_COMBO (get_widget (widget, RESPONSIBLE_COMBO));
   fix_ver_combo = GTK_COMBO (get_widget (widget, FIX_VER_COMBO));
   subject_entry = GTK_ENTRY (get_widget (widget, SUBJECT_ENTRY));
   status_clist = GTK_CLIST (get_widget (widget, STATUS_CLIST));
   pr_type_entry = GTK_ENTRY (get_widget (widget, PROBLEM_TYPE_ENTRY));
   severity_clist = GTK_CLIST (get_widget (widget, SEVERITY_CLIST));
   descr_text = GTK_TEXT (get_widget (widget, DESCR_EDIT));
   fix_text = GTK_TEXT (get_widget (widget, FIX_DESCR_EDIT));
   change_log_text = GTK_TEXT (get_widget (widget, CHANGE_LOG_TEXT));
   au_tr_text = GTK_TEXT (get_widget (widget, AUDIT_TRAIL_EDIT));
   g_assert (project_entry != NULL);
   g_assert (submitter_entry != NULL);
   g_assert (responsible_combo != NULL);
   g_assert (fix_ver_combo != NULL);
   g_assert (subject_entry != NULL);
   g_assert (status_clist != NULL);
   g_assert (pr_type_entry != NULL);
   g_assert (severity_clist != NULL);
   g_assert (descr_text != NULL);
   g_assert (fix_text != NULL);
   g_assert (change_log_text != NULL);
   g_assert (au_tr_text != NULL);

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                  PK_NUM));
   sql_buffer = g_string_new ("");


   /*
    * Check the locking.
    */
   START_TRANSACTION (conn);
   g_string_sprintf (sql_buffer, CREATE_LOCK, pk_num);
   res = PQexec (conn, sql_buffer->str);
   in_error = !chk_sql_error (res, "Creating voluntary lock");
   if (in_error)
      {
      ROLLBACK (conn);
      }
   else
      {
      COMMIT (conn);
      }
   PQclear (res);

   if (!in_error)
      {
      g_string_sprintf (sql_buffer, SELECT_LOCKER, pk_num);
      res = PQexec (conn, sql_buffer->str);
      in_error = !chk_sql_error (res, "Checking the lock status");
      if (!in_error)
         {
         if (strcmp (PQuser (conn), PQgetvalue (res, 0, 0)))
            {
            msg_str = g_string_new ("");
            g_string_sprintf
               (msg_str,
                _("The selected problem report is locked by: %s\n"),
                PQgetvalue (res, 0, 1));
            msg_str = g_string_append
               (msg_str,
                _("The problem report will be opened for viewing only."));
            msg_dlg = gnome_message_box_new (msg_str->str,
                                             GNOME_MESSAGE_BOX_INFO,
                                             GNOME_STOCK_BUTTON_OK,
                                             NULL);
            gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
            gnome_dialog_run (GNOME_DIALOG (msg_dlg));
            gnome_dialog_set_sensitive (GNOME_DIALOG (widget),
                                        OK_BUTTON_POS,
                                        FALSE);
            gnome_dialog_set_sensitive (GNOME_DIALOG (widget),
                                        APPLY_BUTTON_POS,
                                        FALSE);
            gnome_dialog_set_default (GNOME_DIALOG (widget), CLOSE_BUTTON_POS);
            gtk_widget_set_sensitive (GTK_WIDGET (status_clist), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (severity_clist), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (responsible_combo), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (descr_text), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (fix_text), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (change_log_text), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (subject_entry), FALSE);
            gtk_widget_set_sensitive (GTK_WIDGET (fix_ver_combo), FALSE);
            }
         }
      PQclear (res);
      }
      
   /*
    * Fill in the status list box.
    */
   START_TRANSACTION (conn);
   if (!in_error)
      {
      g_string_sprintf (sql_buffer, SELECT_STATUSES, pk_num);
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Filling the statuses list"))
         {
         n = PQntuples (res);
         g_assert (n);
         for (i = 0; i < n; i++)
            {
            list_buf[0] = g_malloc
               (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
            strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
            gtk_clist_append (status_clist, list_buf);
            curr_pk = atoi (PQgetvalue (res, i, PK_NUM_POS));
            gtk_clist_set_row_data
               (status_clist, i, GINT_TO_POINTER (curr_pk));
            g_free (list_buf[0]);
            }
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * Fill in the severity clist.
    */
   if (!in_error)
      {
      g_string_sprintf (sql_buffer, SELECT_SEVERITIES, pk_num);
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Filling the severities list"))
         {
         n = PQntuples (res);
         for (i = 0; i < n; i++)
            {
            list_buf[0] = g_malloc 
               (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
            strcpy (list_buf[0], PQgetvalue (res, i, NAME_POS));
            gtk_clist_append (severity_clist, list_buf);
            curr_pk = atoi (PQgetvalue (res, i, PK_NUM_POS));
            gtk_clist_set_row_data 
               (severity_clist, i, GINT_TO_POINTER (curr_pk));
            g_free (list_buf[0]);
            }
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * Populate the responsible list, and attach the full list to the
    * combo so we can later retrieve the PK of the person chosen.
    */
   if (!in_error)
      {
      g_string_sprintf (sql_buffer, SELECT_RESPONSIBLE, pk_num);
      res = PQexec (conn, sql_buffer->str);
      if (!chk_sql_error (res, "Getting responsible list"))
         {      
         in_error = TRUE;
         }
      else
         {
         /* 
          * Get all of the names.  Create two lists.  name_list is used
          * to set the responsible combo's drop down list.  full_list
          * is attached to the combo so we can later trace the name
          * back to a login ID.
          *
          * first load the NOBODY person.
          */
         curr_resp = g_malloc (sizeof (name_id_struct));
         curr_resp->name = g_malloc (strlen (RESP_NOBODY_NAME) + 1);
         curr_resp->id = g_malloc (strlen (RESP_NOBODY_ID) + 1);
         strcpy (curr_resp->name, RESP_NOBODY_NAME);
         strcpy (curr_resp->id, RESP_NOBODY_ID);
         name_list = g_list_append (name_list, curr_resp->name);
         full_list = g_list_append (full_list, curr_resp);

         /* 
          * get all of the actual people
          */
         n = PQntuples (res);
         for (i = 0; i < n; i++)
            {
            curr_resp = g_malloc (sizeof (name_id_struct));
            curr_resp->id = g_malloc
               (strlen (PQgetvalue (res, i, RESPONSIBLE_ID_POS)) + 1);
            strcpy (curr_resp->id, PQgetvalue (res, i, RESPONSIBLE_ID_POS));
            curr_resp->name = g_malloc
               (strlen (PQgetvalue (res, i, FULL_NAME_POS)) + 1);
            strcpy (curr_resp->name, PQgetvalue (res, i, FULL_NAME_POS));
            name_list = g_list_append (name_list, curr_resp->name);
            full_list = g_list_append (full_list, curr_resp);
            }
         gtk_combo_set_popdown_strings (responsible_combo, name_list);
         /*
          * Store the full list with the responsible combo object so
          * we can later determine the ID of the selected person.
          */
         gtk_object_set_data (GTK_OBJECT (responsible_combo),
                              RESPONSIBLE_LIST,
                              full_list);
         g_list_free (name_list);
         } /* end else */
      PQclear (res);
      }

   /*
    * Fill in the "Fixed in Version" combo.
    */
   if (!in_error)
      {
      name_list = NULL;
      full_list = NULL;
      g_string_sprintf (sql_buffer, SELECT_VERSIONS, pk_num, pk_num);
      res = PQexec (conn, sql_buffer->str);
      if (!chk_sql_error (res, "Getting version list"))
         {      
         in_error = TRUE;
         }
      else
         {
         /*
          * start with the NULL version.
          */
         curr_ver = g_malloc (sizeof (name_pk_struct));
         curr_ver->name = g_malloc (NULL_NAME_SIZE + 1);
         strcpy (curr_ver->name, NULL_NAME);
         curr_ver->pk = INVALID_PK;
         name_list = g_list_append (name_list, curr_ver->name);
         full_list = g_list_append (full_list, curr_ver);

         /*
          * get the versions
          */
         n = PQntuples (res);
         for (i = 0; i < n; i++)
            {
            curr_ver = g_malloc (sizeof (name_pk_struct));
            curr_ver->name = g_malloc
               (strlen (PQgetvalue (res, i, VERSION_TXT_POS)) + 1);
            strcpy (curr_ver->name, PQgetvalue (res, i, VERSION_TXT_POS));
            curr_ver->pk = atoi (PQgetvalue (res, i, VERSION_NUM_POS));
            name_list = g_list_append (name_list, curr_ver->name);
            full_list = g_list_append (full_list, curr_ver);
            }
         gtk_combo_set_popdown_strings (fix_ver_combo, name_list);
         gtk_object_set_data (GTK_OBJECT (fix_ver_combo),
                              VERSION_NAME_PK_LIST,
                              full_list);
         g_list_free (name_list);
         }
      PQclear (res);
      }
   /*
    * Populate the other widgets.
    */
   if (!in_error)
      {
      g_string_sprintf (sql_buffer, SELECT_PR_DATA, pk_num);
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Populating PR Edit Dialog"))
         {
         g_assert (PQntuples (res) == 1);
         prj_pk = atoi (PQgetvalue (res, 0, PRJ_NUM_POS));
         gtk_entry_set_text (project_entry, 
                             PQgetvalue (res, 0, PRJ_NAME_POS)); 
         gtk_entry_set_text (submitter_entry, 
                             PQgetvalue (res, 0, SUBMITTER_ID_POS)); 
         gtk_entry_set_text (subject_entry, 
                             PQgetvalue (res, 0, TITLE_POS));
         gtk_entry_set_text (pr_type_entry,
                             PQgetvalue (res, 0, PROB_TYPE_POS));
         
         if (PQgetisnull (res, 0, OPEN_VER_POS))
            {
            gtk_entry_set_text (open_ver_entry, NULL_NAME);
            }
         else
            {
            ver_pk = atoi (PQgetvalue (res, 0, OPEN_VER_POS));
            g_string_sprintf (sql_buffer, SELECT_VER_TEXT, ver_pk, prj_pk);
            tmp_res = PQexec (conn, sql_buffer->str);
            g_assert (PQntuples (tmp_res) == 1);
            if (chk_sql_error (tmp_res, "Get open version text"))
               {
               gtk_entry_set_text (open_ver_entry, 
                                   PQgetvalue (tmp_res, 0, NAME_POS));
               }
            PQclear (tmp_res);
            }

         if (PQgetisnull (res, 0, FIX_VER_POS))
            {
            gtk_entry_set_text (fix_ver_entry, NULL_NAME);
            }
         else
            {
            ver_pk = atoi (PQgetvalue (res, 0, FIX_VER_POS));
            g_string_sprintf (sql_buffer, SELECT_VER_TEXT, ver_pk, prj_pk);
            tmp_res = PQexec (conn, sql_buffer->str);
            g_assert (PQntuples (tmp_res) == 1);
            if (chk_sql_error (tmp_res, "Get close version text"))
               {
               gtk_entry_set_text (fix_ver_entry,
                                   PQgetvalue (tmp_res, 0, NAME_POS));
               }
            PQclear (tmp_res);
            }

         gtk_text_insert (descr_text, NULL, NULL, NULL,
                          PQgetvalue (res, 0, DESCR_POS), -1);
         gtk_text_insert (fix_text, NULL, NULL, NULL,
                          PQgetvalue (res, 0, FIX_POS), -1);
         gtk_text_insert (change_log_text, NULL, NULL, NULL,
                          PQgetvalue (res, 0, CHANGE_LOG_TEXT_POS), -1);
         draw_audit_trail (au_tr_text, conn, pk_num);

         lbl = GTK_LABEL (get_widget (widget, FIX_TAB_LBL));
         txt_len = gtk_text_get_length (fix_text) +
            gtk_text_get_length (change_log_text);
         if (txt_len == 0)
            {
            gtk_label_set_text (lbl, FIX_DESCR);
            }
         else
            {
            gtk_label_set_text (lbl, FIX_DESCR_STAR);
            }
         gtk_object_set_data (GTK_OBJECT (widget),
                              FIX_TAB_CHAR_CNT,
                              GINT_TO_POINTER (txt_len));

         curr_pk = atoi (PQgetvalue (res, 0, STATUS_NUM_POS));
         curr_row = gtk_clist_find_row_from_data 
            (status_clist, GINT_TO_POINTER (curr_pk));
         gtk_clist_select_row (status_clist, curr_row, 0);
         gtk_clist_moveto (status_clist, curr_row, 0, 0.5, 0.0);

         curr_pk = atoi (PQgetvalue (res, 0, SEVERITY_NUM_POS));
         curr_row = gtk_clist_find_row_from_data
            (severity_clist, GINT_TO_POINTER (curr_pk));
         gtk_clist_select_row (severity_clist, curr_row, 0);
         gtk_clist_moveto (severity_clist, curr_row, 0, 0.5, 0.0);

         resp_id = g_string_new (PQgetvalue (res, 0, RESPONSIBLE_ID_POS));
         full_list = gtk_object_get_data (GTK_OBJECT (responsible_combo),
                                          RESPONSIBLE_LIST);
         list_iter = g_list_first (full_list);
         while (list_iter &&
                strcmp (resp_id->str, 
                        ((name_id_struct *)list_iter->data)->id) != 0)
            {
            list_iter = list_iter->next;
            }
         /*
          * If we did not find the ID, use the first one in the list
          * which should be the NOBODY string.
          */
         if (!list_iter)
            {
            list_iter = g_list_first (full_list);
            }
         gtk_entry_set_text (GTK_ENTRY (responsible_combo->entry),
                             ((name_id_struct *)list_iter->data)->name);
         g_string_free (resp_id, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * If the current user is in the submitter list, make the description
    * editable.  If the current user is in the reponsible list, make the
    * responsible list pickable and the fix description editable.
    */ 

   /*
    * If an error occurred along the way, destroy the dialog box.
    */
   if (in_error)
      {
      ROLLBACK (conn);
      gtk_widget_destroy (widget);
      }
   else
      {
      COMMIT (conn);
      set_all_clean (widget);
      gnome_dialog_set_sensitive (GNOME_DIALOG (widget),
                                  APPLY_BUTTON_POS, FALSE);
      }
   g_string_free (sql_buffer, TRUE);
}


static void 
on_pr_update_dlg_destroy (GtkObject *object, gpointer  user_data)
{
   GtkWidget *main_widget;
   GtkCombo  *combo;
   GList     *temp_list;
   PGresult  *res;
   GString   *sql_buffer;
   PGconn    *conn;
   gint       pk_num;

   main_widget = GTK_WIDGET (object);

   conn = gtk_object_get_data (GTK_OBJECT (main_widget), MY_DB_CONN);
   g_assert (conn != NULL);
   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_widget),
                                                  PK_NUM));

   sql_buffer = g_string_new ("");
   START_TRANSACTION (conn);
   g_string_sprintf (sql_buffer, REMOVE_LOCK, pk_num);
   res = PQexec (conn, sql_buffer->str);
   chk_sql_error (res, "Removing lock");
   COMMIT (conn);
   PQclear (res);
   g_string_free (sql_buffer, TRUE);

   /*
    * Free up the full list attached to the responsible_clist if that
    * list exists (if the problem report was locked by another user,
    * the responsible list will not even have been filled).
    */
   combo = GTK_COMBO (get_widget (main_widget, RESPONSIBLE_COMBO));
   g_assert (combo != NULL);
   temp_list = gtk_object_get_data (GTK_OBJECT (combo), RESPONSIBLE_LIST);
   destroy_name_id_list (temp_list);

   combo = GTK_COMBO (get_widget (main_widget, FIX_VER_COMBO));
   g_assert (combo != NULL);
   temp_list = gtk_object_get_data (GTK_OBJECT (combo), VERSION_NAME_PK_LIST);
   destroy_name_pk_list (temp_list);

   destroy_dirty_list (main_widget);
}


static gboolean 
apply_changes (GnomeDialog *dlg)
{
   GtkWidget         *main_widget;
   GtkWidget         *parent;  /* main application window */
   GtkCombo          *responsible_combo;
   GtkEntry          *subject_entry;
   GtkCombo          *ver_combo;
   GtkEntry          *ver_entry;
   GList             *ver_list;
   GtkCList          *status_clist;
   GtkCList          *severity_clist;
   GtkText           *descr_text;
   GtkText           *fix_text;
   GtkText           *change_log_text;
   GList             *resp_list;
   GString           *tmp_str;
   GString           *sql_buffer;
   PGresult          *res;
   gint               row_idx;
   gint               status_pk = INVALID_PK;
   gint               severity_pk = INVALID_PK;
   gint               ver_pk = INVALID_PK;
   gboolean           in_error = FALSE;
   gboolean           first = TRUE;
   gchar              num_buffer[NUM_BUFFER_LEN];
   gchar             *char_buf;
   PGconn            *conn;
   gint               pk_num;
   GtkWidget         *msg_dlg;

   main_widget = GTK_WIDGET (dlg);
   conn = gtk_object_get_data (GTK_OBJECT (main_widget), MY_DB_CONN);
   g_assert (conn != NULL);
   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_widget),
                                                  PK_NUM));
   parent = get_widget (main_widget, MY_PARENT);

   /*
    * Get pointers to all the widgets we will need to access.
    */
   responsible_combo = 
      GTK_COMBO (get_widget (main_widget, RESPONSIBLE_COMBO));
   resp_list = gtk_object_get_data (GTK_OBJECT (responsible_combo), 
                                    RESPONSIBLE_LIST);
   subject_entry = 
      GTK_ENTRY (get_widget (main_widget, SUBJECT_ENTRY));
   status_clist = 
      GTK_CLIST (get_widget (main_widget, STATUS_CLIST));
   severity_clist =
      GTK_CLIST (get_widget (main_widget, SEVERITY_CLIST));
   descr_text = GTK_TEXT (get_widget (main_widget, DESCR_EDIT));
   fix_text = GTK_TEXT (get_widget (main_widget, FIX_DESCR_EDIT)); 
   change_log_text = GTK_TEXT (get_widget (main_widget, CHANGE_LOG_TEXT)); 
   ver_combo = GTK_COMBO (get_widget (main_widget, FIX_VER_COMBO));
   ver_entry = GTK_ENTRY (get_widget (main_widget, FIX_VER_ENTRY));
   ver_list = gtk_object_get_data (GTK_OBJECT (ver_combo), 
                                   VERSION_NAME_PK_LIST);

   /*
    * Round up all of the information that we will need.
    *
    * Do validations as needed.
    */
   if (widgets_children_are_dirty (main_widget))
      {
      sql_buffer = g_string_new ("UPDATE PROBLEM_REPORT SET ");
      if (widget_is_dirty (GTK_WIDGET (responsible_combo->entry)))
         {
         if (!first)
            {
            sql_buffer = g_string_append (sql_buffer, ", ");
            }
         first = FALSE;
         tmp_str = g_string_new 
            (find_id_from_name (resp_list,
                                gtk_entry_get_text 
                                   (GTK_ENTRY (responsible_combo->entry))));
         g_string_prepare_db_instr (tmp_str);
         sql_buffer = g_string_append (sql_buffer, "responsible_id = ");
         sql_buffer = g_string_append (sql_buffer, tmp_str->str);
         g_string_free (tmp_str, TRUE);
         }

      if (widget_is_dirty (GTK_WIDGET (subject_entry)))
         {
         if (!first)
            {
            sql_buffer = g_string_append (sql_buffer, ", ");
            }
         first = FALSE;
         tmp_str = g_string_new (gtk_entry_get_text (subject_entry));
         g_string_prepare_db_instr (tmp_str);
         sql_buffer = g_string_append (sql_buffer, "title = ");
         sql_buffer = g_string_append (sql_buffer, tmp_str->str);
         g_string_free (tmp_str, TRUE);
         }
   
      if (widget_is_dirty (GTK_WIDGET (descr_text)))
         {
         if (!first)
            {
            sql_buffer = g_string_append (sql_buffer, ", ");
            }
         first = FALSE;
         char_buf = gtk_editable_get_chars (GTK_EDITABLE (descr_text), 0, -1);
         tmp_str = g_string_new (char_buf);
         g_free (char_buf);
         g_string_prepare_db_instr (tmp_str);
         sql_buffer = g_string_append (sql_buffer, "descr = ");
         sql_buffer = g_string_append (sql_buffer, tmp_str->str);
         g_string_free (tmp_str, TRUE);
         }

      if (widget_is_dirty (GTK_WIDGET (fix_text)))
         {
         if (!first)
            {
            sql_buffer = g_string_append (sql_buffer, ", ");
            }
         first = FALSE;
         char_buf = gtk_editable_get_chars (GTK_EDITABLE (fix_text), 0, -1); 
         tmp_str = g_string_new (char_buf);
         g_free (char_buf);
         g_string_prepare_db_instr (tmp_str);
         sql_buffer = g_string_append (sql_buffer, "fix = ");
         sql_buffer = g_string_append (sql_buffer, tmp_str->str);
         g_string_free (tmp_str, TRUE);
         }

      if (widget_is_dirty (GTK_WIDGET (change_log_text)))
         {
         if (!first)
            {
            sql_buffer = g_string_append (sql_buffer, ", ");
            }
         first = FALSE;
         char_buf = gtk_editable_get_chars 
            (GTK_EDITABLE (change_log_text), 0, -1);
         tmp_str = g_string_new (char_buf);
         g_free (char_buf);
         g_string_prepare_db_instr (tmp_str);
         sql_buffer = g_string_append (sql_buffer, "change_log_text = ");
         sql_buffer = g_string_append (sql_buffer, tmp_str->str);
         g_string_free (tmp_str, TRUE);
         }

      if (widget_is_dirty (GTK_WIDGET (ver_entry)))
         {
         if (!first)
            {
            sql_buffer = g_string_append (sql_buffer, ", ");
            }
         first = FALSE;
         ver_pk = find_pk_from_name (ver_list, 
                                     gtk_entry_get_text (ver_entry));
         sql_buffer = g_string_append (sql_buffer, "close_version = ");
         if (ver_pk == INVALID_PK)
            {
            sql_buffer = g_string_append (sql_buffer, "NULL");
            }
         else
            {
            sprintf (num_buffer, "%d", ver_pk);
            sql_buffer = g_string_append (sql_buffer, num_buffer);
            }
         }

      if (widget_is_dirty (GTK_WIDGET (status_clist)))
         {
         if (status_clist->selection)
            {
            row_idx = GPOINTER_TO_INT (status_clist->selection->data);
            status_pk = GPOINTER_TO_INT (gtk_clist_get_row_data (status_clist,
                                                                 row_idx));
            if (!first)
               {
               sql_buffer = g_string_append (sql_buffer, ", ");
               }
            first = FALSE;
            sprintf (num_buffer, "%d", status_pk);
            sql_buffer = g_string_append (sql_buffer, "status_num = ");
            sql_buffer = g_string_append (sql_buffer, num_buffer);
            }
         else
            {
            msg_dlg = gnome_message_box_new
                (_("You must select a status."),
                GNOME_MESSAGE_BOX_ERROR,
                GNOME_STOCK_BUTTON_OK,
                NULL);
            gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
            gnome_dialog_run (GNOME_DIALOG (msg_dlg));
            in_error = TRUE;
            }
         }

      if (widget_is_dirty (GTK_WIDGET (severity_clist)) && !in_error)
         {
         if (severity_clist->selection)
            {
            row_idx = GPOINTER_TO_INT (severity_clist->selection->data);
            severity_pk = GPOINTER_TO_INT 
               (gtk_clist_get_row_data (severity_clist, row_idx));
            if (!first)
               {
               sql_buffer = g_string_append (sql_buffer, ", ");
               }
            first = FALSE;
            sprintf (num_buffer, "%d", severity_pk);
            sql_buffer = g_string_append (sql_buffer, "severity_num = ");
            sql_buffer = g_string_append (sql_buffer, num_buffer);
            }
         else
            {
            msg_dlg = gnome_message_box_new
               (_("You must select a severity."),
                GNOME_MESSAGE_BOX_ERROR,
                GNOME_STOCK_BUTTON_OK,
                NULL);
            gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
            gnome_dialog_run (GNOME_DIALOG (msg_dlg));
            in_error = TRUE;
            }
         }

      /* 
       * Update the database.
       */
      if (!in_error)
         {
         /*
          * If we are here, their ought to be at least one thing that
          * we are updating...
          */
         g_assert (!first);
         START_TRANSACTION (conn);
         sprintf (num_buffer, "%d", pk_num);
         sql_buffer = g_string_append (sql_buffer, " WHERE problem_num = ");
         sql_buffer = g_string_append (sql_buffer, num_buffer);
         res = PQexec (conn, sql_buffer->str);

         in_error = !chk_sql_error (res, "Updating Problem Report");
         PQclear (res);
         if (!in_error)
            {
            COMMIT (conn);
            refresh_pr_list (parent);
            }
         else
            {
            ROLLBACK (conn);
            }
         }
      g_string_free (sql_buffer, TRUE);
      }

   return !in_error;
}


static void
on_ok_button_clicked (GnomeDialog *dlg)
{
   if (apply_changes (dlg))
      {
      save_dlg_size (GTK_WIDGET (dlg), PROB_UPD_SEC, FALSE);
      gnome_dialog_close (dlg);
      }
}


static void
on_apply_button_clicked (GnomeDialog *dlg)
{
   apply_changes (dlg);
   gnome_dialog_set_sensitive (dlg, APPLY_BUTTON_POS, FALSE);
}


static void 
on_close_button_clicked (GnomeDialog *dlg)
{
   save_dlg_size (GTK_WIDGET (dlg), PROB_UPD_SEC, FALSE);
   gnome_dialog_close (dlg);
}

static void
on_pr_update_dlg_clicked (GnomeDialog *dlg,
                          gint         btn_cd,
                          gpointer     user_data)
{
   switch (btn_cd)
      {
      case OK_BUTTON_POS:
         on_ok_button_clicked (dlg);
         break;
      case APPLY_BUTTON_POS:
         on_apply_button_clicked (dlg);
         break;
      case CLOSE_BUTTON_POS:
         on_close_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached ();
      }
}


static void
check_fix_tab_text (GtkWidget *widget, gpointer user_data)
{
   GtkWidget *main_win = GTK_WIDGET (user_data);
   GtkText   *fix_text;
   GtkText   *chlog_text;
   GtkLabel  *tab_lbl;
   gint       curr_len;
   gint       stored_len;

   fix_text = GTK_TEXT (get_widget (main_win, FIX_DESCR_EDIT));
   chlog_text = GTK_TEXT (get_widget (main_win, CHANGE_LOG_TEXT));
   tab_lbl = GTK_LABEL (get_widget (main_win, FIX_TAB_LBL));
   g_assert (fix_text != NULL);
   g_assert (chlog_text != NULL);
   g_assert (tab_lbl != NULL);

   curr_len = gtk_text_get_length (fix_text) +
      gtk_text_get_length (chlog_text);
   if (curr_len == 0)
      {
      gtk_label_set_text (tab_lbl, FIX_DESCR);
      } 
   else
      {
      stored_len = GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (main_win), FIX_TAB_CHAR_CNT));
      if (!stored_len)
         {
         gtk_label_set_text (tab_lbl, FIX_DESCR_STAR);
         }
      }

   gtk_object_set_data (GTK_OBJECT (main_win),
                        FIX_TAB_CHAR_CNT,
                        GINT_TO_POINTER (curr_len));
}


static void
handle_edit_change (GtkWidget *widget, gpointer user_data)
{
   set_dirty (widget, NULL);
   gnome_dialog_set_sensitive (GNOME_DIALOG (user_data), 1, TRUE);
}


static void
handle_clist_sel_change (GtkWidget      *widget,
                         gint            row,
                         gint            column,
                         GdkEventButton *event,
                         gpointer        user_data)
{
   set_dirty (widget, NULL);
   gnome_dialog_set_sensitive (GNOME_DIALOG (user_data), 1, TRUE);
}


GtkWidget*
create_problem_update_dlg (GtkWidget  *parent,
                           PGconn     *conn,
                           gint        pr_pk)
{
  GtkWidget         *pr_update_dlg;
  GtkWidget         *dialog_vbox1;
  GtkWidget         *vbox1;
  GtkWidget         *vbox2;
  GtkWidget         *open_ver_entry;
  GtkWidget         *fix_ver_combo;
  GtkWidget         *fix_ver_entry;
  GtkWidget         *label;
  GtkWidget         *project_entry;
  GtkWidget         *submitter_entry;
  GtkWidget         *responsible_combo;
  GtkWidget         *subject_entry;
  GtkWidget         *table1;
  GtkWidget         *status_clist;
  GtkWidget         *prob_type_entry;
  GtkWidget         *severity_clist;
  GtkWidget         *notebook1;
  GtkWidget         *scrolledwindow1;
  GtkWidget         *descr_text;
  GtkWidget         *scrolledwindow2;
  GtkWidget         *fix_text;
  GtkWidget         *change_log_text;
  GtkWidget         *scrolledwindow3;
  GtkWidget         *scrolledwindow4;
  GtkWidget         *scrolledwindow6;
  GtkWidget         *scrolledwindow7;
  GtkWidget         *audit_trail_text;
  GtkWidget         *popup;
  GString           *window_title;

  window_title = g_string_new ("");
  g_string_sprintf (window_title, _("Edit Problem Report #%d"), pr_pk);
  pr_update_dlg = gnome_dialog_new (window_title->str,
                                    GNOME_STOCK_BUTTON_OK,
                                    GNOME_STOCK_BUTTON_APPLY,
                                    GNOME_STOCK_BUTTON_CLOSE,
                                    NULL);
  g_string_free (window_title, TRUE);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       "pr_update_dlg", 
                       pr_update_dlg);
  gtk_container_border_width (GTK_CONTAINER (pr_update_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (pr_update_dlg), "show",
                      GTK_SIGNAL_FUNC (on_pr_update_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (pr_update_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_pr_update_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (pr_update_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_pr_update_dlg_clicked),
                      NULL);
  gtk_window_set_policy (GTK_WINDOW (pr_update_dlg), FALSE, TRUE, FALSE);
  gnome_dialog_set_default (GNOME_DIALOG (pr_update_dlg), 0);
  gnome_dialog_set_close (GNOME_DIALOG (pr_update_dlg), FALSE);
  gnome_dialog_set_parent (GNOME_DIALOG (pr_update_dlg),
                           GTK_WINDOW (parent));

  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), TOP_DOG, pr_update_dlg);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), PK_NUM,
                       GINT_TO_POINTER (pr_pk));
  set_dlg_size (pr_update_dlg, PROB_UPD_SEC, 625, 500);

  dialog_vbox1 = GNOME_DIALOG (pr_update_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox1), 5);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 5);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  table1 = gtk_table_new (10, 4, TRUE);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), "table1", table1);
  gtk_widget_set_usize (table1, 600, -2);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, FALSE, FALSE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

  label = gtk_label_new (_("Project"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  project_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       PROJECT_ENTRY, 
                       project_entry);
  gtk_widget_show (project_entry);
  gtk_table_attach (GTK_TABLE (table1), project_entry, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_entry_set_editable (GTK_ENTRY (project_entry), FALSE);
  gtk_widget_set_sensitive (project_entry, FALSE);

  label = gtk_label_new (_("Reported in Version"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 2, 3, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  open_ver_entry = gtk_entry_new ();
  gtk_widget_ref (open_ver_entry);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), 
                            OPEN_VER_ENTRY, 
                            open_ver_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (open_ver_entry);
  gtk_table_attach (GTK_TABLE (table1), open_ver_entry, 2, 3, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_entry_set_editable (GTK_ENTRY (open_ver_entry), FALSE);
  gtk_widget_set_sensitive (open_ver_entry, FALSE);

  label = gtk_label_new (_("Fixed In Version:"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 3, 4, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  fix_ver_combo = gtk_combo_new ();
  gtk_widget_ref (fix_ver_combo);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg),
                            FIX_VER_COMBO,
                            fix_ver_combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fix_ver_combo);
  gtk_table_attach (GTK_TABLE (table1), fix_ver_combo, 3, 4, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  fix_ver_entry = GTK_COMBO (fix_ver_combo)->entry;
  gtk_widget_ref (fix_ver_entry);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg),
                            FIX_VER_ENTRY,
                            fix_ver_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fix_ver_entry);
  gtk_entry_set_editable (GTK_ENTRY (fix_ver_entry), FALSE);
  gtk_signal_connect (GTK_OBJECT (fix_ver_entry), "changed",
                      GTK_SIGNAL_FUNC (handle_edit_change),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg, fix_ver_entry);

  label = gtk_label_new (_("Submitted By"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  submitter_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       SUBMITTER_ENTRY, 
                       submitter_entry);
  gtk_widget_show (submitter_entry);
  gtk_table_attach (GTK_TABLE (table1), submitter_entry, 0, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_entry_set_editable (GTK_ENTRY (submitter_entry), FALSE);
  gtk_widget_set_sensitive (submitter_entry, FALSE);

  label = gtk_label_new (_("Assigned To"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 2, 4, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  responsible_combo = gtk_combo_new ();
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       RESPONSIBLE_COMBO, 
                       responsible_combo);
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (responsible_combo)->entry),
                          FALSE);
  gtk_widget_show (responsible_combo);
  gtk_table_attach (GTK_TABLE (table1), responsible_combo, 2, 4, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (responsible_combo)->entry),
                      "changed", GTK_SIGNAL_FUNC (handle_edit_change),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg,
                            GTK_COMBO (responsible_combo)->entry);

  label = gtk_label_new (_("Subject"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 4, 4, 5,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  subject_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       SUBJECT_ENTRY, 
                       subject_entry);
  gtk_widget_show (subject_entry);
  gtk_table_attach (GTK_TABLE (table1), subject_entry, 0, 4, 5, 6,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (subject_entry), "changed",
                      GTK_SIGNAL_FUNC (handle_edit_change),
                      pr_update_dlg);
  popup = attach_generic_edit_popup_menu (subject_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), "popup1", popup,
                            (GtkDestroyNotify) gtk_widget_unref);
  add_widget_to_dirty_list (pr_update_dlg, subject_entry);

  label = gtk_label_new (_("Status"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 2, 6, 7,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Problem Type"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  prob_type_entry = gtk_entry_new ();
  gtk_widget_ref (prob_type_entry);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), 
                            PROBLEM_TYPE_ENTRY, 
                            prob_type_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prob_type_entry);
  gtk_table_attach (GTK_TABLE (table1), prob_type_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_entry_set_editable (GTK_ENTRY (prob_type_entry), FALSE);
  gtk_widget_set_sensitive (prob_type_entry, FALSE);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow4);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow4, 
                    0, 2, CLIST_START, CLIST_END,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);
  
  status_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       STATUS_CLIST,
                       status_clist);
  gtk_widget_show (status_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), status_clist);
  gtk_clist_set_column_width (GTK_CLIST (status_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (status_clist));
  gtk_signal_connect (GTK_OBJECT (status_clist), "select_row",
                      GTK_SIGNAL_FUNC (handle_clist_sel_change),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg, status_clist);

  label = gtk_label_new (_("Severity"));
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 2, 4, 6, 7,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow6);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow6, 
                    2, 4, CLIST_START, CLIST_END,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6),
                                  GTK_POLICY_NEVER,
                                  GTK_POLICY_AUTOMATIC);

  severity_clist = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       SEVERITY_CLIST, 
                       severity_clist);
  gtk_widget_show (severity_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), severity_clist);
  gtk_clist_set_column_width (GTK_CLIST (severity_clist), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (severity_clist));
  gtk_signal_connect (GTK_OBJECT (severity_clist), "select_row",
                      GTK_SIGNAL_FUNC (handle_clist_sel_change),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg, severity_clist);

  notebook1 = gtk_notebook_new ();
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       "notebook1", 
                       notebook1);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook1, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), 
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  descr_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), DESCR_EDIT, descr_text);
  gtk_widget_show (descr_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), descr_text);
  gtk_widget_set_usize (descr_text, -2, 150);
  gtk_text_set_editable (GTK_TEXT (descr_text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (descr_text), TRUE);
  popup = attach_generic_edit_popup_menu (descr_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), "popup2", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), 
                            "vbox2",
                            vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox2);
  gtk_widget_show (vbox2);
  gtk_signal_connect (GTK_OBJECT (descr_text), "changed",
                      GTK_SIGNAL_FUNC (handle_edit_change),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg, descr_text);

  label = gtk_label_new (_("Detailed Fix Description"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg),
                            "dtl_fix_descr_label",
                            label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow2, TRUE, TRUE, 0);
  gtk_widget_show (scrolledwindow2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  fix_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), FIX_DESCR_EDIT, fix_text);
  gtk_widget_show (fix_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), fix_text);
  gtk_text_set_editable (GTK_TEXT (fix_text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (fix_text), TRUE);
  gtk_signal_connect (GTK_OBJECT (fix_text), "changed",
                      GTK_SIGNAL_FUNC (handle_edit_change),
                      pr_update_dlg);
  gtk_signal_connect (GTK_OBJECT (fix_text), "changed",
                      GTK_SIGNAL_FUNC (check_fix_tab_text),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg, fix_text);
  popup = attach_generic_edit_popup_menu (fix_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), "popup3", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  label = gtk_label_new (_("ChangeLog Description"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg),
                            "change_log_descr_label",
                            label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow7);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg),
                            "scrolledwindow7",
                            scrolledwindow7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow7, TRUE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow7, -2, 20);
  gtk_widget_show (scrolledwindow7);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  change_log_text = gtk_text_new (NULL, NULL);
  gtk_widget_ref (change_log_text);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), 
                            CHANGE_LOG_TEXT, 
                            change_log_text,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (change_log_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), change_log_text);
  gtk_text_set_editable (GTK_TEXT (change_log_text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (change_log_text), TRUE);
  gtk_signal_connect (GTK_OBJECT (change_log_text), "changed",
                      GTK_SIGNAL_FUNC (handle_edit_change),
                      pr_update_dlg);
  gtk_signal_connect (GTK_OBJECT (change_log_text), "changed",
                      GTK_SIGNAL_FUNC (check_fix_tab_text),
                      pr_update_dlg);
  add_widget_to_dirty_list (pr_update_dlg, change_log_text);
  popup = attach_generic_edit_popup_menu (change_log_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), "popup4", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  audit_trail_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (pr_update_dlg), 
                       AUDIT_TRAIL_EDIT, 
                       audit_trail_text);
  gtk_widget_show (audit_trail_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), audit_trail_text);

  label = gtk_label_new (_("Description"));
  gtk_widget_show (label);
  set_notebook_tab (notebook1, 0, label);

  label = gtk_label_new (_("Fix Description"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (pr_update_dlg), FIX_TAB_LBL, label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  set_notebook_tab (notebook1, 1, label);

  label = gtk_label_new (_("Audit Trail"));
  gtk_widget_show (label);
  set_notebook_tab (notebook1, 2, label);

  return pr_update_dlg;
}
