/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * project_dlg
 *
 * Synopsis:
 *   Handle the project listing and editing dialog boxes.
 *
 * $Id: project_dlg.c,v 1.53 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <ctype.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gnome.h>
#include <libpq-fe.h>

#include <preps/gstr_utils.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "prepsmain.h"
#include "project_dlg.h"
#include "util.h"

/*
 *  Define the names of the significant widgets
 */
#define PROJECT_DIALOG      "project_dlg"
#define PROJECT_LIST        "project_clist"
#define ADD_BUTTON          "add_button"
#define MOD_BUTTON          "modify_button"
#define DEL_BUTTON          "delete_button"
#define DISMISS_BUTTON      "dismiss_button"
#define OK_BUTTON           "ok_button"
#define APPLY_BUTTON        "apply_button"
#define CANCEL_BUTTON       "cancel_button"
#define SHOW_INACTIVE_CB    "show_inactive_cb"
#define PROJECT_EDIT_DIALOG "project_edit_dlg"
#define NAME_ENTRY          "name_entry"
#define ACTIVE_CB           "active_cb"
#define DESCR_TEXT          "descr_text"
#define LEADER_COMBO        "leader_combo"
#define SUBMITTER_LIST      "submitter_clist"
#define SUB_ADD_ALL_BTN     "sub_add_all_btn"
#define SUB_RM_ALL_BTN      "sub_rm_all_btn"
#define PRJ_MEMBER_LIST     "project_member_clist"
#define PRJ_MEM_ADD_ALL_BTN "prjmem_add_all_btn"
#define PRJ_MEM_RM_ALL_BTN  "prjmem_rm_all_btn"
#define LEADER_LIST         "leader_list"
#define NEW_PR_NOTIFY_GRP   "new_pr_notify_grp"
#define NEW_PR_NOTIFY_PL    "new_pr_notify_pl"
#define ASSIGN_NOTIFY_RESP  "assign_notify_resp"
#define ASSIGN_NOTIFY_SUB   "assign_notify_sub"
#define ASSIGN_NOTIFY_PL    "assign_notify_pl"
#define UPD_NOTIFY_GRP      "upd_notify_grp"
#define UPD_NOTIFY_PL       "upd_notify_pl"
#define UPD_NOTIFY_RESP     "upd_notify_resp"
#define UPD_NOTIFY_SUB      "upd_notify_sub"
#define AUTO_ASSIGN_COMBO   "auto_assign_combo"
#define ADD_VER_BUTTON      "add_version_btn"
#define MOD_VER_BUTTON      "mod_version_btn"
#define DEL_VER_BUTTON      "del_version_btn"
#define DEF_OPEN_COMBO      "def_open_combo"
#define DEF_OPEN_ENTRY      "def_open_entry"
#define DEF_CLOSE_COMBO     "def_close_combo"
#define DEF_CLOSE_ENTRY     "def_close_entry"
#define SHOW_ALL_VER_CB     "show_all_ver_cb"
#define VER_CLIST           "version_clist"
#define VER_ACTIVE_CB       "ver_active_cb"
#define VER_ENTRY           "version_entry"
#define VER_DESCR_ENTRY     "version_descr_entry"
#define NAME_LIST           "name_list"
#define FULL_LIST           "full_list"

#define OK_BTN              0
#define APPLY_BTN           1
#define CANCEL_BTN          2

/*
 * 'Null' version display data (for the combo boxes).
 */
#define NULL_NAME           _("<None>")
#define NULL_NAME_SIZE      6

/*
 * Keys to store "add" and "remove" lists with the SUBMITTER_LIST and
 * PRJ_MEMBER_LIST.  The add and remove lists keep track of who needs 
 * to be added and removed from the correstponding tables in the database.
 */
#define ADD_LIST            "add_list"
#define RM_LIST             "rm_list"

/*
 * Keys to store the version information in linked lists for various
 * purposes. 
 *
 * VERSION_LIST : List of full version information stored in the database
 *                and displayed in VER_CLIST.  This list is then processed
 *                to update the database when the dialog closes.
 * VERSION_NAME_PK_LIST : Version name/pk pairings used in the default
 *                        open/close version combos.
 *
 * Both of these lists are attached to the VER_CLIST widget.
 */
#define VERSION_LIST          "ver_list"
#define VERSION_NAME_PK_LIST  "ver_name_pk_list"

/*
 * Column positions within certain clists.
 */
#define LOGIN_ID_COL            1

/*
 *  This is the order in the main SELECT.  If the order changes in 
 *  read_project_data, these must change to match it.
 *  
 *  In order for stuff to work right, these queries MUST select
 *  stuff back in the same order.  This helps make the extraction
 *  functions more generic.
 */
#define PK_NUM_POS              0
#define ID_POS                  0
#define NAME_POS                1
#define ACTIVE_POS              2
#define LOGIN_ID_POS            3
#define DESCR_POS               4
#define LAST_NAME_POS           5
#define FIRST_NAME_POS          6
#define NEW_PR_GRP_POS          5
#define NEW_PR_PL_POS           6
#define ASSIGN_RESP_POS         7
#define ASSIGN_SUB_POS          8
#define ASSIGN_PL_POS           9
#define UPD_GRP_POS             10
#define UPD_PL_POS              11
#define UPD_RESP_POS            12
#define UPD_SUB_POS             13
#define OPEN_VER_POS            14
#define CLOSE_VER_POS           15
#define AUTO_ASSIGN_POS         16
#define ORDER_NUM_POS           3
#define PROJECT_QUERY "SELECT project_num, name, active, login_id, \
                              descr, new_pr_notify_grp, new_pr_notify_pl, \
                              assign_notify_resp, assign_notify_sub, \
                              assign_notify_pl, upd_notify_grp, \
                              upd_notify_pl, upd_notify_resp, upd_notify_sub,\
                              open_version, close_version, \
                              auto_assign_login_id \
                         FROM project \
                        WHERE project_num = %d "

#define PERSON_QUERY "SELECT 0, 1, active, login_id, 4, last_name, \
                             first_name \
                        FROM person \
                       ORDER BY last_name, first_name, login_id "

#define VERSION_QUERY "SELECT version_num, version_text, active, \
                              order_num, descr \
                         FROM project_version \
                        WHERE project_num = %d \
                        ORDER BY order_num"      


/*
 *  Various queries used to insert, update, and remove data.
 */
#define RM_RESPONSIBLE_SQL "DELETE \
                              FROM responsible \
                             WHERE project_num = %d \
                               AND login_id = '%s'"
                               
#define RM_SUBMITTER_SQL "DELETE \
                            FROM submitter \
                           WHERE project_num = %d \
                             AND login_id = '%s'"
                             
#define INS_RESPONSIBLE_SQL "INSERT \
                               INTO responsible (project_num, login_id) \
                             VALUES (%d, '%s')"

#define INS_SUBMITTER_SQL "INSERT \
                             INTO submitter (project_num, login_id) \
                           VALUES (%d, '%s')"

#define RESP_LIST_SQL "\
SELECT distinct r.login_id, \
       p.last_name || ', ' || p.first_name || ' (' || p.login_id || ')' \
  FROM responsible r, person p \
 WHERE p.login_id = r.login_id \
   AND r.project_num = %d \
 ORDER BY 2"

#define GET_USER_FULL_NAME "\
SELECT last_name || ', ' || first_name || ' (' || login_id || ')' \
  FROM person \
 WHERE login_id = '%s'"

#define UPDATE_AUTO_ASSIGN "\
UPDATE project \
   SET auto_assign_login_id = '%s' \
 WHERE project_num = %d "

#define UPDATE_AUTO_ASSIGN_NULL "\
UPDATE project \
   SET auto_assign_login_id = NULL \
 WHERE project_num = %d "

#define SUB_LIST_SQL "SELECT  distinct login_id \
                        FROM submitter \
                       WHERE project_num = %d"

#define INSERT_VERSION "\
INSERT \
  INTO project_version (version_num, project_num, version_text, descr, \
                        active, order_num) \
VALUES (%d, %d, '%s', '%s', %s, %d)"

#define UPDATE_VERSION "\
UPDATE project_version \
   SET version_text = '%s', \
       descr = '%s', \
       active = %s, \
       order_num = %d \
 WHERE version_num = %d \
   AND project_num = %d"

#define DELETE_VERSION "\
DELETE \
  FROM project_version \
 WHERE version_num = %d \
   and project_num = %d"

static const gchar *my_child_list = "CHILD_LIST";
                             
/*
 * This application can only have one projet list dialog visible at any
 * given time.  This is that dialog...
 */
static GtkWidget  *project_dlg = NULL;

/*
 *  Local prototypes
 */
static gboolean draw_project_list (GtkWindow *parent, PGconn *conn);

/*
 * PrjVerStruct
 *
 * Store the data needed for one item in the project version list.
 */
typedef enum {
   NO_ACTION,
   INSERT,
   UPDATE,
   DELETE
} ActionType;

typedef struct PrjVerStruct {
   gint          version_num;
   GString      *version_str;
   GString      *descr_str;
   gboolean      is_active;
   gint          order_num;
   ActionType    action; 
} PrjVerStruct;


/*
 * Responsible and Submitter list handling:
 *
 * Rather than deleting all current rows in the responsible/submitter
 * tables, and then refreshing them with the current selections, the
 * submitter and responsible clists keep track of who needs to be 
 * added and who needs to be removed.  All others remain untouched.
 *
 * In order to facilitate this, the submitter and responsible clists 
 * both contain two other lists.
 *   o add_list : The list of items that are to be added to the 
 *                sub/resp table for this project
 *   o rm_list  : The list of items that are to be removed from the
 *                sub/resp table for this project
 * In other words, the add_list contains people who are not currently
 * in the resp or sub table, but are being added this time.  The rm_list
 * contains people who currently are in the resp or sub table but are
 * being removed from it this time.
 *
 * When a selection is made, that person is either removed from the 
 * current rm_list (they were unselected and reselected in the same
 * invocation of the screen), or is added to the add_list (they were
 * not selected at the time we started the screen, but are now).
 *
 * Whe an unselection is made, that person is either removed from the
 * current add_list (they were not selected when we started the screen,
 * were then selected, and now are being unselected), or is added to the
 * remove list (they were selected at the time we started the screen,
 * and are now being unselected).
 *
 * This is all handled by the selection_made and unselection_made 
 * functions.
 *
 * The insert_responsible, insert_submitter, rm_responsible, rm_submitter
 * functions traverse these lists when the 'OK' button is pressed and 
 * do the actual inserts and deletes.
 *
 * destroy_rm_add_lists destroys any ADD or RM lists that are associated 
 * with the given clist.
 */
static void 
selection_made (GtkWidget      *clist,
                gint            row,
                gint            column,
                GdkEventButton *event,
                gpointer        data)
{
   GList     *add_list;
   GList     *rm_list;
   GList     *tmp = NULL;
   gchar     *user_id;
   GtkWidget *main_win;


   add_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), ADD_LIST);
   rm_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), RM_LIST);

   gtk_clist_get_text (GTK_CLIST (clist), row, LOGIN_ID_COL, &user_id); 

   /*
    * See if this person is in our current "remove" list
    */
   if (rm_list)
      {
      tmp = g_list_find (rm_list, user_id);
      }

   /*
    * If the person is in the "remove" list, remove them from the
    * remove list.  Otherwise add them to the "add" list.
    * Update the object data as appropriate.
    */
   if (tmp)
      {
      rm_list = g_list_remove (rm_list, user_id);
      gtk_object_set_data (GTK_OBJECT (clist), RM_LIST, rm_list);
      }
   else
      {
      add_list = g_list_append (add_list, user_id);
      gtk_object_set_data (GTK_OBJECT (clist), ADD_LIST, add_list);
      }

   if (add_list != NULL)
      {
      main_win = get_widget (clist, TOP_DOG);
      g_assert (main_win != NULL);
      gnome_dialog_set_sensitive (GNOME_DIALOG (main_win), APPLY_BTN, TRUE);
      }
} 


static void 
unselection_made (GtkWidget      *clist,
                  gint            row,
                  gint            column,
                  GdkEventButton *event,
                  gpointer        data)
{
   GList     *add_list;
   GList     *rm_list;
   GList     *tmp = NULL;
   gchar     *user_id;
   GtkWidget *main_win;

   add_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), ADD_LIST);
   rm_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), RM_LIST);

   gtk_clist_get_text (GTK_CLIST (clist), row, LOGIN_ID_COL, &user_id); 

   /*
    * See if this person is in our current "add" list
    */
   if (add_list)
      {
      tmp = g_list_find (add_list, user_id);
      }

   /*
    * If the person is in the "add" list, remove them from the
    * "add" list.  Otherwise add them to the "remove" list.
    * Update the object data as appropriate.
    */
   if (tmp)
      {
      add_list = g_list_remove (add_list, user_id);
      gtk_object_set_data (GTK_OBJECT (clist), ADD_LIST, add_list);
      }
   else
      {
      rm_list = g_list_append (rm_list, user_id);
      gtk_object_set_data (GTK_OBJECT (clist), RM_LIST, rm_list);
      }

   if (rm_list != NULL)
      {
      main_win = get_widget (clist, TOP_DOG);
      g_assert (main_win != NULL);
      gnome_dialog_set_sensitive (GNOME_DIALOG (main_win), APPLY_BTN, TRUE);
      }
} 


/*
 * As people are added or deleted from the list of project memebers, we need
 * to add or remove their names from the auto-assign widget.  The next two
 * functions handle that logic.
 */
static void 
add_to_auto_assign (GtkWidget      *clist,
                    gint            row,
                    gint            column,
                    GdkEventButton *event,
                    gpointer        data)
{
   GtkCombo        *combo;
   GList           *name_list;
   GList           *full_list;
   gchar           *login_id;
   PGresult        *res;
   PGconn          *conn;
   GString         *sql_buffer;
   name_id_struct  *curr_name;

   combo = GTK_COMBO (get_widget (GTK_WIDGET (data), AUTO_ASSIGN_COMBO));
   g_assert (combo != NULL);
   name_list = gtk_object_get_data (GTK_OBJECT (combo), NAME_LIST);
   g_assert (name_list != NULL);
   full_list = gtk_object_get_data (GTK_OBJECT (combo), FULL_LIST);
   g_assert (full_list != NULL);

   gtk_clist_get_text (GTK_CLIST (clist), row, LOGIN_ID_COL, &login_id);

   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, GET_USER_FULL_NAME, login_id);
   conn = gtk_object_get_data (GTK_OBJECT (data), MY_DB_CONN);
   g_assert (conn != NULL);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, "fetch user's full name"))
      {
      g_assert (PQntuples (res) == 1);
      curr_name = g_malloc (sizeof (name_id_struct));
      curr_name->id = g_malloc (strlen (login_id) + 1);
      curr_name->name = g_malloc (strlen (PQgetvalue (res, 0, 0)) + 1);
      strcpy (curr_name->id, login_id);
      strcpy (curr_name->name, PQgetvalue (res, 0, 0));
      full_list = g_list_append (full_list, curr_name); 
      name_list = g_list_insert_sorted (name_list, 
                                        curr_name->name,
                                        (GCompareFunc) strcmp);
      gtk_combo_set_popdown_strings (combo, name_list);
      gtk_object_set_data (GTK_OBJECT (combo), NAME_LIST, name_list);
      gtk_object_set_data (GTK_OBJECT (combo), FULL_LIST, full_list);
      }
   g_string_free (sql_buffer, TRUE); 
   PQclear (res);
}


static void 
remove_from_auto_assign (GtkWidget      *clist,
                         gint            row,
                         gint            column,
                         GdkEventButton *event,
                         gpointer        data)
{
   GtkCombo        *combo;
   GList           *full_list;
   GList           *name_list;
   name_id_struct  *curr_name;
   gchar           *login_id;

   combo = GTK_COMBO (get_widget (GTK_WIDGET (data), AUTO_ASSIGN_COMBO));
   g_assert (combo != NULL);
   name_list = gtk_object_get_data (GTK_OBJECT (combo), NAME_LIST);
   g_assert (name_list != NULL);
   full_list = gtk_object_get_data (GTK_OBJECT (combo), FULL_LIST);
   g_assert (full_list != NULL);

   gtk_clist_get_text (GTK_CLIST (clist), row, LOGIN_ID_COL, &login_id);
   curr_name = find_name_id_from_id (full_list, login_id);

   name_list = g_list_remove (name_list, curr_name->name);
   full_list = g_list_remove (full_list, curr_name);

   gtk_combo_set_popdown_strings (combo, name_list);

   /*
    * If we removed the person who is currently set as the auto-assign
    * user, we will need to set the value to someone else.  Use NULL.
    */
   if (strcmp (curr_name->name, 
               gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry))) == 0)
      {
      gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(combo)->entry), NULL_NAME);
      }

   gtk_object_set_data (GTK_OBJECT (combo), NAME_LIST, name_list);
   gtk_object_set_data (GTK_OBJECT (combo), FULL_LIST, full_list);
}


/*
 * user_data should be a pointer to the clist that the add/rm lists
 * are attached to.
 */
static void
on_add_all_clicked (GtkButton  *button, gpointer  user_data)
{
   GList     *add_list;
   GList     *rm_list;
   gint       row;
   gchar     *user_id = NULL;
   GList     *tmp = NULL;
   GtkCList  *clist = GTK_CLIST (user_data);
   
   add_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), 
                                            ADD_LIST);
   rm_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), 
                                           RM_LIST);
   gtk_object_remove_data (GTK_OBJECT (clist), ADD_LIST);
   gtk_object_remove_data (GTK_OBJECT (clist), RM_LIST);

   /*
    * Go through the full list of people, and manipulate the add and rm
    * lists appropriately.  There are two basic scenarios.  The person
    * is either highlighted, or not.
    *
    * If highlighted, the rm and add lists are already OK for this 
    * person.
    *
    * If not highlighted, the person either needs to be removed from the
    * rm list, or added to the add list.
    */
   gtk_clist_freeze (clist);
   for (row = 0; row < clist->rows; row++)
      {
      if (!g_list_find (clist->selection, GINT_TO_POINTER (row)))
         {
         gtk_clist_get_text (GTK_CLIST (clist), row, 
                             LOGIN_ID_COL, &user_id); 

         if (rm_list)
            {
            tmp = g_list_find (rm_list, user_id);
            }
         else
            {
            tmp = NULL;
            }

         /*
          * tmp will be non-null if rm_list is non-null AND
          * user_id was found.  Otherwise, it will be NULL.
          */
         if (tmp)
            {
            rm_list = g_list_remove (rm_list, user_id);
            }
         else
            {
            add_list = g_list_append (add_list, user_id);
            }
         gtk_clist_select_row (clist, row, 0);
         }
      }
   gtk_object_set_data (GTK_OBJECT (clist), ADD_LIST, add_list);
   gtk_object_set_data (GTK_OBJECT (clist), RM_LIST, rm_list);
   gtk_clist_thaw (clist);
}


/*
 * user_data should be a pointer to the clist that the add/rm lists
 * are attached to.
 */
static void
on_rm_all_clicked (GtkButton  *button, gpointer  user_data)
{
   GList     *add_list;
   GList     *rm_list;
   gint       row;
   gchar     *user_id = NULL;
   GList     *tmp = NULL;
   GtkCList  *clist = GTK_CLIST (user_data);
   
   add_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), 
                                            ADD_LIST);
   rm_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), 
                                           RM_LIST);
   gtk_object_remove_data (GTK_OBJECT (clist), ADD_LIST);
   gtk_object_remove_data (GTK_OBJECT (clist), RM_LIST);

   /*
    * Go through the full list of people, and manipulate the add and rm
    * lists appropriately.  There are two basic scenarios.  The person
    * is either highlighted, or not.
    *
    * If not highlighted, the rm and add lists are already OK for this 
    * person.
    *
    * If highlighted, the person either needs to be removed from the
    * add list, or added to the rm list.
    */
   gtk_clist_freeze (clist);
   for (row = 0; row < clist->rows; row++)
      {
      if (g_list_find (clist->selection, GINT_TO_POINTER (row)))
         {
         gtk_clist_get_text (GTK_CLIST (clist), row, 
                             LOGIN_ID_COL, &user_id); 

         if (add_list)
            {
            tmp = g_list_find (add_list, user_id);
            }
         else
            {
            tmp = NULL;
            }

         /*
          * If tmp is non-null, it was found in the add_list.  Remove
          * it (we don't want to add anyone).  Otherwise, the person 
          * needs to be added to the rm_list.
          */
         if (tmp)
            {
            add_list = g_list_remove (add_list, user_id);
            }
         else
            {
            rm_list = g_list_append (rm_list, user_id);
            }
         gtk_clist_unselect_row (clist, row, 0);
         }
      }
   gtk_object_set_data (GTK_OBJECT (clist), ADD_LIST, add_list);
   gtk_object_set_data (GTK_OBJECT (clist), RM_LIST, rm_list);
   gtk_clist_thaw (clist);
}


static void 
destroy_rm_add_lists (GtkCList   *clist)
{
   GList  *the_list;

   the_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), ADD_LIST);
   if (the_list)
      {
      gtk_object_remove_data (GTK_OBJECT (clist), ADD_LIST);
      g_list_free (the_list);
      }

   the_list = (GList *)gtk_object_get_data (GTK_OBJECT (clist), RM_LIST);
   if (the_list)
      {
      gtk_object_remove_data (GTK_OBJECT (clist), RM_LIST);
      g_list_free (the_list);
      }
}


static gboolean 
rm_submitter (PGconn       *conn, 
              gint          project_num,
              const gchar  *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;
   
   buffer = g_string_new ("");
   g_string_sprintf (buffer, RM_SUBMITTER_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Removing Submitters"));
   
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


static gboolean 
rm_responsible (PGconn       *conn, 
                gint          project_num,
                const gchar  *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;
   
   buffer = g_string_new ("");
   g_string_sprintf (buffer, RM_RESPONSIBLE_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Removing Responsible"));
   
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


static gboolean 
insert_submitter (PGconn      *conn, 
                  gint         project_num,
                  const gchar *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;
   
   buffer = g_string_new ("");
   g_string_sprintf (buffer, INS_SUBMITTER_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Inserting Submitter"));
   
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


static gboolean 
insert_responsible (PGconn      *conn, 
                    gint         project_num,
                    const gchar *login_id)
{
   GString  *buffer;
   PGresult *res;
   gboolean  ok;
   
   buffer = g_string_new ("");
   g_string_sprintf (buffer, INS_RESPONSIBLE_SQL, project_num, login_id);
   res = PQexec (conn, buffer->str);
   ok = chk_sql_error (res, _("Inserting Project Member"));
   
   PQclear (res);
   g_string_free (buffer, TRUE);

   return ok;
}


/*
 * Queries the database for all persons defined, active or not.
 */
static PGresult *
read_person_data (PGconn *conn)
{
   GString  *buffer;
   PGresult *res;
   
   buffer = g_string_new (PERSON_QUERY);
   res = PQexec (conn, buffer->str);
   
   g_string_free (buffer, TRUE);
   return res;
}


/*
 * destroy_project_version
 *
 * Take the PrjVerStruct and free all memory used by it.
 */
static void
destroy_project_version (PrjVerStruct *ver)
{
   g_assert (ver);

   g_string_free (ver->version_str, TRUE);
   g_string_free (ver->descr_str, TRUE);

   g_free (ver);
}


/*
 * find_version_in_list
 *
 * Given a version string, attempt to find it in the list.  Return a pointer
 * to that record if found.  Otherwise return NULL.
 */
static PrjVerStruct *
find_version_in_list (const gchar *str, const GList *list)
{
   GList         *list_iter = NULL;
   PrjVerStruct  *rtn_val = NULL;
   GString       *tmp_str;

   list_iter = g_list_first ((GList *)list);
   while (list_iter)
      {
      tmp_str = ((PrjVerStruct *)list_iter->data)->version_str;
      if (strcmp (tmp_str->str, str) == 0)
         {
         rtn_val = (PrjVerStruct *)list_iter->data;
         list_iter = NULL;
         }
      else
         {
         list_iter = list_iter->next;
         }
      }

   return rtn_val;
}


/*
 * find_ver_num_in_list
 *
 * Given a version number, attempt to find it in the list.  Return a pointer
 * to that record if found.  Otherwise return NULL.
 */
static PrjVerStruct *
find_ver_num_in_list (gint ver_num, const GList *list)
{
   GList        *list_iter = NULL;
   PrjVerStruct *rtn_val = NULL;

   list_iter = g_list_first ((GList *)list);
   while (list_iter)
      {
      if (((PrjVerStruct *)list_iter->data)->version_num == ver_num)
         {
         rtn_val = (PrjVerStruct *)list_iter->data;
         list_iter = NULL;
         }
      else
         {
         list_iter = list_iter->next;
         }
      }

   return rtn_val;
}


/*
 * get_next_version_pk
 *
 * Get the next version number from the database for this project.  This 
 * is done at insertion time.  Up until then, the version number used 
 * in the version list is just a unique guess.
 */
static gint
get_next_version_pk (PGconn *conn, gint prj)
{
   GString   *sql_buffer;
   PGresult  *res;
   gint       ver_num = 0;

   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, "SELECT next_version_num( %d )", prj);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Get next project version number")))
      {
      ver_num = atoi (PQgetvalue (res, 0, 0));
      }

   PQclear (res);
   g_string_free (sql_buffer, TRUE);

   return ver_num;
}


/*
 * get_max_ver_num
 *
 * Get the maximum version number in the list.  Return 0 if there are no
 * versions in the list.
 */
static gint
get_max_ver_num (GList *list)
{
   GList         *list_iter;
   gint           curr_max;
   PrjVerStruct  *curr_ver;

   curr_max = 0;
   list_iter = g_list_first (list);
   while (list_iter)
      {
      curr_ver = (PrjVerStruct *)list_iter->data;
      curr_max = (curr_ver->version_num > curr_max) ? curr_ver->version_num 
                                                    : curr_max;
      list_iter = list_iter->next;
      }

   return curr_max;
}


/*
 * get_max_order_num
 *
 * Get the maximum order number in the list.  Return 0 if there are no
 * versions in the list.
 */
static gint
get_max_order_num (GList *list)
{
   GList         *list_iter;
   gint           curr_max;
   PrjVerStruct  *curr_ver;

   curr_max = 0;
   list_iter = g_list_first (list);
   while (list_iter)
      {
      curr_ver = (PrjVerStruct *)list_iter->data;
      curr_max = (curr_ver->order_num > curr_max) ? curr_ver->order_num 
                                                  : curr_max;
      list_iter = list_iter->next;
      }

   return curr_max;
}


/*
 * load_version_data
 *
 * Load the version data for the current project.  Store the data in 
 * a linked list.
 */
static GList *
load_version_data (PGconn *conn, gint project_num)
{
   GString       *sql_buffer;
   PGresult      *res;
   GList         *list = NULL;
   PrjVerStruct  *tmp;
   gint           n, i;

   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, VERSION_QUERY, project_num);
   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Getting project version data")))
      {
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         tmp = g_malloc (sizeof (PrjVerStruct));
         tmp->version_num = atoi (PQgetvalue (res, i, PK_NUM_POS));
         tmp->version_str = 
            g_string_new (PQgetvalue (res, i, NAME_POS));
         tmp->descr_str =
            g_string_new (PQgetvalue (res, i, DESCR_POS));
         tmp->is_active = 
            (toupper ((PQgetvalue (res, i, ACTIVE_POS))[0]) == 'T');
         tmp->order_num = atoi (PQgetvalue (res, i, ORDER_NUM_POS));
         tmp->action = NO_ACTION;
         list = g_list_append (list, tmp);
         }
      }

   g_string_free (sql_buffer, TRUE);
   PQclear (res);

   return g_list_first (list);
}


/*
 * update_version_combos
 *
 * Update the list used in the version_combos.  Use the version list
 * currently attached to the version clist.
 */
static void
update_version_combo (GtkWidget *parent_widget)
{
   GtkCombo        *open_combo;
   GtkCombo        *close_combo;
   GtkCList        *ver_clist;
   GList           *ver_list = NULL;
   GList           *full_list = NULL;
   GList           *name_list = NULL; 
   GList           *list_iter = NULL;
   gint             old_open = INVALID_PK;
   gint             old_close = INVALID_PK;
   PrjVerStruct    *curr_ver;
   name_pk_struct  *curr_name_pk;
   gchar           *tmp_txt;

   open_combo = GTK_COMBO (get_widget (parent_widget, DEF_OPEN_COMBO));
   g_assert (open_combo);
   close_combo = GTK_COMBO (get_widget (parent_widget, DEF_CLOSE_COMBO));
   g_assert (close_combo);
   ver_clist = GTK_CLIST (get_widget (parent_widget, VER_CLIST));
   g_assert (ver_clist);
   ver_list = gtk_object_get_data (GTK_OBJECT (ver_clist), VERSION_LIST);
   full_list = gtk_object_get_data (GTK_OBJECT (ver_clist), 
                                    VERSION_NAME_PK_LIST);

   /*
    * If there was a full list, destroy it.  We will be building
    * a new one.
    */
   if (full_list)
      {
      old_open = find_pk_from_name 
         (full_list, 
          gtk_entry_get_text (GTK_ENTRY (open_combo->entry))); 
      old_close = find_pk_from_name
         (full_list,
          gtk_entry_get_text (GTK_ENTRY (close_combo->entry)));
      gtk_object_remove_data (GTK_OBJECT (ver_clist), VERSION_NAME_PK_LIST);
      destroy_name_pk_list (full_list);
      full_list = NULL;
      }

   /*
    * Always start with <None>
    */
   curr_name_pk = g_malloc (sizeof (name_pk_struct));
   curr_name_pk->pk = INVALID_PK;
   curr_name_pk->name = g_malloc (NULL_NAME_SIZE + 1);
   strcpy (curr_name_pk->name, NULL_NAME);
   name_list = g_list_append (name_list, curr_name_pk->name);
   full_list = g_list_append (full_list, curr_name_pk); 

   /*
    * Fill in the rest of the active versions.  Attach the full list
    * to the version clist.
    */
   list_iter = g_list_first (ver_list);
   while (list_iter)
      {
      curr_ver = list_iter->data;
      if (curr_ver->is_active && (curr_ver->action != DELETE))
         {
         curr_name_pk = g_malloc (sizeof (name_pk_struct));
         curr_name_pk->pk = curr_ver->version_num;
         curr_name_pk->name = g_malloc (curr_ver->version_str->len + 1);
         strcpy (curr_name_pk->name, curr_ver->version_str->str);
         name_list = g_list_append (name_list, curr_name_pk->name);
         full_list = g_list_append (full_list, curr_name_pk); 
         }
      list_iter = list_iter->next;
      }
   gtk_combo_set_popdown_strings (open_combo, name_list);
   gtk_combo_set_popdown_strings (close_combo, name_list);
   g_list_free (name_list);
   gtk_object_set_data (GTK_OBJECT (ver_clist), 
                        VERSION_NAME_PK_LIST, 
                        full_list);

   /*
    * Attempt to restore the text part of the combo to the original
    * text (if it is still in the list).  If not, default to <None>.
    */
   tmp_txt = find_name_from_pk (full_list, old_open);
   gtk_entry_set_text (GTK_ENTRY (open_combo->entry), 
                       (tmp_txt != NULL) ? tmp_txt : NULL_NAME);
   tmp_txt = find_name_from_pk (full_list, old_close);
   gtk_entry_set_text (GTK_ENTRY (close_combo->entry), 
                       (tmp_txt != NULL) ? tmp_txt : NULL_NAME);
   set_clean (close_combo->entry, NULL);
   set_clean (open_combo->entry, NULL);
}


/*
 * Translates the 't' or 'f' of the active field to a 
 * 'Yes' or 'No' for display.
 */
static const char *
get_active_yn (PGresult *res, int n)
{
   if (n >= PQntuples (res))
      {
      return "";
      }
   
   if ((PQgetvalue (res, n, ACTIVE_POS))[0] == 't')
      {
      return "Yes";
      }
   
   return "No";
}


/*
 *  Given the clist and the login ID, highlight the user in the
 *  list matching the ID.
 */
static void 
highlight_user (GtkCList *clist, const gchar *login_id)
{
   gchar *buffer;
   gint curr_row = 0;
   
   while (curr_row < clist->rows)
      {
      gtk_clist_get_text (clist, curr_row, LOGIN_ID_COL, &buffer);
      if (!strcmp (login_id, buffer))
         {
         gtk_clist_select_row (clist, curr_row, LOGIN_ID_COL);
         /*
          * This seems to be a little buggy right now
          * gtk_clist_moveto (clist, curr_row, 0, 0.5, 0.0);
          */
         return;
         }
      curr_row++;
      }
   
   /*
    * We should always return while in the loop (we should find
    * any login ID given).
    */
   g_assert (FALSE);
}
   

/*
 * can_edit_project
 *
 * Determine if the current user is able to edit the project.
 */
static gboolean
can_edit_project (PGconn *conn, gint pk)
{
   PGresult  *res;
   GString   *sql_buffer;
   gboolean   access = FALSE;
 
   sql_buffer = g_string_new ("");
   g_string_sprintf (sql_buffer, "SELECT can_edit_project (%d)", pk);

   res = PQexec (conn, sql_buffer->str);
   if (chk_sql_error (res, _("Getting project access")))
      {
      access = (toupper (PQgetvalue (res, 0, 0)[0]) == 'T');
      }
   g_string_free (sql_buffer, TRUE);
   PQclear (res);

   return access;
}
/*
 * Draw the name list.  Include all persons in the database.
 */
static gboolean 
draw_name_list (GtkCList   *clist,
                PGconn     *conn)
{
   char            *buffer[3] = {NULL, NULL, NULL};
   gint             ntuples;
   gint             i;
   PGresult        *res;
   gboolean         rtn_val = FALSE;
   
   gtk_clist_clear (clist);
   res = read_person_data (conn);
   
   if (chk_sql_error (res, _("reading project person data")))
      {
      ntuples = PQntuples (res);
      
      for (i = 0; i < ntuples; i++)
         {
         /* format the first and last name into one column */
         buffer[0] = (char *)g_malloc
            (strlen (PQgetvalue (res, i, LAST_NAME_POS)) +
             strlen (PQgetvalue (res, i, FIRST_NAME_POS)) +
             3);
         sprintf (buffer[0], "%s, %s",
                  PQgetvalue (res, i, LAST_NAME_POS),
                  PQgetvalue (res, i, FIRST_NAME_POS));
         
         buffer[1] = (char *)g_malloc
            (strlen (PQgetvalue (res, i, LOGIN_ID_POS)) + 1);
         strcpy (buffer[1], PQgetvalue (res, i, LOGIN_ID_POS));
         
         buffer[2] = (char *)g_malloc (strlen (get_active_yn (res, i)));
         strcpy (buffer[2], get_active_yn (res, i));
         
         gtk_clist_append (clist, buffer);
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         }
      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val;
}


/*
 * draw_version_list
 *
 * Draw the version data in the version clist.  The data to be drawn
 * is stored in a linked list that is stored as object data of the
 * clist.
 */
static void
draw_version_list (GtkCList  *clist, gboolean show_inactive)
{
   PrjVerStruct    *tmp_data;
   GList           *ver_list;
   GList           *list_iter;
   gchar           *buffer[3];
   gint             i = 0;

   gtk_clist_clear (clist);
   ver_list = gtk_object_get_data (GTK_OBJECT (clist), VERSION_LIST);

   list_iter = g_list_first (ver_list);
   while (list_iter)
      {
      tmp_data = list_iter->data;
      if (tmp_data->action != DELETE &&
          (show_inactive || tmp_data->is_active))
         {
         buffer[0] = g_malloc (tmp_data->version_str->len + 1);
         strcpy (buffer[0], tmp_data->version_str->str);

         buffer[1] = g_malloc (tmp_data->descr_str->len + 1);
         strcpy (buffer[1], tmp_data->descr_str->str);

         if (tmp_data->is_active)
            {
            buffer[2] = g_malloc (4);
            strcpy (buffer[2], "Yes");
            }
         else
            {
            buffer[2] = g_malloc (3);
            strcpy (buffer[2], "No");
            }

         gtk_clist_append (clist, buffer);
         gtk_clist_set_row_data (clist, i, 
                                 GINT_TO_POINTER (tmp_data->version_num));
         i++;
         
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         }
      list_iter = list_iter->next;
      }

   return;
}


static void 
on_project_edit_dlg_show (GtkWidget  *widget,
                          gpointer    user_data)
{
   gboolean         in_error = TRUE; /* set false if resp and submitter
                                        lists all draw OK. */
   PGconn          *conn;
   GString         *query;
   GtkWidget       *child;
   GtkCList        *lb_widget;
   GtkCombo        *auto_assign_combo;
   PGresult        *result;
   GList           *leader_list;
   GList           *ver_list;
   GList           *full_list = NULL;
   GList           *name_list = NULL;
   gint             i, n;
   gint             def_open_ver = INVALID_PK;
   gint             def_close_ver = INVALID_PK;
   gboolean         show_inactive;
   name_id_struct  *curr_name;
   gint             pk_num;
   gboolean         is_update;

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                  PK_NUM));
   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
                                                      IS_UPDATE)) == 1);

   /*
    *  First display the submitter and project member lists, highlighting
    *  as appropriate for the current project.  (Note that for new projects,
    *  the queries will return no data, so no harm done).
    *
    *  draw_name_list will just draw the name list, it will not highlight
    *  the names that need to be highlighted.
    */
   lb_widget = GTK_CLIST (get_widget (widget, SUBMITTER_LIST));
   g_assert (lb_widget != NULL);
   auto_assign_combo = GTK_COMBO (get_widget (widget, AUTO_ASSIGN_COMBO));
   g_assert (auto_assign_combo != NULL);
   if (draw_name_list (lb_widget, conn))
      {
      query = g_string_new ("");
      g_string_sprintf (query, SUB_LIST_SQL, pk_num);
      result = PQexec (conn, query->str);
      if (chk_sql_error (result, _("Getting Submitter List")))
         {
         gtk_clist_freeze (lb_widget);
         n = PQntuples (result);
         for (i = 0; i < n; i++)
            {
            highlight_user (lb_widget, PQgetvalue (result, i, 0));
            }
         gtk_clist_thaw (lb_widget);
         }
      PQclear (result);
      g_string_free (query, TRUE);
      gtk_object_set_data (GTK_OBJECT (lb_widget), ADD_LIST, NULL);
      gtk_object_set_data (GTK_OBJECT (lb_widget), RM_LIST, NULL);
      /*
       * Now that the initial drawing and highlighting is done,
       * attach the signal handlers that deal with user selection 
       * and unselection actions.
       */
      gtk_signal_connect (GTK_OBJECT (lb_widget),
                          "select_row",
                          GTK_SIGNAL_FUNC (selection_made),
                          NULL);
      gtk_signal_connect (GTK_OBJECT (lb_widget),
                          "unselect_row",
                          GTK_SIGNAL_FUNC (unselection_made),
                          NULL);

      lb_widget = GTK_CLIST (get_widget (widget, PRJ_MEMBER_LIST));
      g_assert (lb_widget != NULL);
      if (draw_name_list (lb_widget, conn))
         {
         /*
          * Both lists drew fine, so set the flag to no error.
          */
         in_error = FALSE;

         /*
          * While we are going through highlighting the current project
          * members, we will take the oppertunity to populate the 
          * auto-assign box.
          *
          * Always start with <None>
          */
         curr_name = g_malloc (sizeof (name_id_struct));
         curr_name->id = g_malloc (sizeof (gchar) * (INVALID_ID_LEN + 1));
         curr_name->name = g_malloc (sizeof (gchar) * (NULL_NAME_SIZE + 1));
         strcpy (curr_name->id, INVALID_ID);
         strcpy (curr_name->name, NULL_NAME);
         name_list = g_list_append (name_list, curr_name->name);
         full_list = g_list_append (full_list, curr_name); 
         
         query = g_string_new ("");
         g_string_sprintf (query, RESP_LIST_SQL, pk_num);
         result = PQexec (conn, query->str);
         if (chk_sql_error (result, _("Getting Responsible List")))
            {
            gtk_clist_freeze (lb_widget);
            n = PQntuples (result);
            for (i = 0; i < n; i++)
               {
               curr_name = g_malloc (sizeof (name_id_struct));
               curr_name->id = g_malloc 
                  (strlen (PQgetvalue (result, i, ID_POS)) + 1);
               curr_name->name = g_malloc
                  (strlen (PQgetvalue (result, i, NAME_POS)) + 1);
               strcpy (curr_name->id, PQgetvalue (result, i, ID_POS));
               strcpy (curr_name->name, PQgetvalue (result, i, NAME_POS));
               name_list = g_list_append (name_list, curr_name->name);
               full_list = g_list_append (full_list, curr_name); 

               highlight_user (lb_widget, curr_name->id);
               }
            gtk_clist_thaw (lb_widget);
            }
         PQclear (result);
         g_string_free (query, TRUE);
         gtk_object_set_data (GTK_OBJECT (lb_widget), ADD_LIST, NULL);
         gtk_object_set_data (GTK_OBJECT (lb_widget), RM_LIST, NULL);
         gtk_combo_set_popdown_strings (GTK_COMBO (auto_assign_combo),
                                        name_list);
         gtk_object_set_data (GTK_OBJECT (auto_assign_combo),
                              FULL_LIST, full_list);
         gtk_object_set_data (GTK_OBJECT (auto_assign_combo),
                              NAME_LIST, name_list);
         /*
          * Now that the initial drawing and highlighting is done,
          * attach the signal handlers that deal with user selection 
          * and unselection actions.
          */
         gtk_signal_connect (GTK_OBJECT (lb_widget),
                             "select_row",
                             GTK_SIGNAL_FUNC (selection_made),
                             NULL);
         gtk_signal_connect (GTK_OBJECT (lb_widget),
                             "unselect_row",
                             GTK_SIGNAL_FUNC (unselection_made),
                             NULL);
         gtk_signal_connect (GTK_OBJECT (lb_widget),
                             "select_row",
                             GTK_SIGNAL_FUNC (add_to_auto_assign),
                             widget);
         gtk_signal_connect (GTK_OBJECT (lb_widget),
                             "unselect_row",
                             GTK_SIGNAL_FUNC (remove_from_auto_assign),
                             widget);
         }
      }
   
   /*
    * If we are updating an existing project, load the current project 
    * information into the dialog box.
    */
   if (!in_error)
      {
      if (is_update)
         {
         query = g_string_new ("");
         g_string_sprintf (query, PROJECT_QUERY, pk_num);
      
         result = PQexec (conn, query->str);
         if (chk_sql_error (result, _("getting project data to edit")))
            {
            /*
             * At this point, the full_list should still be the
             * full_list from setting up the auto-assign combo.
             */
            child = get_widget (widget, AUTO_ASSIGN_COMBO);
            g_assert (child != NULL);
            if (!PQgetisnull (result, 0, AUTO_ASSIGN_POS))
               {
               gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (child)->entry),
                     find_name_from_id (full_list,
                                        PQgetvalue (result,
                                                    0,
                                                    AUTO_ASSIGN_POS)));
               }
            else
               {
               gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (child)->entry),
                                   NULL_NAME); 
               }

            child = get_widget (widget, NAME_ENTRY);
            g_assert (child != NULL);
            gtk_entry_set_text (GTK_ENTRY (child),
                  PQgetvalue (result, 0, NAME_POS));
         
            child = get_widget (widget, DESCR_TEXT);
            g_assert (child != NULL);
            gtk_text_insert (GTK_TEXT (child), NULL, NULL, NULL,
                  PQgetvalue (result, 0, DESCR_POS), -1);
         
            /*
             *  The leader combo contains a list that matches the
             *  displayed text with the login_id.
             */
            child = get_widget (widget, LEADER_COMBO);
            g_assert (child != NULL);
            leader_list = gtk_object_get_data (GTK_OBJECT (child),
                                               LEADER_LIST);
            gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (child)->entry),
                  find_name_from_id (leader_list,
                                     PQgetvalue (result, 0, LOGIN_ID_POS)));

            child = get_widget (widget, ACTIVE_CB);
            g_assert (child != NULL);
            if (toupper ((PQgetvalue (result, 0, ACTIVE_POS))[0]) == 'T')
               {
               gtk_toggle_button_set_active 
                  (GTK_TOGGLE_BUTTON (child), TRUE);
               }
            else
               {
               gtk_toggle_button_set_active
                  (GTK_TOGGLE_BUTTON (child), FALSE);
               }

            child = get_widget (widget, NEW_PR_NOTIFY_GRP); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, NEW_PR_GRP_POS))[0]) =='T');

            child = get_widget (widget, NEW_PR_NOTIFY_PL); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, NEW_PR_PL_POS))[0]) =='T');

            child = get_widget (widget, ASSIGN_NOTIFY_RESP); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, ASSIGN_RESP_POS))[0]) =='T');

            child = get_widget (widget, ASSIGN_NOTIFY_SUB); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, ASSIGN_SUB_POS))[0]) =='T');

            child = get_widget (widget, ASSIGN_NOTIFY_PL); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, ASSIGN_PL_POS))[0]) =='T');

            child = get_widget (widget, UPD_NOTIFY_GRP); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, UPD_GRP_POS))[0]) =='T');

            child = get_widget (widget, UPD_NOTIFY_PL); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, UPD_PL_POS))[0]) =='T');

            child = get_widget (widget, UPD_NOTIFY_RESP); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, UPD_RESP_POS))[0]) =='T');

            child = get_widget (widget, UPD_NOTIFY_SUB); 
            g_assert (child != NULL);
            gtk_toggle_button_set_active 
               (GTK_TOGGLE_BUTTON (child),
                toupper ((PQgetvalue (result, 0, UPD_SUB_POS))[0]) =='T');

            if (PQgetisnull (result, 0, OPEN_VER_POS))
               {
               def_open_ver = INVALID_PK;
               }
            else
               {
               def_open_ver = atoi (PQgetvalue (result, 0, OPEN_VER_POS));
               }

            if (PQgetisnull (result, 0, CLOSE_VER_POS))
               {
               def_close_ver = INVALID_PK;
               }
            else
               {
               def_close_ver = atoi (PQgetvalue (result, 0, CLOSE_VER_POS));
               }
            }
         else
            {
            in_error = TRUE;
            }
         PQclear (result);
         g_string_free (query, TRUE);

         /*
          * If there has not been a error to this point, fetch the version
          * information for the current project and display it.
          */
         if (!in_error)
            {
            child = get_widget (widget, SHOW_ALL_VER_CB);
            g_assert (child != NULL);
            show_inactive = gtk_toggle_button_get_active
               (GTK_TOGGLE_BUTTON (child));
            child = get_widget (widget, VER_CLIST);
            g_assert (child != NULL);
            ver_list = load_version_data (conn, pk_num);
            gtk_object_set_data (GTK_OBJECT (child), VERSION_LIST, ver_list); 
            draw_version_list (GTK_CLIST (child), show_inactive);
            update_version_combo (widget);
            /*
             * Now that we have the version list loaded and drawn, and
             * we have the default version combo box drop downs loaded
             * with values, we need to initialize the values that 
             * those combos display.
             */
            full_list = gtk_object_get_data (GTK_OBJECT (child),
                                             VERSION_NAME_PK_LIST);
            if (full_list)
               {
               child = get_widget (widget, DEF_OPEN_ENTRY);
               g_assert (child);
               gtk_entry_set_text (GTK_ENTRY (child),
                                   find_name_from_pk (full_list,
                                                      def_open_ver));
               child = get_widget (widget, DEF_CLOSE_ENTRY);
               g_assert (child);
               gtk_entry_set_text (GTK_ENTRY (child),
                                   find_name_from_pk (full_list, 
                                                      def_close_ver));
               }
            }
         }
      /*
       * Done for both a new project and an update of an existing project.
       * Load the combo boxes with all valid versions on an update.  On
       * a new project, initialize them to <None>.
       */
      update_version_combo (widget);
      }
   
   if (in_error)
      {
      gnome_dialog_close (GNOME_DIALOG (widget));
      }
   else
      {
      set_all_clean (widget);
      gnome_dialog_set_sensitive (GNOME_DIALOG (widget), APPLY_BTN, FALSE);
      }
}


static void 
on_project_edit_dlg_destroy (GtkObject  *object,
                             gpointer    user_data)
{
   GList     *temp_list;
   GList     *list_iter;
   GtkCombo  *combo;
   GtkCList  *submitter_list;
   GtkCList  *member_list;
   GtkCList  *ver_clist;
   
   combo = GTK_COMBO (get_widget (GTK_WIDGET (object), LEADER_COMBO));
   g_assert (combo != NULL);
   ver_clist = GTK_CLIST (get_widget (GTK_WIDGET (object), VER_CLIST));
   g_assert (ver_clist != NULL);
   submitter_list = GTK_CLIST (get_widget (GTK_WIDGET (object), 
                                           SUBMITTER_LIST));
   g_assert (submitter_list != NULL);
   member_list  = GTK_CLIST (get_widget (GTK_WIDGET (object), 
                                         PRJ_MEMBER_LIST));
   g_assert (member_list != NULL);

   temp_list = gtk_object_get_data (GTK_OBJECT (combo), LEADER_LIST);
   g_assert (temp_list != NULL);
   destroy_name_id_list (temp_list);

   temp_list = gtk_object_get_data 
      (GTK_OBJECT (ver_clist), VERSION_NAME_PK_LIST);
   destroy_name_pk_list (temp_list);

   destroy_rm_add_lists (submitter_list);
   destroy_rm_add_lists (member_list);

   temp_list = gtk_object_get_data (GTK_OBJECT (ver_clist), VERSION_LIST);
   list_iter = g_list_first (temp_list);
   while (list_iter)
      {
      destroy_project_version (list_iter->data);
      list_iter = list_iter->next;
      }

   combo = GTK_COMBO (get_widget (GTK_WIDGET (object), AUTO_ASSIGN_COMBO));
   temp_list = gtk_object_get_data (GTK_OBJECT (combo), NAME_LIST);
   g_list_free (temp_list);
   temp_list = gtk_object_get_data (GTK_OBJECT (combo), FULL_LIST);
   destroy_name_id_list (temp_list);
   destroy_dirty_list (GTK_WIDGET (object));
}


static gboolean
update_auto_assign (PGconn   *conn,
                    gint      pk_num,
                    GtkCombo *combo)
{
   GString  *sql_buffer;
   PGresult *res;
   GList    *full_list;
   gchar    *name;
   gchar    *id;
   gboolean  ok;

   full_list = gtk_object_get_data (GTK_OBJECT (combo), FULL_LIST);
   g_assert (full_list);
   name = gtk_entry_get_text (GTK_ENTRY (combo->entry));
                           
   sql_buffer = g_string_new ("");
   if (strcmp (name, NULL_NAME) == 0)
      {
      g_string_sprintf (sql_buffer,
                        UPDATE_AUTO_ASSIGN_NULL,
                        pk_num);
      }
   else
      {
      id = find_id_from_name (full_list, name);
      g_string_sprintf (sql_buffer,
                        UPDATE_AUTO_ASSIGN,
                        id, pk_num);
      }
   res = PQexec (conn, sql_buffer->str);
   ok = chk_sql_error (res, _("Updating auto assign login id"));
   PQclear (res);
   g_string_free (sql_buffer, TRUE);

   return ok;
}


static gboolean
validate_project (GtkWidget  *dlg)
{
   GtkEntry  *prj_name_entry;
   GtkCList  *submitter_clist;
   GtkCList  *prj_mem_clist;
   gboolean   retval = TRUE;
   GtkWidget *msg_dlg;

   prj_name_entry = GTK_ENTRY (get_widget (dlg, NAME_ENTRY));
   g_assert (prj_name_entry);
   submitter_clist = GTK_CLIST (get_widget (dlg, SUBMITTER_LIST));
   g_assert (submitter_clist);
   prj_mem_clist = GTK_CLIST (get_widget (dlg, PRJ_MEMBER_LIST));
   g_assert (prj_mem_clist);

   /*
    * For each check, only check if retval is still good (TRUE).
    */
   if (retval && strlen (gtk_entry_get_text (prj_name_entry)) == 0)
      {
      msg_dlg = gnome_message_box_new
         (_("You must supply a project name."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      retval = FALSE;
      }

   if (retval && !submitter_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You have not selected an Submitters for this project.\n"
            "You will not be able to enter problem reports against this\n"
            "project until you select some submitters.\n"
            "Are you sure this is what you want?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      retval = (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0);
      }
      
   if (retval && !prj_mem_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You have not selected any Project Members for this project.\n"
            "You will not be able to assign problems reports to anyone\n"
            "until you select some project members.\n"
            "Are you sure this is what you want?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      retval = (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0);
      }

   return retval;
}


static gboolean
validate_version_info (const PrjVerStruct *ver,
                       const GList        *list)
{
   GString   *tmp_str;
   gboolean   is_ok = TRUE;
   GtkWidget *msg_dlg;

   if (ver->version_str->len == 0)
      {
      is_ok = FALSE;
      msg_dlg = gnome_message_box_new
         (_("You must supply a version string."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }

   if (ver->descr_str->len == 0)
      {
      is_ok = FALSE;
      msg_dlg = gnome_message_box_new
         (_("You must supply a description."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }

   if (find_version_in_list (ver->version_str->str, list))
      {
      is_ok = FALSE;
      tmp_str = g_string_new (_("Version "));
      tmp_str = g_string_append (tmp_str, ver->version_str->str);
      tmp_str = g_string_append (tmp_str, 
                                 _(" has already been used in this project."));
      msg_dlg = gnome_message_box_new (tmp_str->str,
                                       GNOME_MESSAGE_BOX_ERROR,
                                       GNOME_STOCK_BUTTON_OK,
                                       NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      g_string_free (tmp_str, TRUE);
      }

   return is_ok;
}


static void 
on_show_inactive_ver_cb_toggled (GtkToggleButton  *togglebutton,
                                 gpointer          user_data)
{
   GtkCList             *clist;

   clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), VER_CLIST));
   draw_version_list (clist, gtk_toggle_button_get_active (togglebutton));
}


/*
 * set_ver_add_mod_btn_state
 *
 * Set the sensitivity of the project version add and modification buttons
 * based on whether or not all fields have data, and whether or not a 
 * version is selected in the clist (for mod...)
 */
static void
set_ver_add_mod_btn_state (GtkWidget  *widget, gpointer user_data)
{
   GtkWidget       *add_ver_btn;
   GtkWidget       *mod_ver_btn;
   GtkWidget       *del_ver_btn;
   GtkToggleButton *active_cb;
   GtkEntry        *ver_entry;
   GtkEntry        *ver_descr_entry;
   GtkCList        *ver_clist;
   GtkWidget       *parent;
   gboolean         add_sensitive = FALSE;
   gboolean         mod_sensitive = FALSE;
   gboolean         del_sensitive = FALSE;
   gchar           *ver_entry_str;
   gchar           *ver_descr_str;
   PrjVerStruct    *ver;
   gint             ver_num;
   GList           *ver_list;
   gint             row;

   parent = GTK_WIDGET (user_data);

   add_ver_btn = get_widget (parent, ADD_VER_BUTTON);
   g_assert (add_ver_btn != NULL);
   mod_ver_btn = get_widget (parent, MOD_VER_BUTTON);
   g_assert (mod_ver_btn != NULL);
   del_ver_btn = get_widget (parent, DEL_VER_BUTTON);
   g_assert (del_ver_btn != NULL);

   ver_entry = GTK_ENTRY (get_widget (parent, VER_ENTRY));
   g_assert (ver_entry != NULL);
   ver_descr_entry = GTK_ENTRY (get_widget (parent, VER_DESCR_ENTRY));
   g_assert (ver_descr_entry != NULL);
   ver_clist = GTK_CLIST (get_widget (parent, VER_CLIST));
   g_assert (ver_clist != NULL);
   active_cb = GTK_TOGGLE_BUTTON (get_widget (parent, VER_ACTIVE_CB));
   g_assert (active_cb != NULL);

   /*
    *  Determine which of the version buttons should be sensative
    *
    *  The add button should be sensitive if the version text and version
    *  description both have data, and the version text is different from
    *  the currently selected item's version text (if an item is currently
    *  selected).
    *
    *  The modify button should be sensitive if a version is selected and
    *  one of the following have been modified: version text, version 
    *  description, or the active check box.
    *
    *  The delete button should be sensitive if a version is selected.
    */ 
   ver_entry_str = gtk_entry_get_text (ver_entry);
   ver_descr_str = gtk_entry_get_text (ver_descr_entry);
   if (ver_clist->selection)
      {
      del_sensitive = TRUE;
      if (strlen (ver_entry_str) > 0 && strlen (ver_descr_str) > 0)
         {
         /*
          * Get the information for the current selection so we can
          * look for any changes.
          */
         row = GPOINTER_TO_INT (ver_clist->selection->data);
         ver_list = gtk_object_get_data (GTK_OBJECT (ver_clist),
                                         VERSION_LIST);
         ver_num = GPOINTER_TO_INT (gtk_clist_get_row_data (ver_clist, row)); 
         ver = find_ver_num_in_list (ver_num, ver_list);

         /*
          * If the version text has changed, the user can either add or
          * modify.
          */
         if (strcmp (ver_entry_str, ver->version_str->str) != 0)
            {
            add_sensitive = TRUE;
            mod_sensitive = TRUE;
            }

         /*
          * In addition, check the toggle button and the descr.  If these
          * have been changed, the user may modify.
          */
         if (strcmp (ver_descr_str, ver->descr_str->str) != 0  ||
               ver->is_active != gtk_toggle_button_get_active (active_cb))
            {
            mod_sensitive = TRUE;
            }
         }
      }
   else
      {
      if (strlen (ver_entry_str) > 0 && strlen (ver_descr_str) > 0)
         {
         add_sensitive = TRUE;
         }
      }

   gtk_widget_set_sensitive (add_ver_btn, add_sensitive);
   gtk_widget_set_sensitive (mod_ver_btn, mod_sensitive);
   gtk_widget_set_sensitive (del_ver_btn, del_sensitive);
}


static void
on_ver_clist_unsel (GtkCList        *clist,
                    gint             row,
                    gint             col,
                    GdkEventButton  *event,
                    gpointer         user_data)
{
   GtkEntry             *ver_entry;
   GtkEntry             *ver_descr_entry;
   GtkToggleButton      *ver_active_cb;
   GtkWidget            *add_ver_btn;
   GtkWidget            *mod_ver_btn;

   ver_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), VER_ENTRY));
   g_assert (ver_entry != NULL);
   ver_descr_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), 
                                            VER_DESCR_ENTRY));
   g_assert (ver_descr_entry != NULL);
   ver_active_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                  VER_ACTIVE_CB));
   g_assert (ver_active_cb != NULL);
   add_ver_btn = get_widget (GTK_WIDGET (user_data), ADD_VER_BUTTON);
   g_assert (add_ver_btn != NULL);
   mod_ver_btn = get_widget (GTK_WIDGET (user_data), MOD_VER_BUTTON);
   g_assert (mod_ver_btn != NULL);

   gtk_entry_set_text (ver_entry, "");
   gtk_entry_set_text (ver_descr_entry, "");
   gtk_toggle_button_set_active (ver_active_cb, TRUE);
   gtk_widget_set_sensitive (add_ver_btn, FALSE);
   gtk_widget_set_sensitive (mod_ver_btn, FALSE);
}


static void
on_ver_clist_select (GtkCList        *clist,
                     gint             row,
                     gint             col,
                     GdkEventButton  *event,
                     gpointer         user_data)
{
   gint                  ver_num;
   PrjVerStruct         *ver;
   GList                *ver_list;
   GtkEntry             *ver_entry;
   GtkEntry             *ver_descr_entry;
   GtkToggleButton      *ver_active_cb;

   /*
    * Get the selection and display it in the edit controls.
    */
   ver_list = gtk_object_get_data (GTK_OBJECT (clist), VERSION_LIST);
   ver_num = GPOINTER_TO_INT (gtk_clist_get_row_data (clist, row)); 
   ver = find_ver_num_in_list (ver_num, ver_list);

   if (ver)
      {
      ver_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), VER_ENTRY));
      g_assert (ver_entry);
      ver_descr_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), 
                                               VER_DESCR_ENTRY));
      g_assert (ver_descr_entry);
      ver_active_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                     VER_ACTIVE_CB));
      g_assert (ver_active_cb);

      gtk_entry_set_text (ver_entry, ver->version_str->str);
      gtk_entry_set_text (ver_descr_entry, ver->descr_str->str);
      gtk_toggle_button_set_active (ver_active_cb, ver->is_active);
      }
}


static void
on_add_version_clicked (GtkButton *button, gpointer user_data)
{
   GtkCheckButton       *ver_active_cb;
   GtkCheckButton       *show_inactive_cb;
   GtkEntry             *ver_entry;
   GtkEntry             *descr_entry;
   GtkCList             *ver_clist;
   GList                *ver_list;
   PrjVerStruct         *prj_ver;
   
   /*
    * Get pointers to the widgets and other data that we are going to
    * need.  The widgets are stored in the parent dlg widget.  The version
    * list is stored with the version CList.  The version info list may be
    * NULL, so it is not g_asserted.
    */
   ver_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data),
                                      VER_ENTRY));
   g_assert (ver_entry != NULL);
   descr_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data),
                                        VER_DESCR_ENTRY));
   g_assert (descr_entry != NULL);
   ver_active_cb = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (user_data),
                                     VER_ACTIVE_CB));
   g_assert (ver_active_cb != NULL);
   ver_clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), VER_CLIST));
   g_assert (ver_clist != NULL);
   show_inactive_cb = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                    SHOW_ALL_VER_CB));
   g_assert (show_inactive_cb != NULL);
   ver_list = gtk_object_get_data (GTK_OBJECT (ver_clist), VERSION_LIST);

   prj_ver = g_malloc (sizeof (PrjVerStruct));
   prj_ver->version_str = g_string_new (gtk_entry_get_text (ver_entry));
   prj_ver->descr_str = g_string_new (gtk_entry_get_text (descr_entry));
   prj_ver->is_active =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ver_active_cb));
   prj_ver->version_num = get_max_ver_num (ver_list) + 1;
   prj_ver->order_num = get_max_order_num (ver_list) + 1;
   prj_ver->action = INSERT;
   if (validate_version_info (prj_ver, ver_list))
      {
      gtk_object_remove_data (GTK_OBJECT (ver_clist), VERSION_LIST);
      ver_list = g_list_append (ver_list, prj_ver);
      gtk_object_set_data (GTK_OBJECT (ver_clist), VERSION_LIST, ver_list);
      draw_version_list (ver_clist, 
                         gtk_toggle_button_get_active
                            (GTK_TOGGLE_BUTTON (show_inactive_cb)));
      update_version_combo (GTK_WIDGET (user_data));
      gtk_entry_set_text (ver_entry, "");
      gtk_entry_set_text (descr_entry, "");
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ver_active_cb), TRUE);
      set_clean (GTK_WIDGET (ver_entry), NULL);
      set_clean (GTK_WIDGET (descr_entry), NULL);
      set_clean (GTK_WIDGET (ver_active_cb), NULL);
      }
   else
      {
      destroy_project_version (prj_ver);
      }
}
 
                  
static void
on_mod_version_clicked (GtkButton *button, gpointer user_data)
{
   GtkCList             *ver_clist;
   GList                *ver_list;
   gint                  ver_num;
   GtkEntry             *ver_entry;
   GtkEntry             *descr_entry;
   GtkToggleButton      *ver_active_cb;
   GtkToggleButton      *show_inactive_cb;
   PrjVerStruct         *ver;      /* The version being modified */
   gint                  pos;      /* The position ver has in ver_list */
   PrjVerStruct         *tmp_ver;  /* Temp holding for new data  */
   GtkWidget            *msg_dlg;

   ver_clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), VER_CLIST));
   g_assert (ver_clist != NULL);
   ver_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), VER_ENTRY));
   g_assert (ver_entry != NULL);
   descr_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data),
                                        VER_DESCR_ENTRY));
   g_assert (descr_entry != NULL);
   ver_active_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                  VER_ACTIVE_CB)); 
   g_assert (ver_active_cb != NULL);
   show_inactive_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                     SHOW_ALL_VER_CB));
   g_assert (show_inactive_cb != NULL);

   if (!ver_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a version in order to modify it."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      return;
      }

   /*
    * We should never get this far without having something in the version
    * list, so g_assert that.
    */
   ver_list = gtk_object_get_data (GTK_OBJECT (ver_clist), VERSION_LIST);
   g_assert (ver_list);
   ver_num = GPOINTER_TO_INT
      (gtk_clist_get_row_data (ver_clist,
                               GPOINTER_TO_INT (ver_clist->selection->data)));
   ver = find_ver_num_in_list (ver_num, ver_list);
   g_assert (ver);

   /*
    * Temporarily remove the version from the version list so we can
    * validate the changes in the "normal" way.  We will then either
    * restore the old data if the validation of the change fails, or 
    * replace the data if the validiation succeeds.
    */
   pos = g_list_index (ver_list, ver);
   ver_list = g_list_remove (ver_list, ver);
   tmp_ver = g_malloc (sizeof (PrjVerStruct));
   tmp_ver->version_str = g_string_new (gtk_entry_get_text (ver_entry));
   tmp_ver->descr_str = g_string_new (gtk_entry_get_text (descr_entry));
   tmp_ver->is_active = gtk_toggle_button_get_active (ver_active_cb);
   if (validate_version_info (tmp_ver, ver_list))
      {
      ver->version_str = g_string_assign (ver->version_str,
                                          tmp_ver->version_str->str);
      ver->descr_str = g_string_assign (ver->descr_str,
                                        tmp_ver->descr_str->str);
      ver->is_active = tmp_ver->is_active;
      ver->action = (ver->action == INSERT) ? INSERT : UPDATE;

      gtk_entry_set_text (ver_entry, "");
      gtk_entry_set_text (descr_entry, "");
      gtk_toggle_button_set_active (ver_active_cb, TRUE);
      set_clean (GTK_WIDGET (ver_entry), NULL);
      set_clean (GTK_WIDGET (descr_entry), NULL);
      set_clean (GTK_WIDGET (ver_active_cb), NULL);
      }

   destroy_project_version (tmp_ver);
   ver_list = g_list_insert (ver_list, ver, pos);
   draw_version_list (ver_clist, 
         gtk_toggle_button_get_active (show_inactive_cb));
   update_version_combo (GTK_WIDGET (user_data));
}


static void
on_del_version_clicked (GtkButton *button, gpointer user_data)
{
   GtkCList             *ver_clist;
   GList                *ver_list;
   GtkEntry             *ver_entry;
   GtkEntry             *descr_entry;
   GtkToggleButton      *ver_active_cb;
   gint                  ver_num;
   GtkToggleButton      *show_inactive_cb;
   GString              *sql_buffer;
   PGresult             *res;
   PrjVerStruct         *ver;
   PGconn               *conn;
   gint                  pk_num;
   GtkWidget            *msg_dlg;

   ver_clist = GTK_CLIST (get_widget (GTK_WIDGET (user_data), VER_CLIST));
   g_assert (ver_clist);
   
   /*
    * Before deleting, do a couple of checks.  First, make sure something
    * has been selected to delete.  Second, check with the user.  These
    * deletes are perminent, so make sure they want to do them.
    */
   if (!ver_clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("You must select a version in order to delete it."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      return;
      }

   msg_dlg = gnome_message_box_new
      (_("Deleting a version cannot be cancelled or reversed.\n"
         "Are you sure that you want to delete the selected version?"),
       GNOME_MESSAGE_BOX_QUESTION,
       GNOME_STOCK_BUTTON_YES,
       GNOME_STOCK_BUTTON_NO,
       NULL);
   gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
   if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 1)
      {
      return;
      }
          

   /*
    * We should never get this far without having something in the version
    * list, so g_assert that.
    */
   ver_list = gtk_object_get_data (GTK_OBJECT (ver_clist), VERSION_LIST);
   g_assert (ver_list != NULL);
   ver_num = GPOINTER_TO_INT
      (gtk_clist_get_row_data (ver_clist,
                               GPOINTER_TO_INT (ver_clist->selection->data)));
   ver = find_ver_num_in_list (ver_num, ver_list);
   g_assert (ver);
   show_inactive_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                     SHOW_ALL_VER_CB));
   g_assert (show_inactive_cb != NULL);

   /*
    * If this was one of the new versions (we have not added it to the
    * database yet), just remove it from the version list.  If this
    * is a version that is already in the database, attempt to remove
    * it (there should be some validation triggers there), and only
    * remove it from the list if the removal from the database works.
    */
   if (ver->action == INSERT)
      {
      gtk_object_remove_data (GTK_OBJECT (ver_clist), VERSION_LIST);
      ver_list = g_list_remove (ver_list, ver);
      destroy_project_version (ver);
      gtk_object_set_data (GTK_OBJECT (ver_clist), VERSION_LIST, ver_list);
      }
   else
      {
      conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
      g_assert (conn != NULL);
      pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (user_data),
                                                     PK_NUM));
      sql_buffer = g_string_new ("");
      g_string_sprintf (sql_buffer, DELETE_VERSION,
                        ver->version_num, pk_num);
      START_TRANSACTION (conn);
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, _("Deleting version")))
         {
         gtk_object_remove_data (GTK_OBJECT (ver_clist), VERSION_LIST);
         ver_list = g_list_remove (ver_list, ver);
         destroy_project_version (ver);
         gtk_object_set_data (GTK_OBJECT (ver_clist), VERSION_LIST, ver_list);
         COMMIT (conn);
         }
      else
         {
         ROLLBACK (conn);
         }
      PQclear (res);
      g_string_free (sql_buffer, TRUE);
      }
   draw_version_list (ver_clist,
         gtk_toggle_button_get_active (show_inactive_cb));
   update_version_combo (GTK_WIDGET (user_data));

   ver_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data), VER_ENTRY));
   g_assert (ver_entry != NULL);
   descr_entry = GTK_ENTRY (get_widget (GTK_WIDGET (user_data),
                                        VER_DESCR_ENTRY));
   g_assert (descr_entry != NULL);
   ver_active_cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (user_data),
                                                  VER_ACTIVE_CB)); 
   g_assert (ver_active_cb != NULL);
   gtk_entry_set_text (ver_entry, "");
   gtk_entry_set_text (descr_entry, "");
   gtk_toggle_button_set_active (ver_active_cb, TRUE);
   set_clean (GTK_WIDGET (ver_entry), NULL);
   set_clean (GTK_WIDGET (descr_entry), NULL);
   set_clean (GTK_WIDGET (ver_active_cb), NULL);
}


static gboolean
process_version_list (GtkWidget *main_widget, gint prj_num)
{
   GList           *list_iter;
   GList           *full_list;
   GList           *ver_list;
   name_pk_struct  *cur_el;
   GtkCList        *ver_clist;
   GString         *sql_buffer = NULL;
   GString         *err_str = NULL;
   PGresult        *res;
   PrjVerStruct    *cur_ver;
   gboolean         error = FALSE;
   PGconn          *conn;

   conn = gtk_object_get_data (GTK_OBJECT (main_widget), MY_DB_CONN);
   g_assert (conn != NULL);
   ver_clist = GTK_CLIST (get_widget (main_widget, VER_CLIST));
   g_assert (ver_clist != NULL);
   full_list = gtk_object_get_data (GTK_OBJECT (ver_clist),
                                    VERSION_NAME_PK_LIST);
   ver_list = gtk_object_get_data (GTK_OBJECT (ver_clist), VERSION_LIST);
   list_iter = g_list_first (ver_list);
   while (list_iter && !error)
      {
      cur_ver = (PrjVerStruct *)list_iter->data;
      list_iter = list_iter->next;

      /*
       * Build the SQL statement based on what needs to be done.
       */
      switch (cur_ver->action)
         {
         case INSERT:
            /*
             * For the insert, we need to first get the next available
             * version_num.  The one we have been using to this point
             * has just been a unique guess.
             */
            cur_el = find_name_pk_from_pk (full_list, cur_ver->version_num);
            g_assert (cur_el);
            cur_el->pk = get_next_version_pk (conn, prj_num);
            sql_buffer = g_string_new ("");
            g_string_sprintf (sql_buffer, 
                              INSERT_VERSION,
                              cur_el->pk, 
                              prj_num,
                              cur_ver->version_str->str, 
                              cur_ver->descr_str->str,
                              (cur_ver->is_active) ? "TRUE" : "FALSE",
                              cur_ver->order_num);
            err_str = g_string_new (_("Inserting version: "));
            err_str = g_string_append (err_str, cur_ver->version_str->str);
            break;

         case UPDATE:
            sql_buffer = g_string_new ("");
            g_string_sprintf (sql_buffer, UPDATE_VERSION,
                              cur_ver->version_str->str,
                              cur_ver->descr_str->str,
                              (cur_ver->is_active) ? "TRUE" : "FALSE",
                              cur_ver->order_num, 
                              cur_ver->version_num, 
                              prj_num);
            err_str = g_string_new (_("Updating version: "));
            err_str = g_string_append (err_str, cur_ver->version_str->str);
            break;

         case DELETE:
            sql_buffer = g_string_new ("");
            g_string_sprintf (sql_buffer, DELETE_VERSION,
                              cur_ver->version_num, prj_num);
            err_str = g_string_new (_("Deleting version: "));
            err_str = g_string_append (err_str, cur_ver->version_str->str);
            break;

         default:
            /*
             * If'n we are here, there really shouldn't be anything
             * that needs to be done.
             */
            break;
         }

      cur_ver->action = NO_ACTION;
      if (sql_buffer)
         {
         res = PQexec (conn, sql_buffer->str);
         error = !chk_sql_error (res, err_str->str); 
         g_string_free (sql_buffer, TRUE);
         g_string_free (err_str, TRUE);
         sql_buffer = NULL;
         err_str = NULL;
         PQclear (res);
         }
      }

   return !error;
}


/*
 * update_def_versions
 *
 * Update the default open and close versions for the current project.
 * This needs to be done ONLY after the project itself has been inserted
 * or updated, and the list of version data (VERSION_LIST) has been 
 * processed.
 */
static gboolean
update_def_versions (GtkWidget *main_widget, gint prj_num)
{
   GList     *full_list;
   GtkCList  *ver_clist;
   GtkEntry  *combo_entry;
   gint       def_open_ver;
   gint       def_close_ver;
   gboolean   def_open_dirty;
   gboolean   def_close_dirty;
   GString   *sql_buffer;
   gchar      scratchpad[NUM_BUFFER_LEN];
   gboolean   ok = TRUE;
   PGresult  *res;
   PGconn    *conn;

   conn = gtk_object_get_data (GTK_OBJECT (main_widget), MY_DB_CONN);
   g_assert (conn != NULL);
   /*
    * Get the version/pk list that is attached to the version clist.
    * At this time, it should contain the same PK's as is in the database.
    */
   ver_clist = GTK_CLIST (get_widget (main_widget, VER_CLIST));
   g_assert (ver_clist != NULL);
   full_list = gtk_object_get_data (GTK_OBJECT (ver_clist),
                                    VERSION_NAME_PK_LIST);

   /*
    * Get the PK's for the selected versions.  INVALID_PK coinsides
    * either with '<None>' or some kind of invalid data.  In that case,
    * use NULL.
    */
   combo_entry = GTK_ENTRY (get_widget (main_widget, DEF_CLOSE_ENTRY));
   def_close_dirty = widget_is_dirty (GTK_WIDGET (combo_entry));
   set_clean (GTK_WIDGET (combo_entry), NULL);
   def_close_ver = find_pk_from_name (full_list, 
                                      gtk_entry_get_text (combo_entry));
   combo_entry = GTK_ENTRY (get_widget (main_widget, DEF_OPEN_ENTRY));
   def_open_dirty = widget_is_dirty (GTK_WIDGET (combo_entry));
   set_clean (GTK_WIDGET (combo_entry), NULL);
   def_open_ver = find_pk_from_name (full_list,
                                     gtk_entry_get_text (combo_entry));

   if (!(def_close_dirty || def_open_dirty))
      {
      return ok;
      }

   /*
    * Build the SQL string:
    *   UPDATE project
    *      SET open_version = def_open_ver,
    *          close_version = def_close_ver
    *    WHERE project_num = prj_num
    */
   sql_buffer = g_string_new ("UPDATE project SET ");
   if (def_open_dirty)
      {
      sql_buffer = g_string_append (sql_buffer, "open_version = ");
      if (def_open_ver == INVALID_PK)
         {
         sql_buffer = g_string_append (sql_buffer, "NULL");
         }
      else
         {
         sprintf (scratchpad, "%d", def_open_ver);
         sql_buffer = g_string_append (sql_buffer, scratchpad);
         }
      if (def_close_dirty)
         {
         sql_buffer = g_string_append (sql_buffer, ", ");
         }
      }
   if (def_close_dirty)
      {
      sql_buffer = g_string_append (sql_buffer, "close_version = ");
      if (def_close_ver == INVALID_PK)
         {
         sql_buffer = g_string_append (sql_buffer, "NULL");
         }
      else
         {
         sprintf (scratchpad, "%d", def_close_ver);
         sql_buffer = g_string_append (sql_buffer, scratchpad);
         }
      }
   sql_buffer = g_string_append (sql_buffer, " WHERE project_num = ");
   sprintf (scratchpad, "%d", prj_num);
   sql_buffer = g_string_append (sql_buffer, scratchpad);

   res = PQexec (conn, sql_buffer->str);
   if (!chk_sql_error (res, _("Updating default versions")))
      {
      ok = FALSE;
      }

   PQclear (res);
   g_string_free (sql_buffer, TRUE);

   return ok;
}


static GString *
build_prj_insert_str (GnomeDialog *dlg, gint pk_num)
{
   GtkEntry        *entry;
   GtkText         *text;
   GtkCombo        *combo;
   GtkCheckButton  *checkbox;
   GString         *str;
   GString         *tmp;
   gchar           *char_buf;
   GList           *list;

   tmp = g_string_new ("");
   str = g_string_new ("INSERT INTO project (project_num, name, active, ");
   str = g_string_append (str, "login_id, descr, new_pr_notify_grp, ");
   str = g_string_append (str, "new_pr_notify_pl, assign_notify_resp, ");
   str = g_string_append (str, "assign_notify_sub, assign_notify_pl, ");
   str = g_string_append (str, "upd_notify_grp, upd_notify_pl, ");
   str = g_string_append (str, "upd_notify_resp, upd_notify_sub) ");

   str = g_string_append (str, "VALUES (");

   g_string_sprintf (tmp, "%d", pk_num);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), NAME_ENTRY));
   g_assert (entry != NULL);
   tmp = g_string_assign (tmp, gtk_entry_get_text (entry));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg), ACTIVE_CB));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   combo = GTK_COMBO (get_widget (GTK_WIDGET (dlg), LEADER_COMBO));
   g_assert (combo != NULL);
   list = gtk_object_get_data (GTK_OBJECT (combo), LEADER_LIST);
   g_assert (list != NULL);
   tmp = g_string_assign
      (tmp, 
       find_id_from_name (list,
                          gtk_entry_get_text (GTK_ENTRY (combo->entry))));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (text != NULL);
   char_buf = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
   tmp = g_string_new (char_buf);
   g_free (char_buf);
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                NEW_PR_NOTIFY_GRP));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                NEW_PR_NOTIFY_PL));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                ASSIGN_NOTIFY_RESP));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                ASSIGN_NOTIFY_SUB));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                ASSIGN_NOTIFY_PL));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_GRP));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_PL));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_RESP));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_SUB));
   g_assert (checkbox != NULL);
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
      {
      str = g_string_append (str, "TRUE) ");
      }
   else
      {
      str = g_string_append (str, "FALSE) ");
      }

   g_string_free (tmp, TRUE);

   return str;
}

static GString *
build_prj_update_str (GnomeDialog *dlg, gint pk_num)
{
   GtkEntry        *entry;
   GtkText         *text;
   GtkCombo        *combo;
   GtkCheckButton  *checkbox;
   GString         *upd;
   GString         *tmp;
   gchar           *char_buf;
   GList           *list;
   gboolean         first = TRUE;

   upd = g_string_new ("UPDATE project SET ");
   tmp = g_string_new ("");

   entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), NAME_ENTRY));
   g_assert (entry != NULL);
   if (widget_is_dirty (GTK_WIDGET (entry)))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (entry));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "NAME = ");
      upd = g_string_append (upd, tmp->str);
      }

   text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (text != NULL);
   if (widget_is_dirty (GTK_WIDGET (text)))
      {
      char_buf = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
      tmp = g_string_new (char_buf);
      g_free (char_buf);
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "DESCR = ");
      upd = g_string_append (upd, tmp->str);
      }

   combo = GTK_COMBO (get_widget (GTK_WIDGET (dlg), LEADER_COMBO));
   g_assert (combo != NULL);
   list = gtk_object_get_data (GTK_OBJECT (combo), LEADER_LIST);
   g_assert (list != NULL);
   if (widget_is_dirty (GTK_WIDGET (combo->entry)))
      {
      tmp = g_string_assign (tmp, 
          find_id_from_name (list,
                             gtk_entry_get_text (GTK_ENTRY (combo->entry))));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "LOGIN_ID = ");
      upd = g_string_append (upd, tmp->str);
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg), ACTIVE_CB));
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "ACTIVE = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                NEW_PR_NOTIFY_GRP)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "NEW_PR_NOTIFY_GRP = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                NEW_PR_NOTIFY_PL)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "NEW_PR_NOTIFY_PL = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                ASSIGN_NOTIFY_RESP)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "ASSIGN_NOTIFY_RESP = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                ASSIGN_NOTIFY_SUB)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "ASSIGN_NOTIFY_SUB = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                ASSIGN_NOTIFY_PL)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "ASSIGN_NOTIFY_PL = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_GRP)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "UPD_NOTIFY_GRP = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_PL)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "UPD_NOTIFY_PL = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_RESP)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "UPD_NOTIFY_RESP = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   checkbox = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (dlg),
                                UPD_NOTIFY_SUB)); 
   g_assert (checkbox != NULL);
   if (widget_is_dirty (GTK_WIDGET (checkbox)))
      {
      if (!first)
         {
         upd = g_string_append (upd, ", ");
         }
      first = FALSE;
      upd = g_string_append (upd, "UPD_NOTIFY_SUB = ");
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox)))
         {
         upd = g_string_append (upd, "TRUE");
         }
      else
         {
         upd = g_string_append (upd, "FALSE");
         }
      }

   upd = g_string_append (upd, " WHERE PROJECT_NUM = ");
   g_string_sprintf (tmp, "%d", pk_num);
   upd = g_string_append (upd, tmp->str);

   g_string_free (tmp, TRUE);

   return upd;
}


static gboolean 
apply_db_changes (GnomeDialog *dlg)
{
   GtkCombo            *auto_assign_combo;
   GtkCList            *submitter_list;
   GtkCList            *member_list;
   GList               *sel_list;
   PGresult            *res;
   gint                 pk_num;
   gboolean             in_error = FALSE;
   PGconn              *conn;
   gboolean             is_update;
   GString             *sqlstr;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                      IS_UPDATE)) == 1);

   auto_assign_combo = GTK_COMBO (get_widget (GTK_WIDGET (dlg),
                                              AUTO_ASSIGN_COMBO));
   g_assert (auto_assign_combo != NULL);
   submitter_list = GTK_CLIST (get_widget (GTK_WIDGET (dlg),
                                           SUBMITTER_LIST));
   g_assert (submitter_list != NULL);
   member_list  = GTK_CLIST (get_widget (GTK_WIDGET (dlg),
                                         PRJ_MEMBER_LIST));
   g_assert (member_list != NULL);

   START_TRANSACTION (conn);

   /*
    * To get into this function, either this must be an update, or 
    * at least some of the "main SQL" related widgets must be dirty.
    * Otherwise, validation should not have been passed.
    *
    * In other words, their is no reason to check for dirty widgets
    * in the else case, since the widgets have to be dirty, other wise
    * we should not have passed validation.
    */
   g_assert (is_update || widgets_children_are_dirty (GTK_WIDGET (dlg)));
   if (is_update)
      {
      pk_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                     PK_NUM)); 
      if (widgets_children_are_dirty (GTK_WIDGET (dlg)))
         {
         sqlstr = build_prj_update_str (dlg, pk_num);
         res = PQexec (conn, sqlstr->str);
         in_error = !chk_sql_error (res, _("updating project data"));
         PQclear (res);
         g_string_free (sqlstr, TRUE);
         }
      }
   else
      {
      res = PQexec (conn, "select nextval ('project_seq')");
      pk_num = atoi (PQgetvalue (res, 0, 0));
      PQclear (res);
      sqlstr = build_prj_insert_str (dlg, pk_num);
      res = PQexec (conn, sqlstr->str);
      in_error = !chk_sql_error (res, _("inserting project data"));
      PQclear (res);
      g_string_free (sqlstr, TRUE);
      }
   
   /*
    * Process the project_version list, and then update the open and
    * close versions.
    */
   if (!in_error)
      {
      in_error = !process_version_list (GTK_WIDGET (dlg), pk_num);
      if (!in_error)
         {
         in_error = !update_def_versions (GTK_WIDGET (dlg), pk_num);
         }
      }

   /*
    * Add and remove users according to the "add" and "remove" lists
    * that were built in response to the selections and unselections
    * done by the user.
    *
    * When we are done processing the lists, they should be destroyed.
    */
   sel_list = gtk_object_get_data (GTK_OBJECT (submitter_list), RM_LIST);
   while (!in_error && sel_list != NULL)
      {
      in_error = !rm_submitter (conn, pk_num, sel_list->data);
      sel_list = sel_list->next;
      }
   sel_list = gtk_object_get_data (GTK_OBJECT (submitter_list), ADD_LIST);
   while (!in_error && sel_list != NULL)
      {
      in_error = !insert_submitter (conn, pk_num, sel_list->data);
      sel_list = sel_list->next;
      }
   destroy_rm_add_lists (submitter_list);

   sel_list = gtk_object_get_data (GTK_OBJECT (member_list), RM_LIST);
   while (!in_error && sel_list != NULL)
      {
      in_error = !rm_responsible (conn, pk_num, sel_list->data);
      sel_list = sel_list->next;
      }
   sel_list = gtk_object_get_data (GTK_OBJECT (member_list), ADD_LIST);
   while (!in_error && sel_list != NULL)
      {
      in_error = !insert_responsible (conn, pk_num, sel_list->data);
      sel_list = sel_list->next;
      }
   destroy_rm_add_lists (member_list);

   /*
    * This is always done here because the resposible table must be
    * updated first due to RI.
    */
   if (!in_error && widget_is_dirty (GTK_WIDGET (auto_assign_combo->entry)))
      {
      in_error = !update_auto_assign (conn, pk_num, auto_assign_combo);
      if (!in_error)
         {
         set_clean (GTK_WIDGET (auto_assign_combo), NULL);
         }
      }

   if (!in_error)
      {
      COMMIT (conn);
      set_all_clean (GTK_WIDGET (dlg));
      gnome_dialog_set_sensitive (dlg, APPLY_BTN, FALSE);
      /*
       * If the user did an apply, and we just did an insert, then
       * we should now be in update mode.  If we already were in 
       * update mode, we should remain there.  If the user pressed
       * OK, it does not matter because we are leaving.  In the end, 
       * either we need to be in update mode, or it does not matter 
       * what mode we are in.  Thus, just set the darn mode to update
       * and be done with it rather then checking a bunch of states
       * first...
       */
      gtk_object_set_data (GTK_OBJECT (dlg), IS_UPDATE, GINT_TO_POINTER (1));
      gtk_object_set_data (GTK_OBJECT (dlg), PK_NUM,
                           GINT_TO_POINTER (pk_num));
      }
   else
      {
      ROLLBACK (conn);
      }

   return !in_error;
}


static void
on_ok_button_clicked (GnomeDialog *dlg)
{
   GtkWidget  *parent;
   GtkWidget  *main_win;
   PGconn     *conn;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   parent = gtk_object_get_data (GTK_OBJECT (dlg), MY_PARENT);
   g_assert (parent != NULL);
   /*
    * The main win should be the parent of the project list dialog, 
    * which should be the parent of this edit dialog.
    */
   main_win = get_widget (parent, MY_PARENT);
   g_assert (main_win != NULL);

   if (validate_project (GTK_WIDGET (dlg)))
      {
      if (apply_db_changes (dlg))
         {
         /*
          * refresh the project list dialog (draw_project_list()), AND
          * on the main window (refresh_project_list()).
          */
         draw_project_list (GTK_WINDOW (parent), conn);
         refresh_project_list (main_win);
         gnome_dialog_close (dlg);
         }
      }
}

static void
on_apply_button_clicked (GnomeDialog *dlg)
{
   GtkWidget  *parent;
   GtkWidget  *main_win;
   PGconn     *conn;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   parent = gtk_object_get_data (GTK_OBJECT (dlg), MY_PARENT);
   g_assert (parent != NULL);
   /*
    * The main win should be the parent of the project list dialog, 
    * which should be the parent of this edit dialog.
    */
   main_win = get_widget (parent, MY_PARENT);
   g_assert (main_win != NULL);

   if (validate_project (GTK_WIDGET (dlg)))
      {
      if (apply_db_changes (dlg))
         {
         /*
          * refresh the project list dialog (draw_project_list()), AND
          * on the main window (refresh_project_list()).
          */
         draw_project_list (GTK_WINDOW (parent), conn);
         refresh_project_list (main_win);
         }
      }
}


static void 
on_cancel_button_clicked (GnomeDialog  *dlg)
{
   gnome_dialog_close (dlg);
}


static void
on_project_edit_dlg_clicked (GnomeDialog *dlg,
                             gint         btn_cd,
                             gpointer     user_data)
{
   switch (btn_cd)
      {
      case OK_BTN:
         on_ok_button_clicked (dlg);
         break;
      case APPLY_BTN:
         on_apply_button_clicked (dlg);
         break;
      case CANCEL_BTN:
         on_cancel_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


static void
on_widget_contents_modified (GtkWidget *widget, gpointer user_data)
{
   GnomeDialog *dlg = GNOME_DIALOG (user_data);

   gnome_dialog_set_sensitive (dlg, APPLY_BTN, TRUE);
   set_dirty (widget, NULL);
}


static GtkWidget* 
create_project_edit_dlg (gboolean   is_update,
                         gint       pk_num,
                         PGconn    *conn,
                         GtkWidget *parent,
                         gboolean   user_is_admin)
{
  GtkWidget           *project_edit_dlg;
  GtkWidget           *dialog_vbox14;
  GtkWidget           *notebook2;
  GtkWidget           *vbox27;
  GtkWidget           *name_entry;
  GtkWidget           *descr_entry;
  GtkWidget           *hbox31;
  GtkWidget           *vbox28;
  GtkWidget           *leader_combo;
  GtkWidget           *active_cb;
  GtkWidget           *scrolledwindow6;
  GtkWidget           *submitter_vbox;
  GtkWidget           *submitter_hbox;
  GtkWidget           *submitter_clist;
  GtkWidget           *submitter_add_all;
  GtkWidget           *submitter_rm_all;
  GtkWidget           *label;
  GtkWidget           *scrollwindow7;
  GtkWidget           *prj_member_vbox;
  GtkWidget           *prj_member_hbox;
  GtkWidget           *prj_member_clist;
  GtkWidget           *prj_member_add_all;
  GtkWidget           *prj_member_rm_all;
  GtkWidget           *info_lbl;
  GtkWidget           *submitter_label;
  GtkWidget           *resp_lbl;
  GtkWidget           *vbox31;
  GtkWidget           *vbox32;
  GtkWidget           *hbox35;
  GtkWidget           *frame;
  GtkWidget           *new_pr_notify_grp_cb;
  GtkWidget           *new_pr_notify_pl_cb;
  GtkWidget           *vbox33;
  GtkWidget           *assign_pr_notify_resp_cb;
  GtkWidget           *assign_pr_notify_sub_cb;
  GtkWidget           *assign_pr_notify_pl_cb;
  GtkWidget           *vbox34;
  GtkWidget           *upd_pr_notify_grp_cb;
  GtkWidget           *upd_pr_notify_resp_cb;
  GtkWidget           *upd_pr_notify_sub_cb;
  GtkWidget           *upd_pr_notify_pl_cb;
  GtkWidget           *vbox35;
  GtkWidget           *auto_assign_combo;
  GtkWidget           *version_lbl;
  GtkWidget           *scrolledwindow14;
  GtkWidget           *version_clist;
  GtkWidget           *hseparator;
  GtkWidget           *table5;
  GtkWidget           *table6;
  GtkWidget           *ver_entry;
  GtkWidget           *ver_descr_entry;
  GtkWidget           *ver_active_cb;
  GtkWidget           *show_inactive_cb;
  GtkWidget           *hbuttonbox;
  GtkWidget           *ver_add_btn;
  GtkWidget           *ver_mod_btn;
  GtkWidget           *ver_del_btn;
  GtkWidget           *def_open_combo;
  GtkWidget           *def_open_entry;
  GtkWidget           *def_close_combo;
  GtkWidget           *def_close_entry;
  GtkWidget           *popup;
  GtkTooltips         *tooltips;
  GList               *name_list = NULL; /* just the names for the drop down */
  GList               *full_list = NULL; /* store display names keyed by ID */
  name_id_struct      *curr_person;
  gint                 i;
  gint                 n;
  PGresult            *res;
  gboolean             can_edit;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  if (is_update)
     {
     project_edit_dlg = gnome_dialog_new (_("Update Project"),
                                          GNOME_STOCK_BUTTON_OK,
                                          GNOME_STOCK_BUTTON_APPLY,
                                          GNOME_STOCK_BUTTON_CANCEL,
                                          NULL);
     }
  else
     {
     project_edit_dlg = gnome_dialog_new (_("New Project"),
                                          GNOME_STOCK_BUTTON_OK,
                                          GNOME_STOCK_BUTTON_APPLY,
                                          GNOME_STOCK_BUTTON_CANCEL,
                                          NULL);
     }
  gnome_dialog_set_parent (GNOME_DIALOG (project_edit_dlg), GTK_WINDOW (parent));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       PROJECT_EDIT_DIALOG, 
                       project_edit_dlg);
  gtk_container_border_width (GTK_CONTAINER (project_edit_dlg), 5);
  gtk_window_set_policy (GTK_WINDOW (project_edit_dlg), FALSE, FALSE, TRUE);
  gtk_signal_connect (GTK_OBJECT (project_edit_dlg), "show",
                      GTK_SIGNAL_FUNC (on_project_edit_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (project_edit_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_project_edit_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (project_edit_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_project_edit_dlg_clicked),
                      NULL);

  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), TOP_DOG,
                       project_edit_dlg);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), PK_NUM,
                       GINT_TO_POINTER (pk_num));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), IS_UPDATE,
                       GINT_TO_POINTER (is_update ? 1 : 0));
  if (is_update)
     {
     can_edit = can_edit_project (conn, pk_num);
     }
  else
     {
     can_edit = user_is_admin;
     }
  gnome_dialog_set_close (GNOME_DIALOG (project_edit_dlg), FALSE);
  if (can_edit)
     {
     gnome_dialog_set_default (GNOME_DIALOG (project_edit_dlg), OK_BTN);
     }
  else
     {
     gnome_dialog_set_default (GNOME_DIALOG (project_edit_dlg), CANCEL_BTN);
     gnome_dialog_set_sensitive (GNOME_DIALOG (project_edit_dlg),
                                 OK_BTN, FALSE);
     gnome_dialog_set_sensitive (GNOME_DIALOG (project_edit_dlg),
                                 APPLY_BTN, FALSE);
     }

  dialog_vbox14 = GNOME_DIALOG (project_edit_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox14), 5);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       "dialog_vbox14", 
                       dialog_vbox14);
  gtk_widget_show (dialog_vbox14);

  notebook2 = gtk_notebook_new ();
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "notebook2", notebook2);
  gtk_widget_show (notebook2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox14), notebook2, TRUE, TRUE, 0);

  vbox27 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "vbox27", vbox27);
  gtk_widget_show (vbox27);
  gtk_container_add (GTK_CONTAINER (notebook2), vbox27);
  gtk_container_set_border_width (GTK_CONTAINER (vbox27), 4);

  label = gtk_label_new (_("Project Name:"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label11", label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox27), label, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), NAME_ENTRY, name_entry);
  gtk_widget_show (name_entry);
  gtk_box_pack_start (GTK_BOX (vbox27), name_entry, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, name_entry, 
                        _("The name of the project (required)."), NULL);
  gtk_widget_set_sensitive (name_entry, can_edit);
  gtk_signal_connect (GTK_OBJECT (name_entry), "changed",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, name_entry);
  popup = attach_generic_edit_popup_menu (name_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "popup1", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  label = gtk_label_new (_("Description:"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label12", label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox27), label, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  descr_entry = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       DESCR_TEXT, 
                       descr_entry);
  gtk_widget_show (descr_entry);
  gtk_box_pack_start (GTK_BOX (vbox27), descr_entry, TRUE, TRUE, 0);
  gtk_widget_set_usize (descr_entry, 400, -2);
  gtk_text_set_editable (GTK_TEXT (descr_entry), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (descr_entry), TRUE);
  gtk_tooltips_set_tip (tooltips, 
                        descr_entry, 
                        _("A short description of the project (optional)."), 
                        NULL);
  gtk_widget_set_sensitive (descr_entry, can_edit);
  gtk_signal_connect (GTK_OBJECT (descr_entry), "changed",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, descr_entry);
  popup = attach_generic_edit_popup_menu (descr_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "popup2", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  hbox31 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "hbox31", hbox31);
  gtk_widget_show (hbox31);
  gtk_box_pack_start (GTK_BOX (vbox27), hbox31, TRUE, TRUE, 0);

  vbox28 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "vbox28", vbox28);
  gtk_widget_show (vbox28);
  gtk_box_pack_start (GTK_BOX (hbox31), vbox28, TRUE, TRUE, 0);

  label = gtk_label_new (_("Project Leader:"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label10", label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox28), label, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  leader_combo = gtk_combo_new ();
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       LEADER_COMBO, 
                       leader_combo);
  gtk_widget_set_sensitive (leader_combo, can_edit);
  /*
   *  Create a list containing the display data keyed by the LOGIN ID.
   *  This makes it easy to determine the proper key given the displayed
   *  name, and vice versa.
   *  
   *  The login ID has been included in the display name it ensure 
   *  a unique display name.
   */
  res = read_person_data (conn);
  if (chk_sql_error (res, _("getting person data")))
     {
     n = PQntuples (res);
     for (i = 0; i < n; i++)
        {
        curr_person = g_malloc (sizeof (name_id_struct));
        curr_person->id = (gchar *)g_malloc (strlen 
              (PQgetvalue (res, i, LOGIN_ID_POS)) + 1);
        curr_person->name = (gchar *)g_malloc 
           (strlen (PQgetvalue (res, i, LAST_NAME_POS)) +
            strlen (PQgetvalue (res, i, FIRST_NAME_POS)) +
            strlen (PQgetvalue (res, i, LOGIN_ID_POS)) + 6);
        strcpy (curr_person->id, PQgetvalue (res, i, LOGIN_ID_POS));
        sprintf (curr_person->name, "%s, %s (%s)",
                 PQgetvalue (res, i, LAST_NAME_POS),
                 PQgetvalue (res, i, FIRST_NAME_POS),
                 curr_person->id);
        name_list = g_list_append (name_list, curr_person->name);
        full_list = g_list_append (full_list, curr_person);
        }
     gtk_combo_set_popdown_strings (GTK_COMBO (leader_combo),
                                    name_list);
     gtk_object_set_data (GTK_OBJECT (leader_combo),
                          LEADER_LIST,
                          full_list);
     g_list_free (name_list);
     }
  PQclear (res);
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (leader_combo)->entry),
                          FALSE);
  gtk_widget_show (leader_combo);
  gtk_box_pack_start (GTK_BOX (vbox28), leader_combo, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (leader_combo)->entry),
                      "changed",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, GTK_COMBO (leader_combo)->entry);

  active_cb = gtk_check_button_new_with_label ("Active");
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), ACTIVE_CB, active_cb);
  gtk_widget_show (active_cb);
  gtk_box_pack_start (GTK_BOX (hbox31), active_cb, TRUE, TRUE, 0);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (active_cb), TRUE);
  gtk_widget_set_sensitive (active_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (active_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, active_cb);

  submitter_vbox = gtk_vbox_new (FALSE, 5);
  gtk_widget_ref (submitter_vbox);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "submitter_vbox",
                            submitter_vbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (submitter_vbox);
  gtk_container_add (GTK_CONTAINER (notebook2), submitter_vbox);
  gtk_container_set_border_width (GTK_CONTAINER (submitter_vbox), 4);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow6);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "scrolledwindow6", 
                            scrolledwindow6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow6);
  gtk_box_pack_start (GTK_BOX (submitter_vbox), scrolledwindow6, 
                      TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  /*
   * This object has several signal handlers that handle selections and
   * unselection.  However, they are not attached until the list is filled,
   * and the currenct project memebers are highlighted.  This is done in
   * on_project_edit_dlg_show().
   */
  submitter_clist = gtk_clist_new (3);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       SUBMITTER_LIST, 
                       submitter_clist);
  gtk_widget_show (submitter_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), submitter_clist);
  gtk_clist_set_column_width (GTK_CLIST (submitter_clist), 0, 277);
  gtk_clist_set_column_width (GTK_CLIST (submitter_clist), 1, 126);
  gtk_clist_set_column_width (GTK_CLIST (submitter_clist), 2, 80);
  gtk_clist_column_titles_show (GTK_CLIST (submitter_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (submitter_clist),
                                GTK_SELECTION_MULTIPLE);
  label = gtk_label_new (_("Name"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label13", label);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (submitter_clist), 0, label);
  gtk_widget_set_sensitive (submitter_clist, can_edit);

  label = gtk_label_new (_("Login ID"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label14", label);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (submitter_clist), 1, label);

  label = gtk_label_new (_("Active"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label15", label);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (submitter_clist), 2, label);

  submitter_hbox = gtk_hbox_new (TRUE, 40);
  gtk_widget_ref (submitter_hbox);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "submitter_hbox",
                            submitter_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (submitter_hbox);
  gtk_box_pack_start (GTK_BOX (submitter_vbox), submitter_hbox,
                      FALSE, TRUE, 0);
  gtk_widget_set_sensitive (submitter_hbox, can_edit);

  submitter_add_all = gtk_button_new_with_label (_("Add All"));
  gtk_widget_ref (submitter_add_all);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            SUB_ADD_ALL_BTN,
                            submitter_add_all,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (submitter_add_all);
  gtk_box_pack_start (GTK_BOX (submitter_hbox), submitter_add_all,
                      TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (submitter_add_all), "clicked",
                      GTK_SIGNAL_FUNC (on_add_all_clicked),
                      submitter_clist);

  submitter_rm_all = gtk_button_new_with_label (_("Remove All"));
  gtk_widget_ref (submitter_rm_all);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            SUB_RM_ALL_BTN,
                            submitter_rm_all,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (submitter_rm_all);
  gtk_box_pack_start (GTK_BOX (submitter_hbox), submitter_rm_all,
                      TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (submitter_rm_all), "clicked",
                      GTK_SIGNAL_FUNC (on_rm_all_clicked),
                      submitter_clist);

  prj_member_vbox = gtk_vbox_new (FALSE, 5);
  gtk_widget_ref (prj_member_vbox);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "prj_member_vbox",
                            prj_member_vbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prj_member_vbox);
  gtk_container_add (GTK_CONTAINER (notebook2), prj_member_vbox);
  gtk_container_set_border_width (GTK_CONTAINER (prj_member_vbox), 4);

  scrollwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       "scrollwindow7", 
                       scrollwindow7);
  gtk_widget_show (scrollwindow7);
  gtk_box_pack_start (GTK_BOX (prj_member_vbox), scrollwindow7, 
                      TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwindow7), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  /*
   * This object has several signal handlers that handle selections and
   * unselection.  However, they are not attached until the list is filled,
   * and the currenct project memebers are highlighted.  This is done in
   * on_project_edit_dlg_show().
   */
  prj_member_clist = gtk_clist_new (3);
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       PRJ_MEMBER_LIST, 
                       prj_member_clist);
  gtk_object_set_data (GTK_OBJECT (prj_member_clist), ADD_LIST, NULL);
  gtk_object_set_data (GTK_OBJECT (prj_member_clist), RM_LIST, NULL);
  gtk_widget_show (prj_member_clist);
  gtk_container_add (GTK_CONTAINER (scrollwindow7), prj_member_clist);
  gtk_clist_set_column_width (GTK_CLIST (prj_member_clist), 0, 277);
  gtk_clist_set_column_width (GTK_CLIST (prj_member_clist), 1, 126);
  gtk_clist_set_column_width (GTK_CLIST (prj_member_clist), 2, 80);
  gtk_clist_column_titles_show (GTK_CLIST (prj_member_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (prj_member_clist),
                                GTK_SELECTION_MULTIPLE);
  gtk_widget_set_sensitive (prj_member_clist, can_edit);

  label = gtk_label_new (_("Name"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label16", label);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (prj_member_clist), 0, label);

  label = gtk_label_new (_("Login ID"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label17", label);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (prj_member_clist), 1, label);

  label = gtk_label_new (_("Active"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "label18", label);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (prj_member_clist), 2, label);

  prj_member_hbox = gtk_hbox_new (TRUE, 40);
  gtk_widget_ref (prj_member_hbox);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "prj_member_hbox",
                            prj_member_hbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prj_member_hbox);
  gtk_box_pack_start (GTK_BOX (prj_member_vbox), prj_member_hbox,
                      FALSE, TRUE, 0);
  gtk_widget_set_sensitive (prj_member_hbox, can_edit);

  prj_member_add_all = gtk_button_new_with_label (_("Add All"));
  gtk_widget_ref (prj_member_add_all);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            PRJ_MEM_ADD_ALL_BTN,
                            prj_member_add_all,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prj_member_add_all);
  gtk_box_pack_start (GTK_BOX (prj_member_hbox), prj_member_add_all,
                      TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (prj_member_add_all), "clicked",
                      GTK_SIGNAL_FUNC (on_add_all_clicked),
                      prj_member_clist);

  prj_member_rm_all = gtk_button_new_with_label (_("Remove All"));
  gtk_widget_ref (prj_member_rm_all);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            PRJ_MEM_RM_ALL_BTN,
                            prj_member_rm_all,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prj_member_rm_all);
  gtk_box_pack_start (GTK_BOX (prj_member_hbox), prj_member_rm_all,
                      TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (prj_member_rm_all), "clicked",
                      GTK_SIGNAL_FUNC (on_rm_all_clicked),
                      prj_member_clist);

  info_lbl = gtk_label_new (_("Project Information"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "info_lbl", info_lbl);
  gtk_widget_show (info_lbl);
  set_notebook_tab (notebook2, 0, info_lbl);

  submitter_label = gtk_label_new (_("Submitters"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), 
                       "submitter_label", 
                       submitter_label);
  gtk_widget_show (submitter_label);
  set_notebook_tab (notebook2, 1, submitter_label);

  resp_lbl = gtk_label_new (_("Project Members"));
  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), "resp_lbl", resp_lbl);
  gtk_widget_show (resp_lbl);
  set_notebook_tab (notebook2, 2, resp_lbl);

  vbox31 = gtk_vbox_new (FALSE, 3);
  gtk_widget_ref (vbox31);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "vbox31", vbox31,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox31);
  gtk_container_add (GTK_CONTAINER (notebook2), vbox31);
  gtk_container_set_border_width (GTK_CONTAINER (vbox31), 5);

  label = gtk_label_new (_("Send a notification message on event:"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label34", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox31), label, FALSE, FALSE, 0);

  hbox35 = gtk_hbox_new (FALSE, 3);
  gtk_widget_ref (hbox35);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "hbox35", hbox35,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox35);
  gtk_box_pack_start (GTK_BOX (vbox31), hbox35, TRUE, TRUE, 0);

  frame = gtk_frame_new (_("New Problem Report"));
  gtk_widget_ref (frame);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "frame6", frame,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (hbox35), frame, TRUE, TRUE, 0);

  vbox32 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox32);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "vbox32", vbox32,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox32);
  gtk_container_add (GTK_CONTAINER (frame), vbox32);

  new_pr_notify_grp_cb = gtk_check_button_new_with_label (_("Notify Group"));
  gtk_widget_ref (new_pr_notify_grp_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            NEW_PR_NOTIFY_GRP, 
                            new_pr_notify_grp_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (new_pr_notify_grp_cb);
  gtk_box_pack_start (GTK_BOX (vbox32), new_pr_notify_grp_cb, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (new_pr_notify_grp_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (new_pr_notify_grp_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, new_pr_notify_grp_cb);

  new_pr_notify_pl_cb = gtk_check_button_new_with_label 
     (_("Notify Project Leader"));
  gtk_widget_ref (new_pr_notify_pl_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            NEW_PR_NOTIFY_PL, 
                            new_pr_notify_pl_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (new_pr_notify_pl_cb);
  gtk_box_pack_start (GTK_BOX (vbox32), new_pr_notify_pl_cb, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (new_pr_notify_pl_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (new_pr_notify_pl_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, new_pr_notify_pl_cb);

  frame = gtk_frame_new (_("Assign Problem Report"));
  gtk_widget_ref (frame);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "frame7", frame,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (hbox35), frame, TRUE, TRUE, 0);

  vbox33 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox33);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "vbox33", vbox33,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox33);
  gtk_container_add (GTK_CONTAINER (frame), vbox33);

  assign_pr_notify_resp_cb = 
     gtk_check_button_new_with_label (_("Notify Assignee"));
  gtk_widget_ref (assign_pr_notify_resp_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            ASSIGN_NOTIFY_RESP, 
                            assign_pr_notify_resp_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (assign_pr_notify_resp_cb);
  gtk_box_pack_start (GTK_BOX (vbox33), 
                      assign_pr_notify_resp_cb, 
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (assign_pr_notify_resp_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (assign_pr_notify_resp_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, assign_pr_notify_resp_cb);

  assign_pr_notify_sub_cb = 
     gtk_check_button_new_with_label (_("Notify Submitter"));
  gtk_widget_ref (assign_pr_notify_sub_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            ASSIGN_NOTIFY_SUB, 
                            assign_pr_notify_sub_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (assign_pr_notify_sub_cb);
  gtk_box_pack_start (GTK_BOX (vbox33), 
                      assign_pr_notify_sub_cb, 
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (assign_pr_notify_sub_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (assign_pr_notify_sub_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, assign_pr_notify_sub_cb);

  assign_pr_notify_pl_cb = 
     gtk_check_button_new_with_label (_("Notify Project Leader"));
  gtk_widget_ref (assign_pr_notify_pl_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            ASSIGN_NOTIFY_PL, 
                            assign_pr_notify_pl_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (assign_pr_notify_pl_cb);
  gtk_box_pack_start (GTK_BOX (vbox33), 
                      assign_pr_notify_pl_cb, 
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (assign_pr_notify_pl_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (assign_pr_notify_pl_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, assign_pr_notify_pl_cb);

  frame = gtk_frame_new (_("Update PR Status"));
  gtk_widget_ref (frame);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "frame8", frame,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (hbox35), frame, TRUE, TRUE, 0);

  vbox34 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox34);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "vbox34", vbox34,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox34);
  gtk_container_add (GTK_CONTAINER (frame), vbox34);

  upd_pr_notify_grp_cb = gtk_check_button_new_with_label (_("Notify Group"));
  gtk_widget_ref (upd_pr_notify_grp_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            UPD_NOTIFY_GRP, 
                            upd_pr_notify_grp_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (upd_pr_notify_grp_cb);
  gtk_box_pack_start (GTK_BOX (vbox34), upd_pr_notify_grp_cb, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_grp_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (upd_pr_notify_grp_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, upd_pr_notify_grp_cb);

  upd_pr_notify_resp_cb = gtk_check_button_new_with_label 
     (_("Notify Assignee"));
  gtk_widget_ref (upd_pr_notify_resp_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            UPD_NOTIFY_RESP, 
                            upd_pr_notify_resp_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (upd_pr_notify_resp_cb);
  gtk_box_pack_start (GTK_BOX (vbox34), upd_pr_notify_resp_cb,
                      FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_resp_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (upd_pr_notify_resp_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, upd_pr_notify_resp_cb);

  upd_pr_notify_sub_cb = gtk_check_button_new_with_label
     (_("Notify Submitter"));
  gtk_widget_ref (upd_pr_notify_sub_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            UPD_NOTIFY_SUB, 
                            upd_pr_notify_sub_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (upd_pr_notify_sub_cb);
  gtk_box_pack_start (GTK_BOX (vbox34), upd_pr_notify_sub_cb, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_sub_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (upd_pr_notify_sub_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, upd_pr_notify_sub_cb);

  upd_pr_notify_pl_cb = 
     gtk_check_button_new_with_label (_("Notify Project Leader"));
  gtk_widget_ref (upd_pr_notify_pl_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            UPD_NOTIFY_PL, 
                            upd_pr_notify_pl_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (upd_pr_notify_pl_cb);
  gtk_box_pack_start (GTK_BOX (vbox34), upd_pr_notify_pl_cb, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (upd_pr_notify_pl_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (upd_pr_notify_pl_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);
  add_widget_to_dirty_list (project_edit_dlg, upd_pr_notify_pl_cb);

  label = gtk_label_new (_("Notification Messages"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "label33", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), 
                              gtk_notebook_get_nth_page 
                                (GTK_NOTEBOOK (notebook2), 3), 
                              label);                                      

  frame = gtk_frame_new (_("Auto-Assign New Problem Reports To:"));
  gtk_widget_ref (frame);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "frame9", frame,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (vbox31), frame, FALSE, FALSE, 0);

  vbox35 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox35);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "vbox35", vbox35,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox35);
  gtk_container_add (GTK_CONTAINER (frame), vbox35);
  gtk_container_set_border_width (GTK_CONTAINER (vbox35), 5);

  auto_assign_combo = gtk_combo_new ();
  gtk_widget_ref (auto_assign_combo);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            AUTO_ASSIGN_COMBO, 
                            auto_assign_combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (auto_assign_combo);
  gtk_box_pack_start (GTK_BOX (vbox35), auto_assign_combo, FALSE, FALSE, 0);
  gtk_widget_set_sensitive (auto_assign_combo, can_edit);
  /*
   * The set_dirty callback is connected so we can determine if this
   * widget is dirty.  However, we do not add it to the dialogs dirty
   * list because this is not part of the "main" SQL statement due to
   * RI constraints.  We check this widget's dirty state manually when
   * we need to...
   */
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (auto_assign_combo)->entry),
                      "changed",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);

  table5 = gtk_table_new (8, 2, FALSE);
  gtk_widget_ref (table5);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "table5", table5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table5);
  gtk_table_set_col_spacings (GTK_TABLE (table5), 10);
  gtk_container_add (GTK_CONTAINER (notebook2), table5);
  gtk_container_set_border_width (GTK_CONTAINER (table5), 5);

  label = gtk_label_new (_("Version Information"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label38", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table5), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  show_inactive_cb = gtk_check_button_new_with_label 
     (_("Show Inactive Versions"));
  gtk_widget_ref (show_inactive_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            SHOW_ALL_VER_CB, 
                            show_inactive_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (show_inactive_cb);
  gtk_table_attach (GTK_TABLE (table5), show_inactive_cb, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_signal_connect (GTK_OBJECT (show_inactive_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_show_inactive_ver_cb_toggled),
                      project_edit_dlg);

  scrolledwindow14 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow14);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "scrolledwindow14", 
                            scrolledwindow14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow14);
  gtk_table_attach (GTK_TABLE (table5), scrolledwindow14, 0, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow14), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  version_clist = gtk_clist_new (3);
  gtk_widget_ref (version_clist);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            VER_CLIST, 
                            version_clist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (version_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow14), version_clist);
  gtk_clist_set_column_width (GTK_CLIST (version_clist), 0, 110);
  gtk_clist_set_column_width (GTK_CLIST (version_clist), 1, 350);
  gtk_clist_set_column_width (GTK_CLIST (version_clist), 2, 50);
  gtk_clist_column_titles_show (GTK_CLIST (version_clist));
  gtk_signal_connect(GTK_OBJECT(version_clist), "select_row",
                     GTK_SIGNAL_FUNC(on_ver_clist_select),
                     project_edit_dlg);
  gtk_signal_connect(GTK_OBJECT(version_clist), "unselect_row",
                     GTK_SIGNAL_FUNC(on_ver_clist_unsel),
                     project_edit_dlg);
  gtk_widget_set_sensitive (version_clist, can_edit);

  label = gtk_label_new (_("Version"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label43", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (version_clist), 0, label);

  label = gtk_label_new (_("Description"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label44", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (version_clist), 1, label);

  label = gtk_label_new (_("Active"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label45", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (version_clist), 2, label);

  hseparator = gtk_hseparator_new ();
  gtk_widget_ref (hseparator);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "hseparator1", hseparator,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator);
  gtk_table_attach (GTK_TABLE (table5), hseparator, 0, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 3);

  table6 = gtk_table_new (2, 3, FALSE);
  gtk_widget_ref (table6);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "table6",
                            table6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_table_set_col_spacings (GTK_TABLE (table6), 5);
  gtk_widget_show (table6);
  gtk_table_attach (GTK_TABLE (table5), table6, 0, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0); 

  label = gtk_label_new (_("Version"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "label48",
                            label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table6), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_widget_set_usize (label, 110, -2);

  label = gtk_label_new (_("Description"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "label49",
                            label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table6), label, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  /*
   * NOTE: The ver_entry, ver_descr_entry, and ver_active_cb widgets
   * are all setup with a callback to set their dirty flag.  However,
   * they are not added to the dlg's dirty list.  
   *
   * The end result is that we can check these widgets to see when
   * they are dirty, but their dirty state is not automatically taken
   * into account when checking to see if the widget is dirty.
   *
   * The reason for this is we do not care about the state of these widgets
   * when building the main SQL string.  However, we may still want to 
   * know there state in other instances (such as if we are closing the
   * dlg).
   *
   * Also, set_dirty is called directly, instead of being called via
   * on_widget_contents_modified.  This is because changing these values
   * does not make the dialog as a whole dirty.  It is only when the 
   * "Add" or "Modify" buttons are pressed that the main dialog itself
   * becomes dirty.
   */
  ver_entry = gtk_entry_new ();
  gtk_widget_ref (ver_entry);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            VER_ENTRY,
                            ver_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ver_entry);
  gtk_table_attach (GTK_TABLE (table6), ver_entry, 0, 1, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_signal_connect (GTK_OBJECT (ver_entry), "changed",
                      GTK_SIGNAL_FUNC (set_ver_add_mod_btn_state),
                      project_edit_dlg);
  gtk_widget_set_usize (ver_entry, 110, -2);
  gtk_widget_set_sensitive (ver_entry, can_edit);
  gtk_signal_connect (GTK_OBJECT (ver_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  popup = attach_generic_edit_popup_menu (ver_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "popup3", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  ver_descr_entry = gtk_entry_new ();
  gtk_widget_ref (ver_descr_entry);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            VER_DESCR_ENTRY,
                            ver_descr_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ver_descr_entry);
  gtk_table_attach (GTK_TABLE (table6), ver_descr_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_signal_connect (GTK_OBJECT (ver_descr_entry), "changed",
                      GTK_SIGNAL_FUNC (set_ver_add_mod_btn_state),
                      project_edit_dlg);
  gtk_widget_set_sensitive (ver_descr_entry, can_edit);
  gtk_signal_connect (GTK_OBJECT (ver_descr_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  popup = attach_generic_edit_popup_menu (ver_descr_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "popup4", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  ver_active_cb = gtk_check_button_new_with_label (_("Active"));
  gtk_widget_ref (ver_active_cb);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            VER_ACTIVE_CB, 
                            ver_active_cb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ver_active_cb);
  gtk_table_attach (GTK_TABLE (table6), ver_active_cb, 2, 3, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  /*
   * set toggle button BEFORE connecting the signal.  We don't want this
   * running the first time.
   */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ver_active_cb), TRUE);
  gtk_signal_connect (GTK_OBJECT (ver_active_cb), "toggled",
                      GTK_SIGNAL_FUNC (set_ver_add_mod_btn_state),
                      project_edit_dlg);
  gtk_widget_set_sensitive (ver_active_cb, can_edit);
  gtk_signal_connect (GTK_OBJECT (ver_active_cb), "toggled",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            "hbuttonbox",
                            hbuttonbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox);
  gtk_table_attach (GTK_TABLE (table5), hbuttonbox, 0, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 3);
  gtk_widget_set_sensitive (hbuttonbox, can_edit);

  ver_add_btn = gtk_button_new_with_label (_("Add as New Version"));
  gtk_widget_ref (ver_add_btn);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            ADD_VER_BUTTON,
                            ver_add_btn,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ver_add_btn);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ver_add_btn);
  gtk_signal_connect (GTK_OBJECT (ver_add_btn), "clicked",
                      GTK_SIGNAL_FUNC (on_add_version_clicked),
                      project_edit_dlg);
  gtk_widget_set_sensitive (ver_add_btn, FALSE);

  ver_mod_btn = gtk_button_new_with_label (_("Modify Selected Version"));
  gtk_widget_ref (ver_mod_btn);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            MOD_VER_BUTTON,
                            ver_mod_btn,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ver_mod_btn);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ver_mod_btn);
  gtk_signal_connect (GTK_OBJECT (ver_mod_btn), "clicked",
                      GTK_SIGNAL_FUNC (on_mod_version_clicked),
                      project_edit_dlg);
  gtk_widget_set_sensitive (ver_mod_btn, FALSE);

  ver_del_btn = gtk_button_new_with_label (_("Delete Selected Version"));
  gtk_widget_ref (ver_del_btn);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            DEL_VER_BUTTON,
                            ver_del_btn,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ver_del_btn);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ver_del_btn);
  gtk_signal_connect (GTK_OBJECT (ver_del_btn), "clicked",
                      GTK_SIGNAL_FUNC (on_del_version_clicked),
                      project_edit_dlg);
  gtk_widget_set_sensitive (ver_del_btn, FALSE);

  hseparator = gtk_hseparator_new ();
  gtk_widget_ref (hseparator);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "hseparator2", hseparator,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hseparator);
  gtk_table_attach (GTK_TABLE (table5), hseparator, 0, 2, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 3);

  label = gtk_label_new (_("Default Opening Version"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label36", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table5), label, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("Default Closing Version"));
  gtk_widget_ref (label);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), "label37", label,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table5), label, 1, 2, 6, 7,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  def_open_combo = gtk_combo_new ();
  gtk_widget_ref (def_open_combo);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            DEF_OPEN_COMBO, 
                            def_open_combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (def_open_combo);
  gtk_table_attach (GTK_TABLE (table5), def_open_combo, 0, 1, 7, 8,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (def_open_combo, can_edit);

  def_open_entry = GTK_COMBO (def_open_combo)->entry;
  gtk_widget_ref (def_open_entry);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            DEF_OPEN_ENTRY, 
                            def_open_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (def_open_entry);
  gtk_entry_set_editable (GTK_ENTRY (def_open_entry), FALSE);
  /*
   * Default open version is not part of the "main" SQL due to RI.
   * We do want to determine if it is dirty, however.  Thus we do
   * not add it to the list, but we set up the handler.
   */
  gtk_signal_connect (GTK_OBJECT (def_open_entry), "changed",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);

  def_close_combo = gtk_combo_new ();
  gtk_widget_ref (def_close_combo);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            DEF_CLOSE_COMBO, 
                            def_close_combo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (def_close_combo);
  gtk_table_attach (GTK_TABLE (table5), def_close_combo, 1, 2, 7, 8,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (def_close_combo, can_edit);

  def_close_entry = GTK_COMBO (def_close_combo)->entry;
  gtk_widget_ref (def_close_entry);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg),
                            DEF_CLOSE_ENTRY, 
                            def_close_entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (def_close_entry);
  gtk_entry_set_editable (GTK_ENTRY (def_close_entry), FALSE);
  /*
   * Default close version is not part of the "main" SQL due to RI.
   * We do want to determine if it is dirty, however.  Thus we do
   * not add it to the list, but we set up the handler.
   */
  gtk_signal_connect (GTK_OBJECT (def_close_entry), "changed",
                      GTK_SIGNAL_FUNC (on_widget_contents_modified),
                      project_edit_dlg);

  version_lbl = gtk_label_new (_("Versions"));
  gtk_widget_ref (version_lbl);
  gtk_object_set_data_full (GTK_OBJECT (project_edit_dlg), 
                            "version_lbl", version_lbl,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (version_lbl);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2), 
                              gtk_notebook_get_nth_page 
                                (GTK_NOTEBOOK (notebook2), 4), 
                              version_lbl);

  gtk_object_set_data (GTK_OBJECT (project_edit_dlg), TOOLTIPS, tooltips);

  return project_edit_dlg;
}


static void
raise_or_create_project_edit_dlg (GtkWidget *parent,
                                  gboolean   is_update,
                                  gint       pk_num,
                                  PGconn    *conn,
                                  gboolean   user_is_admin)
{
   GtkWidget  *dlg;
   GList      *child_list;
   GList      *list_iter;

   /*
    * Go through the existing list of children first.  If we have a
    * child that already uses this project's PK, just bring that one
    * to the top.
    */
   child_list = gtk_object_get_data (GTK_OBJECT (parent), my_child_list);

   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      if (pk_num == GPOINTER_TO_INT
                      (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                            PK_NUM)))
         {
         gdk_window_show (GTK_WIDGET (list_iter->data)->window);
         gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

         return;
         }
      list_iter = list_iter->next;
      }

   /*
    * Didn't find a child already displaying this project, so we need
    * to create one.
    */
   dlg = create_project_edit_dlg (is_update, pk_num, conn,
                                  parent, user_is_admin);
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the 
    * child_list when the child dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);

   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (parent), my_child_list, child_list);
}


/*---------------------------------------------------------------------------
 * Local support functions for the project list dialog box.
 *---------------------------------------------------------------------------
 */
static PGresult *
read_project_data (PGconn *conn, gboolean show_inactive)
{
   GString *buffer;
   PGresult *result;
   
   /*
    *  The order in the following SELECT is tied to #define's at
    *  the start of the file.  Change this SELECT, and be sure to
    *  change those to match.
    */
   buffer = g_string_new ("SELECT p.project_num, p.name, p.active, ");
   buffer = g_string_append (buffer, "p.login_id, p.descr, ");
   buffer = g_string_append (buffer, "u.last_name, u.first_name ");
   buffer = g_string_append (buffer, "FROM project p, person u ");
   buffer = g_string_append (buffer, "WHERE u.login_id = p.login_id "); 
   if (!show_inactive)
      {
      buffer = g_string_append (buffer, "AND p.active = TRUE ");
      }
   else
      {
      /*
       * This should not be needed, but for some reason, it is.
       */
      buffer = g_string_append 
         (buffer, "AND (p.active = TRUE OR p.active = FALSE) ");
      }
   buffer = g_string_append (buffer, "ORDER BY p.name ");
   
   result = PQexec (conn, buffer->str);
   
   g_string_free (buffer, TRUE);
   return result;
}


static PGresult *
delete_project (PGconn *conn, gint pk_num)
{
   GString  *buffer;
   PGresult *res;
   
   buffer = g_string_new ("");
   g_string_sprintf (buffer,
                     "DELETE FROM submitter WHERE project_num = %d",
                     pk_num);
   res = PQexec (conn, buffer->str);
   if (sql_result_ok (res))
      {
      PQclear (res);
      g_string_sprintf (buffer,
                        "DELETE FROM responsible WHERE project_num = %d",
                        pk_num);
      res = PQexec (conn, buffer->str);
      if (sql_result_ok (res))
         {
         PQclear (res);
         g_string_sprintf (buffer,
                           "DELETE FROM project WHERE project_num = %d",
                           pk_num);
         res = PQexec (conn, buffer->str);
         }
      }
   g_string_free (buffer, TRUE);
   
   return res;
}


static gboolean 
draw_project_list (GtkWindow *parent, PGconn *conn)
{
   char            *buffer[4] = {NULL, NULL, NULL, NULL};
   gint             ntuples;
   gint             i;
   gint             pk_num;
   PGresult        *res;
   GtkCList        *lb_widget;
   GtkCheckButton  *cb_widget;
   gboolean         show_inactive;
   gboolean         rtn_val = FALSE;
   
   lb_widget = GTK_CLIST (get_widget (GTK_WIDGET (parent), PROJECT_LIST));
   g_assert (lb_widget != NULL);
   cb_widget = GTK_CHECK_BUTTON (get_widget (GTK_WIDGET (parent),
                                             SHOW_INACTIVE_CB)); 
   g_assert (cb_widget != NULL);
   show_inactive =
      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_widget));
   
   gtk_clist_clear (lb_widget);
   res = read_project_data (conn, show_inactive);
   
   if (chk_sql_error (res, _("reading project data")))
      {
      ntuples = PQntuples (res);
      
      for (i = 0; i < ntuples; i++)
         {
         buffer[0] = (char *)g_malloc
            (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (buffer[0], PQgetvalue (res, i, NAME_POS));
         
         buffer[1] = (char *)g_malloc 
            (strlen (PQgetvalue (res, i, LAST_NAME_POS)) +
             strlen (PQgetvalue (res, i, FIRST_NAME_POS)) +
             3);
         sprintf (buffer[1], "%s, %s",
                  PQgetvalue (res, i, LAST_NAME_POS),
                  PQgetvalue (res, i, FIRST_NAME_POS));
         
         buffer[2] = (char *)g_malloc (strlen (get_active_yn (res, i)) + 1);
         strcpy (buffer[2], get_active_yn (res, i));
         
         buffer[3] = (char *)g_malloc
            (strlen (PQgetvalue (res, i, DESCR_POS)) + 1);
         strcpy (buffer[3], PQgetvalue (res, i, DESCR_POS));
         pk_num = atoi (PQgetvalue (res, i, PK_NUM_POS));
         
         /*
          *  The PK is stored with each object displayed in the list so 
          *  the selection can be mapped back to the database.
          */
         gtk_clist_append (lb_widget, buffer);
         gtk_clist_set_row_data (lb_widget, i, GINT_TO_POINTER (pk_num));
         g_free (buffer[0]);
         g_free (buffer[1]);
         g_free (buffer[2]);
         g_free (buffer[3]);
         }
      rtn_val = TRUE;
      }
   
   PQclear (res);
   return rtn_val;
}



   
/*---------------------------------------------------------------------------
 * Signal handlers for the project list dialog box.
 *---------------------------------------------------------------------------
 */
static void 
on_project_dlg_show (GtkWidget  *widget, gpointer  user_data)
{
   PGconn          *conn;
   GtkToggleButton *cb;
   GString         *config_str;
   gboolean         is_def;
   gboolean         flag;

   config_str = g_string_new ("");

   cb = GTK_TOGGLE_BUTTON (get_widget (widget, SHOW_INACTIVE_CB));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, CONF_WITH_DEF, PACKAGE, DLG_INI_FILE,
                     PRJ_DLG_SEC, SHOW_INACT, "false");
   flag = gnome_config_get_bool_with_default (config_str->str, &is_def);
   gtk_toggle_button_set_active (cb, flag);

   g_string_free (config_str, TRUE);

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   if (!draw_project_list (GTK_WINDOW (widget), conn))
      {
      gtk_widget_destroy (widget);
      }
}


static void 
on_project_dlg_destroy (GtkObject  *object, gpointer  user_data)
{
   GList           *child_list;
   GList           *list_iter;
   GtkToggleButton *cb;
   GString         *config_str;

   config_str = g_string_new ("");

   cb = GTK_TOGGLE_BUTTON (get_widget (GTK_WIDGET (object), SHOW_INACTIVE_CB));
   g_assert (cb != NULL);
   g_string_sprintf (config_str, BASIC_CONF, PACKAGE, DLG_INI_FILE,
                     PRJ_DLG_SEC, SHOW_INACT);
   gnome_config_set_bool (config_str->str, gtk_toggle_button_get_active (cb));

   g_string_sprintf (config_str, CONF_FILE, PACKAGE, DLG_INI_FILE);
   gnome_config_sync_file (config_str->str);

   g_string_free (config_str, TRUE);

   /*
    * Kill all of the children
    */
   child_list = gtk_object_get_data (object, my_child_list);
   list_iter = g_list_first (child_list);
   while (list_iter)
      {
      gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                        child_dead,
                                        (gpointer)my_child_list);
      gtk_widget_destroy (GTK_WIDGET (list_iter->data));
      list_iter = list_iter->next;
      }
   g_list_free (child_list);
}


static void
on_project_dlg_clicked (GnomeDialog *dlg, 
                        gint         btn,
                        gpointer     user_data)
{
   save_dlg_size (GTK_WIDGET (dlg), PRJ_DLG_SEC, FALSE);
}


static void 
on_add_project (GtkWidget *widget, gpointer user_data)
{
   GtkWidget   *dlg;
   PGconn      *conn;
   gboolean     admin_user;
   GList       *child_list;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (user_data),
                                                       ADMIN_USER)) == 1);
   
   dlg = create_project_edit_dlg (FALSE, 0, conn,
                                  GTK_WIDGET (user_data), admin_user);
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when the child dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);

   child_list = gtk_object_get_data (GTK_OBJECT (user_data), my_child_list);
   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (user_data), my_child_list, child_list);
}


static void 
on_modify_project (GtkWidget *widget, gpointer user_data)
{
   GtkCList  *lb_widget;
   gint       row_idx;
   gint       pk_num;
   PGconn    *conn;
   GtkWidget *main_widget = GTK_WIDGET (user_data);
   gboolean   admin_user;
   GtkWidget *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (main_widget, PROJECT_LIST));
   g_assert (lb_widget != NULL);
   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (user_data),
                                                       ADMIN_USER)) == 1);
   
   if (lb_widget->selection)
      {
      row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
      pk_num = GPOINTER_TO_INT (gtk_clist_get_row_data (lb_widget, row_idx));
      raise_or_create_project_edit_dlg (main_widget, TRUE, pk_num,
                                        conn, admin_user);
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To modify a project, you must select a project to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void
on_project_sel_row (GtkWidget       *widget,
                    gint             row,
                    gint             column,
                    GdkEventButton  *event,
                    gpointer         user_data)
{
   GtkCList  *clist = GTK_CLIST (widget);
   gint       pk_num;
   PGconn    *conn;
   GtkWidget *main_widget = GTK_WIDGET (user_data);
   gboolean   admin_user;

   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (user_data),
                                                       ADMIN_USER)) == 1);
   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            pk_num = GPOINTER_TO_INT (gtk_clist_get_row_data (clist, row));
            raise_or_create_project_edit_dlg (main_widget, TRUE, pk_num,
                                              conn, admin_user);
            break;

         default:
            break;
         }
      }
}


static void 
on_delete_project (GtkWidget *widget, gpointer user_data)
{
   GtkCList   *lb_widget;
   PGresult   *res;
   gint        row_idx;
   gint        pk_num;
   PGconn     *conn;
   GtkWidget  *main_widget = GTK_WIDGET (user_data);
   GtkWidget  *main_win;
   GtkWidget  *msg_dlg;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);
   lb_widget = GTK_CLIST (get_widget (main_widget, PROJECT_LIST));
   g_assert (lb_widget != NULL);
   main_win = get_widget (main_widget, MY_PARENT);
   g_assert (main_win != NULL);
   
   if (lb_widget->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("Deleting the project will remove all associated PRs.\n"
            "This action is not reversable.\n"
            "Are you sure you want to continue?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0)
         {
         row_idx = GPOINTER_TO_INT (lb_widget->selection->data);
         pk_num = GPOINTER_TO_INT 
            (gtk_clist_get_row_data (lb_widget, row_idx));
         START_TRANSACTION (conn);
         res = delete_project (conn, pk_num);
         if (chk_sql_error (res, _("Project Delete Failed")))
            {
            COMMIT (conn);
            }
         else
            {
            ROLLBACK (conn);
            }
         /*
          * the project list dialog and the project list on the main
          * window both need re-drawing....
          */
         draw_project_list (GTK_WINDOW (main_widget), conn);
         refresh_project_list (main_win);
         PQclear (res);
         }
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("To delete a project, you must select a project to delete."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
}


static void 
on_show_inactive_cb_toggled (GtkToggleButton  *togglebutton,
                             gpointer          user_data)
{
   PGconn     *conn;
   
   conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
   g_assert (conn != NULL);

   draw_project_list (GTK_WINDOW (user_data), conn);
}


static gboolean
on_button_press_event (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        user_data)
{
   gint        row;
   gint        col;
   gint        val;
   GtkWidget  *popup;
   GtkWidget  *main_win = GTK_WIDGET (user_data);
   gboolean    admin_user;

   static GnomeUIInfo popup_menu[] =
      {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Project"),
                                 N_("Create a new project."),
                                 on_add_project, NULL),
      GNOMEUIINFO_ITEM_NONE 
         (N_("View/_Modify Project"),
          N_("View or Modify the selected project's information."),
                             on_modify_project),
      GNOMEUIINFO_ITEM_NONE (N_("Delete Project"),
                             N_("Delete the selected project."),
                             on_delete_project),
      GNOMEUIINFO_END
      };

   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_win),
                                                       ADMIN_USER)) == 1);

   if (event)
      {
      switch (event->button)
         {
         case 1:
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  val = gtk_clist_get_selection_info (GTK_CLIST (widget),
                                                      event->x,
                                                      event->y,
                                                      &row,
                                                      &col);
                  popup = gnome_popup_menu_new (popup_menu);
                  if (val)
                     {
                     gtk_clist_select_row (GTK_CLIST (widget), row, col);
                     }
                  else
                     {
                     gtk_widget_set_sensitive (popup_menu[1].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     if (GTK_CLIST (widget)->selection != NULL)
                        {
                        gtk_clist_unselect_row
                           (GTK_CLIST (widget),
                            GPOINTER_TO_INT
                            (GTK_CLIST (widget)->selection->data), 0);
                        }
                     }
                  if (!admin_user)
                     {
                     gtk_widget_set_sensitive (popup_menu[0].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     }
                  gnome_popup_menu_do_popup_modal (popup,
                                                   NULL,
                                                   NULL,
                                                   event,
                                                   main_win);
                  gtk_widget_destroy (popup);
                  break;
               default:
                  break;
               }
         default:
            break;
         }
      }

   return FALSE;
}


static GtkWidget* 
create_project_dlg (GtkWidget *parent, PGconn  *conn, gboolean user_is_admin)
{
  GtkWidget       *project_dlg;
  GtkWidget       *dialog_vbox15;
  GtkWidget       *vbox29;
  GtkWidget       *clist7;
  GtkWidget       *name_lbl;
  GtkWidget       *leader_lbl;
  GtkWidget       *active_lbl;
  GtkWidget       *descr_lbl;
  GtkWidget       *hbox33;
  GtkWidget       *add_button;
  GtkWidget       *modify_button;
  GtkWidget       *delete_button;
  GtkWidget       *show_inactive_cb;
  GtkWidget       *scrolledwindow;
  GtkTooltips     *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  project_dlg = gnome_dialog_new (_("Project List"),
                                  GNOME_STOCK_BUTTON_CLOSE,
                                  NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (project_dlg), GTK_WINDOW (parent));
  gtk_object_set_data (GTK_OBJECT (project_dlg), 
                       PROJECT_DIALOG, 
                       project_dlg);
  gtk_container_border_width (GTK_CONTAINER (project_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (project_dlg), "show",
                      GTK_SIGNAL_FUNC (on_project_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (project_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_project_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (project_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_project_dlg_clicked),
                      NULL);
  gtk_window_set_policy (GTK_WINDOW (project_dlg), FALSE, TRUE, FALSE);
  gnome_dialog_set_close (GNOME_DIALOG (project_dlg), TRUE);
  gnome_dialog_set_default (GNOME_DIALOG (project_dlg), 0);
  set_dlg_size (project_dlg, PRJ_DLG_SEC, 550, 265);

  gtk_object_set_data (GTK_OBJECT (project_dlg), TOP_DOG, project_dlg);
  gtk_object_set_data (GTK_OBJECT (project_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (project_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (project_dlg), ADMIN_USER,
        GINT_TO_POINTER (user_is_admin ? 1 : 0));

  dialog_vbox15 = GNOME_DIALOG (project_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox15), 5);
  gtk_object_set_data (GTK_OBJECT (project_dlg), 
                       "dialog_vbox15", 
                       dialog_vbox15);
  gtk_widget_show (dialog_vbox15);

  vbox29 = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (project_dlg), "vbox29", vbox29);
  gtk_widget_show (vbox29);
  gtk_box_pack_start (GTK_BOX (dialog_vbox15), vbox29, TRUE, TRUE, 0);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (project_dlg),
                       "scrolledwindow",
                       scrolledwindow);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_start (GTK_BOX (vbox29), scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);

  clist7 = gtk_clist_new (4);
  gtk_object_set_data (GTK_OBJECT (project_dlg), PROJECT_LIST, clist7);
  gtk_widget_show (clist7);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), clist7);
  gtk_clist_set_column_width (GTK_CLIST (clist7), 0, 120);
  gtk_clist_set_column_width (GTK_CLIST (clist7), 1, 120);
  gtk_clist_set_column_width (GTK_CLIST (clist7), 2, 50);
  gtk_clist_column_titles_show (GTK_CLIST (clist7));
  gtk_signal_connect (GTK_OBJECT (clist7), "select_row",
                      GTK_SIGNAL_FUNC (on_project_sel_row),
                      project_dlg);
  gtk_signal_connect (GTK_OBJECT (clist7), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press_event),
                      project_dlg);

  name_lbl = gtk_label_new (_("Project Name"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), "name_lbl", name_lbl);
  gtk_widget_show (name_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (clist7), 0, name_lbl);

  leader_lbl = gtk_label_new (_("Project Leader"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), "leader_lbl", leader_lbl);
  gtk_widget_show (leader_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (clist7), 1, leader_lbl);

  active_lbl = gtk_label_new (_("Active"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), "active_lbl", active_lbl);
  gtk_widget_show (active_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (clist7), 2, active_lbl);

  descr_lbl = gtk_label_new (_("Description"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), "descr_lbl", descr_lbl);
  gtk_widget_show (descr_lbl);
  gtk_clist_set_column_widget (GTK_CLIST (clist7), 3, descr_lbl);

  hbox33 = gtk_hbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (project_dlg), "hbox33", hbox33);
  gtk_widget_show (hbox33);
  gtk_box_pack_start (GTK_BOX (vbox29), hbox33, FALSE, FALSE, 0);

  add_button = gtk_button_new_with_label (_("Add"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), ADD_BUTTON, add_button);
  gtk_widget_show (add_button);
  gtk_box_pack_start (GTK_BOX (hbox33), add_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, add_button, _("Add a project."), NULL);
  gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
                      GTK_SIGNAL_FUNC (on_add_project),
                      project_dlg);
  gtk_widget_set_sensitive (add_button, user_is_admin);

  modify_button = gtk_button_new_with_label (_("View/Modify"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), MOD_BUTTON, modify_button);
  gtk_widget_show (modify_button);
  gtk_box_pack_start (GTK_BOX (hbox33), modify_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        modify_button, 
                        _("Modify an existing project."), 
                        NULL);
  gtk_signal_connect (GTK_OBJECT (modify_button), "clicked",
                      GTK_SIGNAL_FUNC (on_modify_project),
                      project_dlg);

  delete_button = gtk_button_new_with_label ("Delete");
  gtk_object_set_data (GTK_OBJECT (project_dlg), DEL_BUTTON, delete_button);
  gtk_widget_show (delete_button);
  gtk_box_pack_start (GTK_BOX (hbox33), delete_button, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, delete_button, _("Delete an existing project.  If the project is referenced in problem report, the project cannot be deleted, but can be made inactive."), NULL);
  gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
                      GTK_SIGNAL_FUNC (on_delete_project),
                      project_dlg);
  gtk_widget_set_sensitive (delete_button, user_is_admin);

  show_inactive_cb = gtk_check_button_new_with_label
     (_("Show Inactive Projects"));
  gtk_object_set_data (GTK_OBJECT (project_dlg), 
                       SHOW_INACTIVE_CB, 
                       show_inactive_cb);
  gtk_widget_show (show_inactive_cb);
  gtk_box_pack_start (GTK_BOX (hbox33), show_inactive_cb, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (show_inactive_cb), "toggled",
                      GTK_SIGNAL_FUNC (on_show_inactive_cb_toggled),
                      project_dlg);

  gtk_object_set_data (GTK_OBJECT (project_dlg), TOOLTIPS, tooltips);

  return project_dlg;
}


void
destroy_project_dlg (void)
{
   if (project_dlg != NULL)
      {
      gtk_widget_destroy (project_dlg);
      project_dlg = NULL;
      }
}


void
show_project_dlg (GtkWidget *parent, PGconn *conn, gboolean user_is_admin)
{
   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (project_dlg)
      {
      gdk_window_show (project_dlg->window);
      gdk_window_raise (project_dlg->window);
      }
   else
      {
      project_dlg = create_project_dlg (parent, conn, user_is_admin);

      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(),
       * which will set the "user_data" (&project_dlg in this case) to
       * NULL.
       */
      gtk_signal_connect (GTK_OBJECT (project_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (gtk_widget_destroyed),
                          &project_dlg);
      gtk_widget_show (project_dlg);
      }
}


gboolean
project_dlg_dirty (void)
{
   if (project_dlg == NULL)
      {
      return FALSE;
      }

   return list_dlg_has_dirty_child (project_dlg, my_child_list);
}
