/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * query_dlg
 *
 * Synopsis:
 *   Create and edit problem report queries.
 *
 * $Id: query_dlg.c,v 1.35 2002/01/05 17:43:51 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#include <ctype.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
 
#include <gnome.h>
#include <libpq-fe.h>

#include <preps/gstr_utils.h>
#include <preps/prq_utils.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "util.h"

#define QUERY_CLIST             "query_clist"

#define OK_BUTTON               "ok_button"
#define CAN_BUTTON              "can_button"
#define DESCR_TEXT              "descr_text"
#define NAME_ENTRY              "name_entry"

#define SEVR_CLIST              "sevr_clist"
#define STAT_CLIST              "stat_clist"
#define PT_CLIST                "pt_clist"
#define SUB_CLIST               "sub_clist"
#define RESP_CLIST              "resp_clist" 
#define ORDER_BY_ENTRY          "order_by_entry"

#define IS_SYS_QUERY_LIST       "is_sys_query_list"

#define RAW_SQL_TEXT            "raw_sql_text"

#define NAME_COL                0
#define RAW_YN_COL              1
#define DESCR_COL               2
#define NUM_COLS                3
#define YN_BUFF_SIZE            4
#define YES                     "yes"
#define NO                      "no"

#define STD_COL_WIDTH           120

#define PK_POS                  0
#define QUERY_NUM_POS           0
#define NAME_POS                1
#define DESCR_POS               2
#define IS_RAW_POS              3
#define ORDER_NUM_POS           4
#define RAW_SQL_POS             4
#define ORDER_BY_POS            5
#define GET_QUERY " \
SELECT query_num, name, descr, is_raw_sql, raw_sql, order_by, order_num \
  FROM pr_query \
 WHERE query_num = %d "  

#define SYSTEM_QUERIES " \
SELECT query_num, name, descr, is_raw_sql, order_num \
  FROM pr_query \
 WHERE is_system_query = TRUE \
 ORDER BY order_num"  

#define USER_QUERIES " \
SELECT query_num, name, descr, is_raw_sql, order_num \
  FROM pr_query \
 WHERE is_system_query = FALSE \
   AND creator = getpgusername () \
 ORDER BY order_num "  


#define SELECT_STATUS "\
SELECT status_num, name, order_num \
  FROM status \
 WHERE 1 = 1 "
 
#define SELECT_PROBLEM_TYPE "\
SELECT type_num, name \
  FROM problem_type \
 WHERE 1 = 1 "
 
#define SELECT_SEVERITY "\
SELECT severity_num, name, order_num \
  FROM severity \
 WHERE 1 = 1 "

#define SELECT_PEOPLE "\
SELECT p.login_id, \
       p.last_name || ', ' || p.first_name || ' (' || p.login_id || ')' \
  FROM person p \
 WHERE 1 = 1 "

#define DELETE_QUERY "\
DELETE \
  FROM pr_query \
 WHERE query_num = %d " 

#define UPDATE_ORDER_NUM "\
UPDATE pr_query \
   SET order_num = %d \
 WHERE query_num = %d "

#define ORDER_BY_NUM      " ORDER BY order_num "
#define SUB_RESP_ORDER_BY " ORDER BY 2, 1 "

static const gchar *my_child_list = "CHILD_LIST";
   
/*
 * At any given time, up to two query list dialog may be present (one
 * for system level queries, one for user level queries).
 */
static GtkWidget  *user_dlg = NULL;
static GtkWidget  *sys_dlg = NULL;

/*
 * pk_order_num stores the primary key (pk) and order_num for
 * a given row in the query list.
 */
typedef struct pk_order_num {
   gint      pk;
   gint      order_num;
} pk_order_num; 

typedef struct xtra_data_struct {
   PGconn   *conn;
   gint      pk;
   gboolean  is_sys;
} xtra_data_struct;

static gint return_value;

static void draw_query_list (GtkWidget *parent,
                             PGconn    *conn,
                             gboolean   is_system);

/*
 * Both types of queries have some stuff that must not be NULL.  Make sure
 * it is not NULL.  For raw SQL queries there is an additional sanity
 * check.
 */
static gboolean
validate_query_data (GnomeDialog *dlg, gboolean is_raw)
{
   GtkEntry   *name_entry;
   GtkText    *descr_text;
   GtkText    *raw_sql_text;
   GtkWidget  *msg_dlg;
   gchar      *buff;
   gint        len;

   name_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), NAME_ENTRY));
   g_assert (name_entry != NULL);
   descr_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (descr_text != NULL);

   if (strlen (gtk_entry_get_text (name_entry)) == 0)
      {
      msg_dlg = gnome_message_box_new 
         (_("You must supply a query name."),
          GNOME_MESSAGE_BOX_ERROR,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      return FALSE;
      }

   buff = gtk_editable_get_chars (GTK_EDITABLE (descr_text), 0, -1);
   len = strlen (buff);
   g_free (buff);
   if (len == 0)
      {
      msg_dlg = gnome_message_box_new 
         (_("You have not filled in a description for the query.\n"
            "Are you sure you want a blank description?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      return (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0);
      }

   if (is_raw)
      {
      raw_sql_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), RAW_SQL_TEXT));
      g_assert (raw_sql_text != NULL);
      buff = gtk_editable_get_chars (GTK_EDITABLE (raw_sql_text), 0, -1);
      len = strlen (buff);
      g_free (buff);
      if (len == 0)
         {
         msg_dlg = gnome_message_box_new
            (_("You have not added anything to the WHERE clause.\n"
               "Are you sure this is what you want?"),
             GNOME_MESSAGE_BOX_QUESTION,
             GNOME_STOCK_BUTTON_YES,
             GNOME_STOCK_BUTTON_NO,
             NULL);
         gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
         return (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0);
         }
      }

   return TRUE;
}


/*
 * Build a list of the keys for the selected items in the clist.
 */
static GList *
build_key_list (GtkCList *clist)
{
   GList  *list = NULL;
   GList  *iter;
   gint    row;

   iter = g_list_first (clist->selection);

   while (iter)
      {
      row = GPOINTER_TO_INT (iter->data);
      list = g_list_append (list,
                            gtk_clist_get_row_data (clist, row));
      iter = iter->next;
      }

   return g_list_first (list);
}


static GString *
build_insert_string(GnomeDialog *dlg, gboolean is_raw)
{
   GtkEntry   *name_entry;
   GtkText    *descr_text;
   GtkText    *raw_sql_text;
   GtkCList   *sevr_clist;
   GtkCList   *stat_clist;
   GtkCList   *pt_clist;
   GtkCList   *sub_clist;
   GtkCList   *resp_clist;
   GtkEntry   *order_by_entry;
   GString    *str;
   GString    *tmp;
   GString    *list_str;
   GList      *tmp_list;
   gchar      *buff;
   gboolean    is_sys;

   name_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), NAME_ENTRY));
   g_assert (name_entry != NULL);
   descr_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (descr_text != NULL);

   tmp = g_string_new ("");
   str = g_string_new ("INSERT INTO pr_query (query_num, name, descr, ");
   str = g_string_append (str, "is_system_query, is_raw_sql, raw_sql, ");
   str = g_string_append (str, "severity_nums, status_nums, ");
   str = g_string_append (str, "problem_type_nums, submitter_ids, ");
   str = g_string_append (str, "responsible_ids, order_by) ");
   str = g_string_append (str, "VALUES (nextval ('pr_query_seq'), ");

   tmp = g_string_assign (tmp, gtk_entry_get_text (name_entry));
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   buff = gtk_editable_get_chars (GTK_EDITABLE (descr_text), 0, -1);
   tmp = g_string_assign (tmp, buff);
   g_free (buff);
   g_string_prepare_db_instr (tmp);
   str = g_string_append (str, tmp->str);
   str = g_string_append (str, ", ");

   is_sys = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                   IS_SYS_QUERY_LIST)) == 1);
   if (is_sys)
      {
      str = g_string_append (str, "TRUE, ");
      }
   else
      {
      str = g_string_append (str, "FALSE, ");
      }

   if (is_raw)
      {
      str = g_string_append (str, "TRUE, ");

      raw_sql_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), RAW_SQL_TEXT));
      g_assert (raw_sql_text != NULL);
      buff = gtk_editable_get_chars (GTK_EDITABLE (raw_sql_text), 0, -1);
      tmp = g_string_assign (tmp, buff);
      g_free (buff);
      g_string_prepare_db_instr (tmp);
      str = g_string_append (str, tmp->str);
      str = g_string_append (str, ", ");

      str = g_string_append (str, "NULL, NULL, NULL, NULL, NULL, NULL)");
      }
   else
      {
      str = g_string_append (str, "FALSE, NULL, ");

      sevr_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), SEVR_CLIST));
      g_assert (sevr_clist != NULL);
      stat_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), STAT_CLIST));
      g_assert (stat_clist != NULL);
      pt_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), PT_CLIST));
      g_assert (pt_clist != NULL);
      sub_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), SUB_CLIST));
      g_assert (sub_clist != NULL);
      resp_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), RESP_CLIST));
      g_assert (resp_clist != NULL);
      order_by_entry =
         GTK_ENTRY (get_widget (GTK_WIDGET (dlg), ORDER_BY_ENTRY));
      g_assert (order_by_entry != NULL);

      if (sevr_clist->selection)
         {
         tmp_list = build_key_list (sevr_clist);
         list_str = create_int_arr_str (tmp_list);
         g_string_prepare_db_instr (list_str);
         str = g_string_append (str, list_str->str);
         str = g_string_append (str, ", ");
         g_list_free (tmp_list);
         g_string_free (list_str, TRUE);
         }
      else
         {
         str = g_string_append (str, "NULL, ");
         }

      if (stat_clist->selection)
         {
         tmp_list = build_key_list (stat_clist);
         list_str = create_int_arr_str (tmp_list);
         g_string_prepare_db_instr (list_str);
         str = g_string_append (str, list_str->str);
         str = g_string_append (str, ", ");
         g_list_free (tmp_list);
         g_string_free (list_str, TRUE);
         }
      else
         {
         str = g_string_append (str, "NULL, ");
         }

      if (pt_clist->selection)
         {
         tmp_list = build_key_list (pt_clist);
         list_str = create_int_arr_str (tmp_list);
         g_string_prepare_db_instr (list_str);
         str = g_string_append (str, list_str->str);
         str = g_string_append (str, ", ");
         g_list_free (tmp_list);
         g_string_free (list_str, TRUE);
         }
      else
         {
         str = g_string_append (str, "NULL, ");
         }

      if (sub_clist->selection)
         {
         tmp_list = build_key_list (sub_clist);
         list_str = create_txt_arr_str (tmp_list);
         g_string_prepare_db_instr (list_str);
         str = g_string_append (str, list_str->str);
         str = g_string_append (str, ", ");
         g_list_free (tmp_list);
         g_string_free (list_str, TRUE);
         }
      else
         {
         str = g_string_append (str, "NULL, ");
         }

      if (resp_clist->selection)
         {
         tmp_list = build_key_list (resp_clist);
         list_str = create_txt_arr_str (tmp_list);
         g_string_prepare_db_instr (list_str);
         str = g_string_append (str, list_str->str);
         str = g_string_append (str, ", ");
         g_list_free (tmp_list);
         g_string_free (list_str, TRUE);
         }
      else
         {
         str = g_string_append (str, "NULL, ");
         }

      tmp = g_string_assign (tmp, gtk_entry_get_text (order_by_entry));
      g_string_prepare_db_instr (tmp);
      str = g_string_append (str, tmp->str);
      str = g_string_append (str, ")");
      } /* end else (not raw) */

   g_string_free (tmp, TRUE);

   return str;
}


static GString *
build_update_string (GnomeDialog *dlg, gint pk, gboolean is_raw)
{
   GtkEntry   *name_entry;
   GtkText    *descr_text;
   GtkText    *raw_sql_text;
   GtkCList   *sevr_clist;
   GtkCList   *stat_clist;
   GtkCList   *pt_clist;
   GtkCList   *sub_clist;
   GtkCList   *resp_clist;
   GtkEntry   *order_by_entry;
   GString    *str;
   GString    *tmp;
   GString    *list_str;
   GList      *tmp_list;
   gboolean    first = TRUE;
   gchar      *buff;

   name_entry = GTK_ENTRY (get_widget (GTK_WIDGET (dlg), NAME_ENTRY));
   g_assert (name_entry != NULL);
   descr_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   g_assert (descr_text != NULL);

   str = g_string_new ("UPDATE pr_query SET ");
   tmp = g_string_new ("");

   if (widget_is_dirty (GTK_WIDGET (name_entry)))
      {
      tmp = g_string_assign (tmp, gtk_entry_get_text (name_entry));
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "name = ");
      str = g_string_append (str, tmp->str);
      }

   if (widget_is_dirty (GTK_WIDGET (descr_text)))
      {
      buff = gtk_editable_get_chars (GTK_EDITABLE (descr_text), 0, -1);
      tmp = g_string_assign (tmp, buff);
      g_free (buff);
      g_string_prepare_db_instr (tmp);
      if (!first)
         {
         str = g_string_append (str, ", ");
         }
      first = FALSE;
      str = g_string_append (str, "descr = ");
      str = g_string_append (str, tmp->str);
      }

   if (is_raw)
      {
      raw_sql_text = GTK_TEXT (get_widget (GTK_WIDGET (dlg), RAW_SQL_TEXT));
      g_assert (raw_sql_text != NULL);
      if (widget_is_dirty (GTK_WIDGET (raw_sql_text)))
         {
         buff = gtk_editable_get_chars (GTK_EDITABLE (raw_sql_text), 0, -1);
         tmp = g_string_assign (tmp, buff);
         g_free (buff);
         g_string_prepare_db_instr (tmp);
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "raw_sql = ");
         str = g_string_append (str, tmp->str);
         }
      } /* end is_raw */
   else
      {
      sevr_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), SEVR_CLIST));
      g_assert (sevr_clist != NULL);
      stat_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), STAT_CLIST));
      g_assert (stat_clist != NULL);
      pt_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), PT_CLIST));
      g_assert (pt_clist != NULL);
      sub_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), SUB_CLIST));
      g_assert (sub_clist != NULL);
      resp_clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), RESP_CLIST));
      g_assert (resp_clist != NULL);
      order_by_entry =
         GTK_ENTRY (get_widget (GTK_WIDGET (dlg), ORDER_BY_ENTRY));
      g_assert (order_by_entry != NULL);

      if (widget_is_dirty (GTK_WIDGET (sevr_clist)))
         {
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "severity_nums = ");
         if (sevr_clist->selection)
            {
            tmp_list = build_key_list (sevr_clist);
            list_str = create_int_arr_str (tmp_list);
            g_string_prepare_db_instr (list_str);
            str = g_string_append (str, list_str->str);
            g_list_free (tmp_list);
            g_string_free (list_str, TRUE);
            }
         else
            {
            str = g_string_append (str, "NULL");
            }
         }

      if (widget_is_dirty (GTK_WIDGET (stat_clist)))
         {
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "status_nums = ");
         if (stat_clist->selection)
            {
            tmp_list = build_key_list (stat_clist);
            list_str = create_int_arr_str (tmp_list);
            g_string_prepare_db_instr (list_str);
            str = g_string_append (str, list_str->str);
            g_list_free (tmp_list);
            g_string_free (list_str, TRUE);
            }
         else
            {
            str = g_string_append (str, "NULL");
            }
         }
            
      if (widget_is_dirty (GTK_WIDGET (pt_clist)))
         {
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "problem_type_nums = ");
         if (pt_clist->selection)
            {
            tmp_list = build_key_list (pt_clist);
            list_str = create_int_arr_str (tmp_list);
            g_string_prepare_db_instr (list_str);
            str = g_string_append (str, list_str->str);
            g_list_free (tmp_list);
            g_string_free (list_str, TRUE);
            }
         else
            {
            str = g_string_append (str, "NULL");
            }
         }

      if (widget_is_dirty (GTK_WIDGET (sub_clist)))
         {
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "submitter_ids = ");
         if (sub_clist->selection)
            {
            tmp_list = build_key_list (sub_clist);
            list_str = create_txt_arr_str (tmp_list);
            g_string_prepare_db_instr (list_str);
            str = g_string_append (str, list_str->str);
            g_list_free (tmp_list);
            g_string_free (list_str, TRUE);
            }
         else
            {
            str = g_string_append (str, "NULL");
            }
         }

      if (widget_is_dirty (GTK_WIDGET (resp_clist)))
         {
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "responsible_ids = ");
         if (resp_clist->selection)
            {
            tmp_list = build_key_list (resp_clist);
            list_str = create_txt_arr_str (tmp_list);
            g_string_prepare_db_instr (list_str);
            str = g_string_append (str, list_str->str);
            g_list_free (tmp_list);
            g_string_free (list_str, TRUE);
            }
         else
            {
            str = g_string_append (str, "NULL");
            }
         }

      if (widget_is_dirty (GTK_WIDGET (order_by_entry)))
         {
         tmp = g_string_assign (tmp, gtk_entry_get_text (order_by_entry));
         g_string_prepare_db_instr (tmp);
         if (!first)
            {
            str = g_string_append (str, ", ");
            }
         first = FALSE;
         str = g_string_append (str, "order_by = ");
         str = g_string_append (str, tmp->str);
         }
      } /* end else (not is_raw) */

   g_string_sprintf (tmp, "%d", pk);
   str = g_string_append (str, " WHERE query_num = ");
   str = g_string_append (str, tmp->str);

   g_string_free (tmp, TRUE);

   return str;
}


static void 
on_ok_button_clicked (GnomeDialog *dlg, gboolean is_raw)
{
   GtkWidget   *parent;    /* the parent query list dialog */
   GtkWidget   *main_win;  /* the main window of the application */
   GString     *sql_buffer;
   gint         pk;
   gboolean     is_sys;
   gboolean     is_update;
   gboolean     in_error = FALSE;
   PGconn      *conn;
   PGresult    *res;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   is_sys = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                   IS_SYS_QUERY_LIST)) == 1);
   is_update = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (dlg),
                                                      IS_UPDATE)) == 1);
   parent = get_widget (GTK_WIDGET (dlg), MY_PARENT);
   main_win = get_widget (parent, MY_PARENT);
   return_value = ID_OK;

   if (validate_query_data (dlg, is_raw))
      {
      START_TRANSACTION (conn);
      if (is_update)
         {
         if (widgets_children_are_dirty (GTK_WIDGET (dlg)))
            {
            pk = GPOINTER_TO_INT
               (gtk_object_get_data (GTK_OBJECT (dlg), PK_NUM));
            sql_buffer = build_update_string (dlg, pk, is_raw);
            res = PQexec (conn, sql_buffer->str);
            in_error = !chk_sql_error (res, "Updating Problem Report Query.");
            PQclear (res);
            g_string_free (sql_buffer, TRUE);
            }
         }
      else
         {
         sql_buffer = build_insert_string (dlg, is_raw);
         res = PQexec (conn, sql_buffer->str);
         in_error = !chk_sql_error (res, "Inserting Problem Report Query.");
         PQclear (res);
         g_string_free (sql_buffer, TRUE);
         }

      if (!in_error)
         {
         COMMIT (conn);
         gnome_dialog_close (dlg);
         draw_query_list (parent, conn, is_sys);
         refresh_query_combo (main_win);
         }
      else
         {
         ROLLBACK (conn);
         }
      }

   return;
}


static void 
on_cancel_button_clicked (GnomeDialog *dlg)
{
   gnome_dialog_close (dlg);
   return_value = ID_CANCEL;
}


/*---------------------------------------------------------------------------
 * Raw SQL Edit Dialog Box
 *--------------------------------------------------------------------------- 
 */
static void 
on_raw_sql_edit_dlg_show (GtkWidget  *widget,
                          gpointer    user_data)
{
   GtkEntry  *name_entry;
   GtkText   *descr_text;
   GtkText   *raw_sql_text;
   GString   *sql_buffer;
   GString   *name;
   GString   *descr;
   GString   *raw_sql;
   PGconn    *conn;
   PGresult  *res;
   gint       pk;

   /*
    * Get the PK first, since we only really need to do something here
    * if we have a valid (non-neg) PK.
    */
   pk = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), PK_NUM));

   if (pk >= 0)
      {
      conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
      g_assert (conn != NULL);

      sql_buffer = g_string_new ("");
      g_string_sprintf (sql_buffer, GET_QUERY, pk);
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Getting raw query information"))
         {
         name_entry = GTK_ENTRY (get_widget (widget, NAME_ENTRY));
         descr_text = GTK_TEXT (get_widget (widget, DESCR_TEXT));
         raw_sql_text = GTK_TEXT (get_widget (widget, RAW_SQL_TEXT));
         g_assert (name_entry != NULL);
         g_assert (descr_text != NULL);
         g_assert (raw_sql_text != NULL);

         name = g_string_new (PQgetvalue (res, 0, NAME_POS));
         descr = g_string_new (PQgetvalue (res, 0, DESCR_POS));
         raw_sql = g_string_new (PQgetvalue (res, 0, RAW_SQL_POS));

         gtk_entry_set_text (name_entry, name->str);
         gtk_text_insert (descr_text, NULL, NULL, NULL, descr->str, -1);
         gtk_text_insert (raw_sql_text, NULL, NULL, NULL, raw_sql->str, -1);

         g_string_free (name, TRUE);
         g_string_free (descr, TRUE);
         g_string_free (raw_sql, TRUE);
         set_all_clean (widget);
         }
      else
         {
         gtk_widget_destroy (widget);
         }
      }
}


static void 
on_raw_sql_edit_dlg_destroy (GtkObject  *object,
                             gpointer    user_data)
{
   destroy_dirty_list (GTK_WIDGET (object));
}


static void
on_raw_sql_edit_dlg_clicked (GnomeDialog *dlg,
                             gint         btn_cd,
                             gpointer     user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_ok_button_clicked (dlg, TRUE);
         break;
      case 1:
         on_cancel_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached();
      }
}


static GtkWidget *
create_raw_sql_edit_dlg (GtkWidget  *parent,
                         PGconn     *conn, 
                         gint        query_num,
                         gboolean    is_update,
                         gboolean    is_system,
                         gboolean    user_is_admin)
{
  GtkWidget    *raw_sql_edit_dlg;
  GtkWidget    *dialog_vbox1;
  GtkWidget    *vbox1;
  GtkWidget    *hbox2;
  GtkWidget    *name_entry;
  GtkWidget    *scrolledwindow2;
  GtkWidget    *descr_text;
  GtkWidget    *hseparator1;
  GtkWidget    *label;
  GtkWidget    *table1;
  GtkWidget    *scrolledwindow1;
  GtkWidget    *raw_sql_text;
  GtkWidget    *popup;
  GtkTooltips  *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  raw_sql_edit_dlg = gnome_dialog_new (_("Raw SQL Query"),
                                       GNOME_STOCK_BUTTON_OK,
                                       GNOME_STOCK_BUTTON_CANCEL,
                                       NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (raw_sql_edit_dlg), GTK_WINDOW (parent));
  gtk_container_border_width (GTK_CONTAINER (raw_sql_edit_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (raw_sql_edit_dlg), "show",
                      GTK_SIGNAL_FUNC (on_raw_sql_edit_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (raw_sql_edit_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_raw_sql_edit_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (raw_sql_edit_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_raw_sql_edit_dlg_clicked),
                      NULL);
  gtk_window_set_policy (GTK_WINDOW (raw_sql_edit_dlg), FALSE, FALSE, TRUE);
  gnome_dialog_set_close (GNOME_DIALOG (raw_sql_edit_dlg), FALSE);
  if (is_system && !user_is_admin)
     {
     gnome_dialog_set_sensitive (GNOME_DIALOG (raw_sql_edit_dlg), 0, FALSE);
     gnome_dialog_set_default (GNOME_DIALOG (raw_sql_edit_dlg), 1);
     }
  else
     {
     gnome_dialog_set_default (GNOME_DIALOG (raw_sql_edit_dlg), 0);
     }

  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), TOP_DOG,
                       raw_sql_edit_dlg);
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), IS_SYS_QUERY_LIST,
                       GINT_TO_POINTER (is_system ? 1 : 0));
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), IS_UPDATE,
                       GINT_TO_POINTER (is_update ? 1 : 0));
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), PK_NUM,
                       GINT_TO_POINTER (query_num));

  dialog_vbox1 = GNOME_DIALOG (raw_sql_edit_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox1), 5);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);

  label = gtk_label_new (_("Query Name:"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox2), label, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), NAME_ENTRY, name_entry);
  gtk_widget_show (name_entry);
  gtk_box_pack_start (GTK_BOX (hbox2), name_entry, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, 
                        name_entry, 
                        _("All good queries must have a name."), 
                        NULL);
  gtk_widget_set_sensitive (name_entry, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (name_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (raw_sql_edit_dlg, name_entry);
  popup = attach_generic_edit_popup_menu (name_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (raw_sql_edit_dlg), "popup1", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  label = gtk_label_new (_("Description"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox1), label, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow2, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  descr_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), 
                       DESCR_TEXT, 
                       descr_text);
  gtk_widget_show (descr_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), descr_text);
  gtk_text_set_editable (GTK_TEXT (descr_text), TRUE);
  gtk_text_set_word_wrap (GTK_TEXT (descr_text), TRUE);
  gtk_widget_set_sensitive (descr_text, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (descr_text), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (raw_sql_edit_dlg, descr_text);
  popup = attach_generic_edit_popup_menu (descr_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (raw_sql_edit_dlg), "popup2", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 5);

  label = gtk_label_new (_("SQL:"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox1), label, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  table1 = gtk_table_new (9, 2, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

  label = gtk_label_new ("SELECT");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("prj.name, stat.name, pt.name, sevr.name,");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("pr.title, stat.order_num, sevr.order_num,");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("pr.problem_num");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("project prj, status stat, problem_type pt,");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("severity sevr, problem_report pr");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 4, 5,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("prj.project_num = pr.project_num");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 5, 6,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("stat.status_num = pr.status_num");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 6, 7,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("sevr.severity_num = pr.severity_num");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 7, 8,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("pt.type_num = pr.type_num");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 8, 9,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new ("FROM");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("WHERE");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 5, 6,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("AND");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 6, 7,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("AND");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 7, 8,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("AND");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 8, 9,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  raw_sql_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), 
                       RAW_SQL_TEXT, 
                       raw_sql_text);
  gtk_widget_show (raw_sql_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), raw_sql_text);
  gtk_widget_set_usize (raw_sql_text, -2, 100);
  gtk_text_set_word_wrap (GTK_TEXT (raw_sql_text), TRUE);
  gtk_tooltips_set_tip (tooltips, 
                        raw_sql_text, 
                        _("Use this area to complete the SQL statement started above."), 
                        NULL);
  gtk_text_set_editable (GTK_TEXT (raw_sql_text), TRUE);
  gtk_widget_set_sensitive (raw_sql_text, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (raw_sql_text), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (raw_sql_edit_dlg, raw_sql_text);
  popup = attach_generic_edit_popup_menu (raw_sql_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (raw_sql_edit_dlg), "popup3", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_object_set_data (GTK_OBJECT (raw_sql_edit_dlg), TOOLTIPS, tooltips);

  return raw_sql_edit_dlg;
}


/*---------------------------------------------------------------------------
 * Regular Query Edit Dialog
 *---------------------------------------------------------------------------
 */
static void 
fill_single_col_clist (GtkCList  *clist,
                       PGresult  *res,
                       gboolean   key_is_numeric)
{
   gint    i, n;
   gchar  *str_pk;
   gchar  *buff[1];
 
   gtk_clist_freeze (clist);
   gtk_clist_clear (clist);
   n = PQntuples (res);
   for (i = 0; i < n; i++)
      {
      buff[0] = g_malloc (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
      strcpy (buff[0], PQgetvalue (res, i, NAME_POS));
      gtk_clist_append (clist, buff);
      g_free (buff[0]);
 
      if (key_is_numeric)
         {
         gtk_clist_set_row_data
            (clist, i, GINT_TO_POINTER (atoi (PQgetvalue (res, i, PK_POS))));
         }
      else
         {
         str_pk = g_malloc (strlen (PQgetvalue (res, i, PK_POS)) + 1);
         strcpy (str_pk, PQgetvalue (res, i, PK_POS));
         gtk_clist_set_row_data_full (clist, i, str_pk, g_free);
         }
      }
   gtk_clist_thaw (clist);
 
   return;
}


/*
 * Given a clist and a list of keys to select, select the given rows
 * by matching row data and keys.
 */
static void 
draw_sel (GtkCList  *clist, GList  *sel)
{
   GList  *iter;
   gint    row = 0;

   gtk_clist_freeze (clist);

   iter = g_list_first (sel);
   while (iter)
      {
      row = gtk_clist_find_row_from_data (clist, iter->data);
      gtk_clist_select_row (clist, row, 0);
      iter = iter->next;
      }

   gtk_clist_moveto (clist, row, 0, 0.5, 0.0); 
   gtk_clist_thaw (clist);

}


/*
 * Same idea as "draw_sel", but works with text keys.
 */
static void 
draw_sel_txt_key (GtkCList  *clist, GList  *sel)
{
   GList   *iter;
   gint     row = 0;
   gint     i, n;

   gtk_clist_freeze (clist);

   n = clist->rows;
   iter = g_list_first (sel);

   while (iter)
      {
      row = -1;
      for (i = 0; i < n; i++)
         {
         if (strcmp ((gchar *)gtk_clist_get_row_data (clist, i),
                     (gchar *)iter->data) == 0)
            {
            row = i;
            break;
            }
         }

      if (row != -1)
         {
         gtk_clist_select_row (clist, row, 0);
         gtk_clist_moveto (clist, row, 0, 0.5, 0.0); 
         }
      iter = iter->next;
      }

   gtk_clist_thaw (clist);
}


/*
 * Fill in the 5 clists on the query edit dlg.
 */
static gboolean 
draw_clists (GtkWidget  *main_widget, PGconn  *conn)
{
   GtkCList    *current_clist;
   PGresult    *res;
   gboolean     in_error = FALSE;
   GString     *sql_buffer;

   sql_buffer = g_string_new ("");
   /*
    * status clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_STATUS);
      sql_buffer = g_string_append (sql_buffer, ORDER_BY_NUM); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Getting status data"))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, STAT_CLIST));
         g_assert (current_clist);
         fill_single_col_clist (current_clist, res, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * problem type clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_PROBLEM_TYPE);
      sql_buffer = g_string_append (sql_buffer, ORDER_BY_NUM); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Getting problem type data"))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, PT_CLIST));
         g_assert (current_clist);
         fill_single_col_clist (current_clist, res, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /*
    * severity clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_SEVERITY);
      sql_buffer = g_string_append (sql_buffer, ORDER_BY_NUM); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Getting severity data"))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, SEVR_CLIST));
         g_assert (current_clist);
         fill_single_col_clist (current_clist, res, TRUE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   /* 
    * submitter clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_PEOPLE);
      sql_buffer = g_string_append (sql_buffer, SUB_RESP_ORDER_BY); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Getting submitters"))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, SUB_CLIST));
         g_assert (current_clist);
         fill_single_col_clist (current_clist, res, FALSE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }
   
   /*
    * responsible clist
    */
   if (!in_error)
      {
      sql_buffer = g_string_assign (sql_buffer, SELECT_PEOPLE);
      sql_buffer = g_string_append (sql_buffer, SUB_RESP_ORDER_BY); 
      res = PQexec (conn, sql_buffer->str);
      if (chk_sql_error (res, "Getting responsible users"))
         {
         current_clist = GTK_CLIST (get_widget (main_widget, RESP_CLIST));
         g_assert (current_clist);
         fill_single_col_clist (current_clist, res, FALSE);
         }
      else
         {
         in_error = TRUE;
         }
      PQclear (res);
      }

   return !in_error;
}


static void
on_sql_edit_dlg_clicked (GnomeDialog *dlg,
                         gint         btn_cd,
                         gpointer     user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_ok_button_clicked (dlg, FALSE);
         break;
      case 1:
         on_cancel_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached ();
      }
}


static void 
on_sql_edit_dlg_show (GtkWidget  *widget,
                      gpointer    user_data)
{
   GtkEntry   *name_entry;
   GtkEntry   *order_by_entry;
   GtkText    *descr_text;
   GtkCList   *sevr_clist;
   GtkCList   *stat_clist;
   GtkCList   *pt_clist;
   GtkCList   *sub_clist;
   GtkCList   *resp_clist;
   GString    *sql_buffer;
   GString    *name;
   GString    *descr;
   GString    *order_by;
   GList      *sevr_sel = NULL;
   GList      *stat_sel = NULL;
   GList      *pt_sel = NULL;
   GList      *sub_sel = NULL;
   GList      *resp_sel = NULL;
   GList      *iter;
   PGconn     *conn;
   PGresult   *res;
   gboolean    in_error = FALSE;
   gint        pk;

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   pk = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), PK_NUM));

   in_error = !draw_clists (widget, conn);

   if (!in_error)
      {
      if (pk >= 0)
         {
         name_entry = GTK_ENTRY (get_widget (widget, NAME_ENTRY));
         descr_text = GTK_TEXT (get_widget (widget, DESCR_TEXT));
         sevr_clist = GTK_CLIST (get_widget (widget, SEVR_CLIST));
         stat_clist = GTK_CLIST (get_widget (widget, STAT_CLIST));
         pt_clist = GTK_CLIST (get_widget (widget, PT_CLIST));
         sub_clist = GTK_CLIST (get_widget (widget, SUB_CLIST));
         resp_clist = GTK_CLIST (get_widget (widget, RESP_CLIST));
         order_by_entry = GTK_ENTRY (get_widget (widget, ORDER_BY_ENTRY));
         g_assert (name_entry != NULL);
         g_assert (descr_text != NULL);
         g_assert (sevr_clist != NULL);
         g_assert (stat_clist != NULL);
         g_assert (pt_clist != NULL);
         g_assert (sub_clist != NULL);
         g_assert (resp_clist != NULL);
         g_assert (order_by_entry != NULL);

         sql_buffer = g_string_new ("");
         g_string_sprintf (sql_buffer, GET_QUERY, pk);
         res = PQexec (conn, sql_buffer->str);
         in_error = !chk_sql_error (res, "Getting query information");
         if (!in_error)
            {
            name = g_string_new (PQgetvalue (res, 0, NAME_POS));
            descr = g_string_new (PQgetvalue (res, 0, DESCR_POS));
            order_by = g_string_new (PQgetvalue (res, 0, ORDER_BY_POS));

            gtk_entry_set_text (name_entry, name->str);
            gtk_entry_set_text (order_by_entry, order_by->str);
            gtk_text_insert (descr_text, NULL, NULL, NULL, descr->str, -1);

            sevr_sel = create_num_list (conn, pk, SEVR_NUMS);
            stat_sel = create_num_list (conn, pk, STAT_NUMS);
            pt_sel = create_num_list (conn, pk, TYPE_NUMS);
            sub_sel = create_txt_list (conn, pk, SUB_IDS);
            resp_sel = create_txt_list (conn, pk, RESP_IDS);

            draw_sel (sevr_clist, sevr_sel);
            draw_sel (stat_clist, stat_sel);
            draw_sel (pt_clist, pt_sel);
            draw_sel_txt_key (sub_clist, sub_sel);
            draw_sel_txt_key (resp_clist, resp_sel);

            g_string_free (name, TRUE);
            g_string_free (descr, TRUE);
            g_string_free (order_by, TRUE);
            iter = g_list_first (sub_sel);
            while (iter)
               {
               g_free (iter->data);
               iter = iter->next;
               }
            iter = g_list_first (resp_sel);
            while (iter)
               {
               g_free (iter->data);
               iter = iter->next;
               }
            g_list_free (sevr_sel);
            g_list_free (stat_sel);
            g_list_free (pt_sel);
            g_list_free (sub_sel);
            g_list_free (resp_sel);
            }
         PQclear (res);
         }
      }

   if (in_error)
      {
      gtk_widget_destroy (widget);
      }
   else
      {
      set_all_clean (widget);
      }
}


static void 
on_sql_edit_dlg_destroy (GtkObject  *object, gpointer  user_data)
{
   destroy_dirty_list (GTK_WIDGET (object));
}


static GtkWidget *
create_sql_edit_dlg (GtkWidget *parent,
                     PGconn    *conn,
                     gint       query_num,
                     gboolean   is_update,
                     gboolean   is_system,
                     gboolean   user_is_admin)
{
  GtkWidget    *sql_edit_dlg;
  GtkWidget    *dialog_vbox2;
  GtkWidget    *vbox2;
  GtkWidget    *hbox3;
  GtkWidget    *label29;
  GtkWidget    *name_entry;
  GtkWidget    *label30;
  GtkWidget    *scrolledwindow3;
  GtkWidget    *descr_text;
  GtkWidget    *hseparator2;
  GtkWidget    *table2;
  GtkWidget    *table3;
  GtkWidget    *label31;
  GtkWidget    *label32;
  GtkWidget    *label33;
  GtkWidget    *label34;
  GtkWidget    *label35;
  GtkWidget    *label36;
  GtkWidget    *scrolledwindow4;
  GtkWidget    *clist1;
  GtkWidget    *scrolledwindow5;
  GtkWidget    *clist2;
  GtkWidget    *scrolledwindow6;
  GtkWidget    *clist3;
  GtkWidget    *scrolledwindow7;
  GtkWidget    *clist4;
  GtkWidget    *scrolledwindow8;
  GtkWidget    *clist5;
  GtkWidget    *order_by_entry;
  GtkWidget    *popup;
  GtkTooltips  *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  sql_edit_dlg = gnome_dialog_new (_("Query Edit"),
                                   GNOME_STOCK_BUTTON_OK,
                                   GNOME_STOCK_BUTTON_CANCEL,
                                   NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (sql_edit_dlg), GTK_WINDOW (parent));
  gtk_container_border_width (GTK_CONTAINER (sql_edit_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (sql_edit_dlg), "show",
                      GTK_SIGNAL_FUNC (on_sql_edit_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (sql_edit_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_sql_edit_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (sql_edit_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_sql_edit_dlg_clicked),
                      NULL);
  gtk_window_set_policy (GTK_WINDOW (sql_edit_dlg), FALSE, FALSE, TRUE);
  gnome_dialog_set_close (GNOME_DIALOG (sql_edit_dlg), FALSE);
  if (is_system && !user_is_admin)
     {
     gnome_dialog_set_sensitive (GNOME_DIALOG (sql_edit_dlg), 0, FALSE);
     gnome_dialog_set_default (GNOME_DIALOG (sql_edit_dlg), 1);
     }
  else
     {
     gnome_dialog_set_default (GNOME_DIALOG (sql_edit_dlg), 0);
     }

  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), TOP_DOG, sql_edit_dlg);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), PK_NUM,
                       GINT_TO_POINTER (query_num));
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), IS_SYS_QUERY_LIST,
                       GINT_TO_POINTER (is_system ? 1 : 0));
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), IS_UPDATE,
                       GINT_TO_POINTER (is_update ? 1 : 0));

  dialog_vbox2 = GNOME_DIALOG (sql_edit_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox2), 5);
  gtk_widget_show (dialog_vbox2);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), vbox2, TRUE, TRUE, 0);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox3, TRUE, TRUE, 0);

  label29 = gtk_label_new (_("Query Name:"));
  gtk_widget_show (label29);
  gtk_box_pack_start (GTK_BOX (hbox3), label29, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label29), 0, 0.5);

  name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), NAME_ENTRY, name_entry);
  gtk_widget_show (name_entry);
  gtk_box_pack_start (GTK_BOX (hbox3), name_entry, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (name_entry, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (name_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, name_entry);
  popup = attach_generic_edit_popup_menu (name_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (sql_edit_dlg), "popup1", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  label30 = gtk_label_new (_("Description:"));
  gtk_widget_show (label30);
  gtk_box_pack_start (GTK_BOX (vbox2), label30, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label30), 0, 0.5);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow3);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow3, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  descr_text = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), DESCR_TEXT, descr_text);
  gtk_widget_show (descr_text);
  gtk_text_set_editable (GTK_TEXT (descr_text), TRUE);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), descr_text);
  gtk_text_set_word_wrap (GTK_TEXT (descr_text), TRUE);
  gtk_widget_set_sensitive (descr_text, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (descr_text), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, descr_text);
  popup = attach_generic_edit_popup_menu (descr_text);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (sql_edit_dlg), "popup2", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator2, TRUE, TRUE, 5);

  table2 = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (vbox2), table2, TRUE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 5);

  label31 = gtk_label_new (_("Severity"));
  gtk_widget_show (label31);
  gtk_table_attach (GTK_TABLE (table2), label31, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label31), 0, 0.5);

  label32 = gtk_label_new (_("Status"));
  gtk_widget_show (label32);
  gtk_table_attach (GTK_TABLE (table2), label32, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label32), 0, 0.5);

  label33 = gtk_label_new (_("Problem Type"));
  gtk_widget_show (label33);
  gtk_table_attach (GTK_TABLE (table2), label33, 2, 3, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label33), 0, 0.5);

  table3 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (vbox2), table3, TRUE, TRUE, 0);
  gtk_table_set_col_spacings (GTK_TABLE (table3), 5);

  label34 = gtk_label_new (_("Submitter"));
  gtk_widget_show (label34);
  gtk_table_attach (GTK_TABLE (table3), label34, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label34), 0, 0.5);

  label35 = gtk_label_new (_("Responsible"));
  gtk_widget_show (label35);
  gtk_table_attach (GTK_TABLE (table3), label35, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label35), 0, 0.5);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow4);
  gtk_table_attach (GTK_TABLE (table2), scrolledwindow4, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  clist1 = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), 
                       SEVR_CLIST, 
                       clist1);
  gtk_widget_show (clist1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), clist1);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 0, STD_COL_WIDTH);
  gtk_clist_set_selection_mode (GTK_CLIST (clist1), GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_hide (GTK_CLIST (clist1));
  gtk_widget_set_sensitive (clist1, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (clist1), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  gtk_signal_connect (GTK_OBJECT (clist1), "unselect_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, clist1);

  scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow5);
  gtk_table_attach (GTK_TABLE (table2), scrolledwindow5, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  clist2 = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), 
                       STAT_CLIST, 
                       clist2);
  gtk_widget_show (clist2);
  gtk_container_add (GTK_CONTAINER (scrolledwindow5), clist2);
  gtk_clist_set_column_width (GTK_CLIST (clist2), 0, STD_COL_WIDTH);
  gtk_clist_set_selection_mode (GTK_CLIST (clist2), GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_hide (GTK_CLIST (clist2));
  gtk_widget_set_sensitive (clist2, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (clist2), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  gtk_signal_connect (GTK_OBJECT (clist2), "unselect_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, clist2);

  scrolledwindow6 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow6);
  gtk_table_attach (GTK_TABLE (table2), scrolledwindow6, 2, 3, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow6), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  clist3 = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), PT_CLIST, clist3);
  gtk_widget_show (clist3);
  gtk_container_add (GTK_CONTAINER (scrolledwindow6), clist3);
  gtk_clist_set_column_width (GTK_CLIST (clist3), 0, STD_COL_WIDTH);
  gtk_clist_set_selection_mode (GTK_CLIST (clist3), GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_hide (GTK_CLIST (clist3));
  gtk_widget_set_sensitive (clist3, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (clist3), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  gtk_signal_connect (GTK_OBJECT (clist3), "unselect_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, clist3);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow7);
  gtk_table_attach (GTK_TABLE (table3), scrolledwindow7, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  clist4 = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), SUB_CLIST, clist4);
  gtk_widget_show (clist4);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), clist4);
  gtk_clist_set_column_width (GTK_CLIST (clist4), 0, (STD_COL_WIDTH * 3) / 2);
  gtk_clist_set_selection_mode (GTK_CLIST (clist4), GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_hide (GTK_CLIST (clist4));
  gtk_widget_set_sensitive (clist4, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (clist4), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  gtk_signal_connect (GTK_OBJECT (clist4), "unselect_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, clist4);

  scrolledwindow8 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow8);
  gtk_table_attach (GTK_TABLE (table3), scrolledwindow8, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow8), 
                                  GTK_POLICY_NEVER, 
                                  GTK_POLICY_AUTOMATIC);

  clist5 = gtk_clist_new (1);
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), RESP_CLIST, clist5);
  gtk_widget_show (clist5);
  gtk_container_add (GTK_CONTAINER (scrolledwindow8), clist5);
  gtk_clist_set_column_width (GTK_CLIST (clist5), 0, (STD_COL_WIDTH * 3) / 2);
  gtk_clist_set_selection_mode (GTK_CLIST (clist5), GTK_SELECTION_MULTIPLE);
  gtk_clist_column_titles_hide (GTK_CLIST (clist5));
  gtk_widget_set_sensitive (clist5, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (clist5), "select_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  gtk_signal_connect (GTK_OBJECT (clist5), "unselect_row",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, clist5);

  label36 = gtk_label_new (_("Order By:"));
  gtk_widget_show (label36);
  gtk_box_pack_start (GTK_BOX (vbox2), label36, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label36), 0, 0.5);

  order_by_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), 
                       ORDER_BY_ENTRY, 
                       order_by_entry);
  gtk_widget_show (order_by_entry);
  gtk_box_pack_start (GTK_BOX (vbox2), order_by_entry, TRUE, TRUE, 0);
  gtk_tooltips_set_tip (tooltips, order_by_entry,
        _("You may use any of the following: prj.name, stat.name, pt.name, \
sevr.name, pr.title, stat.order_num, sevr.order_num, pr.problem_num"),
        NULL);
  gtk_widget_set_sensitive (order_by_entry, user_is_admin || !is_system);
  gtk_signal_connect (GTK_OBJECT (order_by_entry), "changed",
                      GTK_SIGNAL_FUNC (set_dirty), NULL);
  add_widget_to_dirty_list (sql_edit_dlg, order_by_entry);
  popup = attach_generic_edit_popup_menu (order_by_entry);
  gtk_widget_ref (popup);
  gtk_object_set_data_full (GTK_OBJECT (sql_edit_dlg), "popup3", popup,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_object_set_data (GTK_OBJECT (sql_edit_dlg), TOOLTIPS, tooltips);

  return sql_edit_dlg;
}


static void
raise_or_create_edit_dlg (GtkWidget  *parent,
                          PGconn     *conn,
                          gint        query_num,
                          gboolean    is_system,
                          gboolean    user_is_admin,
                          gboolean    query_is_raw)
{
   GtkWidget *dlg;
   GList     *child_list;
   GList     *list_iter;

   /*
    * Go through the existing list of children first.  If we have a 
    * child that already uses this query's PK, just bring that one 
    * to the top.
    */
   child_list = gtk_object_get_data (GTK_OBJECT (parent), my_child_list);
   for (list_iter = g_list_first (child_list);
        list_iter != NULL;
        list_iter = list_iter->next)
      {
      if (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (list_iter->data),
                                                PK_NUM)) == query_num)
         {
         gdk_window_show (GTK_WIDGET (list_iter->data)->window);
         gdk_window_raise (GTK_WIDGET (list_iter->data)->window);

         return;
         }
      }

   /*
    * Didn't find a child already displaying this query, so we need to 
    * create one.
    */
   if (query_is_raw)
      {
      dlg = create_raw_sql_edit_dlg (parent, conn, query_num, TRUE,
                                     is_system, user_is_admin);
      }
   else
      {
      dlg = create_sql_edit_dlg (parent, conn, query_num, TRUE,
                                 is_system, user_is_admin);
      }
   gtk_widget_show (dlg);

   /*
    * Setup the child_dead() callback to remove the child from the 
    * child_list when the child dies.
    */
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);
   child_list = g_list_append (child_list, dlg);
   gtk_object_set_data (GTK_OBJECT (parent), my_child_list, child_list);
}


/*--------------------------------------------------------------------------
 * Main query list window
 *--------------------------------------------------------------------------
 */
static void 
draw_query_list (GtkWidget  *parent,
                 PGconn     *conn,
                 gboolean    is_system)
{
   GtkCList      *clist;
   GString       *sql_buffer;
   PGresult      *res;
   gchar         *list_buffer[NUM_COLS];
   gint           i, n;
   gint           query_pk;
   gint           order_num;
   pk_order_num  *pk_on;

   clist = GTK_CLIST (get_widget (parent, QUERY_CLIST));
   gtk_clist_clear (clist);

   if (is_system)
      {
      sql_buffer = g_string_new (SYSTEM_QUERIES);
      }
   else
      {
      sql_buffer = g_string_new (USER_QUERIES);
      }

   res = PQexec (conn, sql_buffer->str);

   if (chk_sql_error (res, "Getting list of queries"))
      {
      n = PQntuples (res);
      for (i = 0; i < n; i++)
         {
         list_buffer[NAME_COL] = g_malloc 
            (strlen (PQgetvalue (res, i, NAME_POS)) + 1);
         strcpy (list_buffer[NAME_COL], PQgetvalue (res, i, NAME_POS));

         list_buffer[RAW_YN_COL] = g_malloc (YN_BUFF_SIZE * sizeof (gchar));
         if (toupper (PQgetvalue (res, i, IS_RAW_POS)[0]) == 'T')
            {
            strcpy (list_buffer[RAW_YN_COL], YES);
            }
         else
            {
            strcpy (list_buffer[RAW_YN_COL], NO);
            }
         
         list_buffer[DESCR_COL] = g_malloc 
            (strlen (PQgetvalue (res, i, DESCR_POS)) + 1);
         strcpy (list_buffer[DESCR_COL], PQgetvalue (res, i, DESCR_POS));

         query_pk = atoi (PQgetvalue (res, i, QUERY_NUM_POS)); 
         if (PQgetisnull (res, i, ORDER_NUM_POS))
            {
            order_num = -1;
            }
         else
            {
            order_num = atoi (PQgetvalue (res, i, ORDER_NUM_POS));
            }

         gtk_clist_insert (clist, i, list_buffer);
         pk_on = g_malloc (sizeof (pk_order_num));
         pk_on->pk = query_pk;
         pk_on->order_num = order_num;
         gtk_clist_set_row_data_full (clist, i, pk_on, g_free);

         g_free (list_buffer[NAME_COL]);
         g_free (list_buffer[RAW_YN_COL]);
         g_free (list_buffer[DESCR_COL]);
         }
      }
   else
      {
      gtk_widget_destroy (parent);
      }

   PQclear (res);
   g_string_free (sql_buffer, TRUE);
}


static void 
on_query_dlg_show (GtkWidget  *widget, gpointer  user_data)
{
   PGconn    *conn;
   gboolean   is_system;

   conn = gtk_object_get_data (GTK_OBJECT (widget), MY_DB_CONN);
   g_assert (conn);
   is_system = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (widget),
                               IS_SYS_QUERY_LIST)) == 1);

   draw_query_list (widget, conn, is_system);
}


static void 
on_query_dlg_destroy (GtkObject  *object, gpointer  user_data)
{
   GList      *child_list;
   GList      *list_iter;

   /*
    * Kill all of the children
    */
   child_list = gtk_object_get_data (object, my_child_list);
   for (list_iter = g_list_first (child_list);
        list_iter != NULL;
        list_iter = list_iter->next)
      {
      gtk_signal_handler_block_by_func (GTK_OBJECT (list_iter->data),
                                        child_dead,
                                        (gpointer)my_child_list);
      gtk_widget_destroy (GTK_WIDGET (list_iter->data));
      }
   g_list_free (child_list);
}


static void 
on_new_button_clicked (GnomeDialog *dlg)
{
   GtkWidget *child_dlg;
   PGconn    *conn;
   gboolean   is_system;
   gboolean   admin_user;
   GList     *child_list;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn);
   is_system = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                               IS_SYS_QUERY_LIST)) == 1);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg), ADMIN_USER)) == 1);

   child_dlg = create_sql_edit_dlg (GTK_WIDGET (dlg), conn, -1, 
                                    FALSE, is_system, admin_user);
   gtk_widget_show (child_dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when the child dies.
    */
   gtk_signal_connect (GTK_OBJECT (child_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);
 
   child_list = gtk_object_get_data (GTK_OBJECT (dlg), my_child_list);
   child_list = g_list_append (child_list, child_dlg);
   gtk_object_set_data (GTK_OBJECT (dlg), my_child_list, child_list);
}


static void 
on_new_raw_button_clicked (GnomeDialog *dlg)
{
   GtkWidget *child_dlg;
   PGconn    *conn;
   gboolean   is_system;
   gboolean   admin_user;
   GList     *child_list;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn);
   is_system = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                               IS_SYS_QUERY_LIST)) == 1);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                               ADMIN_USER)) == 1);

   child_dlg = create_raw_sql_edit_dlg (GTK_WIDGET (dlg),
                                        conn, -1, FALSE, is_system,
                                        admin_user);
   gtk_widget_show (child_dlg);

   /*
    * Setup the child_dead() callback to remove the child from the
    * child_list when the child dies.
    */
   gtk_signal_connect (GTK_OBJECT (child_dlg), "destroy",
                       GTK_SIGNAL_FUNC (child_dead),
                       (gpointer)my_child_list);
 
   child_list = gtk_object_get_data (GTK_OBJECT (dlg), my_child_list);
   child_list = g_list_append (child_list, child_dlg);
   gtk_object_set_data (GTK_OBJECT (dlg), my_child_list, child_list);
}


static void 
on_modify_button_clicked (GnomeDialog *dlg)
{
   GtkCList   *clist;
   PGconn     *conn;
   gboolean    is_system;
   gboolean    admin_user;
   gboolean    is_raw;
   gint        row;
   gint        pk;
   gchar      *yn_text;
   GtkWidget  *msg_dlg;

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn);
   is_system = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                               IS_SYS_QUERY_LIST)) == 1);
   admin_user = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                               ADMIN_USER)) == 1);

   clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), QUERY_CLIST));
   
   if (!clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("In order to modify a query,\n"
            "you must first select a query to modify."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
   else
      {
      row = GPOINTER_TO_INT (clist->selection->data);
      pk = ((pk_order_num *)gtk_clist_get_row_data (clist, row))->pk;
      gtk_clist_get_text (clist, row, RAW_YN_COL, &yn_text);

      is_raw = (strcmp (yn_text, YES) == 0);
      raise_or_create_edit_dlg (GTK_WIDGET (dlg),
                                conn, pk, is_system,
                                admin_user, is_raw);
      }
}


static void
on_query_sel_row (GtkWidget       *widget,
                  gint             row,
                  gint             column,
                  GdkEventButton  *event,
                  gpointer         user_data)
{
   GtkCList  *clist = GTK_CLIST (widget);
   gint       pk_num;
   gchar     *yn_text;
   PGconn    *conn;
   gboolean   is_system;
   gboolean   admin_user;
   gboolean   is_raw;

   if (event)
      {
      switch (event->type)
         {
         case GDK_2BUTTON_PRESS:
            conn = gtk_object_get_data (GTK_OBJECT (user_data), MY_DB_CONN);
            g_assert (conn);
            is_system = (GPOINTER_TO_INT
               (gtk_object_get_data (GTK_OBJECT (user_data),
                                     IS_SYS_QUERY_LIST)) == 1);
            admin_user = (GPOINTER_TO_INT
               (gtk_object_get_data (GTK_OBJECT (user_data),
                                     ADMIN_USER)) == 1);

            pk_num = ((pk_order_num *)gtk_clist_get_row_data (clist, row))->pk;
            gtk_clist_get_text (clist, row, RAW_YN_COL, &yn_text);

            is_raw = (strcmp (yn_text, YES) == 0);
            raise_or_create_edit_dlg (GTK_WIDGET (user_data), 
                                      conn, pk_num, is_system,
                                      admin_user, is_raw);
            break;

         default:
            break;
         }
      }
}


static void 
on_delete_button_clicked (GnomeDialog *dlg)
{
   GtkCList   *clist;
   PGresult   *res;
   PGconn     *conn;
   GString    *sql_buffer;
   gboolean    is_system;
   gint        row;
   gint        pk;
   GtkWidget  *msg_dlg;
   GtkWidget  *main_win;  /* application main window */

   conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn);
   is_system = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                               IS_SYS_QUERY_LIST)) == 1);

   clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), QUERY_CLIST));
   
   if (!clist->selection)
      {
      msg_dlg = gnome_message_box_new
         (_("In order to delete a query,\n"
            "you must first select a query to delete."),
          GNOME_MESSAGE_BOX_INFO,
          GNOME_STOCK_BUTTON_OK,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 0);
      gnome_dialog_run (GNOME_DIALOG (msg_dlg));
      }
   else
      {
      msg_dlg = gnome_message_box_new
         (_("This will perminently delete the selected query.\n"
            "Are you sure that this is what you want to do?"),
          GNOME_MESSAGE_BOX_QUESTION,
          GNOME_STOCK_BUTTON_YES,
          GNOME_STOCK_BUTTON_NO,
          NULL);
      gnome_dialog_set_default (GNOME_DIALOG (msg_dlg), 1);
      if (gnome_dialog_run (GNOME_DIALOG (msg_dlg)) == 0)
         {
         row = GPOINTER_TO_INT (clist->selection->data);
         pk = ((pk_order_num *)gtk_clist_get_row_data (clist, row))->pk;

         sql_buffer = g_string_new ("");
         g_string_sprintf (sql_buffer, DELETE_QUERY, pk);

         START_TRANSACTION (conn);
         res = PQexec (conn, sql_buffer->str);
         if (chk_sql_error (res, _("Deleting Query")))
            {
            COMMIT (conn);
            }
         else
            {
            ROLLBACK (conn);
            }
         PQclear (res);
         g_string_free (sql_buffer, TRUE);
         draw_query_list (GTK_WIDGET (dlg), conn, is_system);
         main_win = get_widget (GTK_WIDGET (dlg), MY_PARENT);
         g_assert (main_win != NULL);
         refresh_query_combo (main_win);
         }
      }
}


static void 
on_dismiss_button_clicked (GnomeDialog *dlg)
{
   GtkWidget     *main_win;
   GtkCList      *clist;
   pk_order_num  *pk_on;
   PGconn        *conn;
   PGresult      *res;
   GString       *sql_buffer;
   gint           i, n;
   gboolean       user_is_admin;
   gboolean       is_sys_query_list;
   gboolean       in_error = FALSE;
   gboolean       in_trans = FALSE;

   main_win = get_widget (GTK_WIDGET (dlg), MY_PARENT);
   g_assert (main_win != NULL);
   user_is_admin = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg), ADMIN_USER)) == 1);
   is_sys_query_list = (GPOINTER_TO_INT
         (gtk_object_get_data (GTK_OBJECT (dlg),
                                           IS_SYS_QUERY_LIST)) == 1);

   /*
    * Only check the order_num info if the user has the access to 
    * modify it.  Then only change records that absolutely need it.
    */
   if (user_is_admin || !is_sys_query_list)
      {
      clist = GTK_CLIST (get_widget (GTK_WIDGET (dlg), QUERY_CLIST));
      g_assert (clist);
      conn = gtk_object_get_data (GTK_OBJECT (dlg), MY_DB_CONN);
      g_assert (conn);
      n = clist->rows;
      sql_buffer = g_string_new ("");
      for (i = 0; i < n; i++)
         {
         pk_on = gtk_clist_get_row_data (clist, i);
         g_assert (pk_on != NULL);
         if (pk_on->order_num != i)
            {
            if (!in_trans)
               {
               START_TRANSACTION (conn);
               in_trans = TRUE;
               }
            g_string_sprintf (sql_buffer, UPDATE_ORDER_NUM, i, pk_on->pk);
            res = PQexec (conn, sql_buffer->str);
            if (!chk_sql_error (res, "Updating query order_num"))
               {
               PQclear (res);
               in_error = TRUE;
               break;
               }
            PQclear (res);
            }
         }
      g_string_free (sql_buffer, TRUE);

      if (in_trans)
         {
         if (in_error)
            {
            ROLLBACK (conn);
            }
         else
            {
            refresh_query_combo (main_win);
            COMMIT (conn);
            }
         }
      }

   if (is_sys_query_list)
      {
      save_dlg_size (GTK_WIDGET (dlg), QRY_SYS_SEC, FALSE);
      }
   else
      {
      save_dlg_size (GTK_WIDGET (dlg), QRY_USR_SEC, FALSE);
      }
   gnome_dialog_close (dlg);
   return_value = ID_OK;
}


/*
 * These functions are callbacks used in a menu to make the menu
 * items behave just like the associated button presses.
 */
static void
on_new_menu_sel (GtkWidget *widget, gpointer user_data)
{
   on_new_button_clicked (GNOME_DIALOG (user_data));
}

static void
on_new_raw_menu_sel (GtkWidget *widget, gpointer user_data)
{
   on_new_raw_button_clicked (GNOME_DIALOG (user_data));
}

static void
on_mod_menu_sel (GtkWidget *widget, gpointer user_data)
{
   on_modify_button_clicked (GNOME_DIALOG (user_data));
}

static void
on_del_menu_sel (GtkWidget *widget, gpointer user_data)
{
   on_delete_button_clicked (GNOME_DIALOG (user_data));
}


static void
on_query_dlg_clicked (GnomeDialog *dlg,
                      gint         btn_cd,
                      gpointer     user_data)
{
   switch (btn_cd)
      {
      case 0:
         on_new_button_clicked (dlg);
         break;
      case 1:
         on_new_raw_button_clicked (dlg);
         break;
      case 2:
         on_modify_button_clicked (dlg);
         break;
      case 3:
         on_delete_button_clicked (dlg);
         break;
      case 4:
         on_dismiss_button_clicked (dlg);
         break;
      case -1:
         break;
      default:
         g_assert_not_reached ();
      }
}


static gboolean
on_button_press_event (GtkWidget      *widget,
                       GdkEventButton *event,
                       gpointer        user_data)
{
   gint        row;
   gint        col;
   gint        val;
   GtkWidget  *popup;
   GtkWidget  *main_win = GTK_WIDGET (user_data);
   gboolean    admin_user;
   gboolean    is_sys;

   static GnomeUIInfo popup_menu[] = 
      {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Query"),
                                 N_("Create a new regular query."),
                                 on_new_menu_sel, NULL),
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New Raw SQL Query"),
                                 N_("Create a new raw SQL query."),
                                 on_new_raw_menu_sel, NULL),
      GNOMEUIINFO_ITEM_NONE (N_("View/_Modify Query"),
                             N_("View or Modify query's definition."),
                             on_mod_menu_sel),
      GNOMEUIINFO_ITEM_NONE (N_("Delete Query"),
                             N_("Delete the selected query."),
                             on_del_menu_sel),
      GNOMEUIINFO_END
      };

   admin_user = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_win),
                                                       ADMIN_USER)) == 1);
   is_sys = (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (main_win),
                                                   IS_SYS_QUERY_LIST)) == 1);
   if (event)
      {
      switch (event->button)
         {
         case 1:
         case 2:
            break;
         case 3:
            switch (event->type)
               {
               case GDK_BUTTON_PRESS:
                  val = gtk_clist_get_selection_info (GTK_CLIST (widget),
                                                      event->x,
                                                      event->y,
                                                      &row,
                                                      &col);
                  popup = gnome_popup_menu_new (popup_menu);
                  if (val)
                     {
                     gtk_clist_select_row (GTK_CLIST (widget), row, col);
                     }
                  else
                     {
                     /*
                      * We are outside of any items on the list, disable
                      * the modify and delete, and unselect whatever may
                      * be selected.
                      */
                     gtk_widget_set_sensitive (popup_menu[2].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[3].widget,
                                               FALSE);
                     if (GTK_CLIST (widget)->selection)
                        {
                        gtk_clist_unselect_row
                           (GTK_CLIST (widget),
                            GPOINTER_TO_INT 
                            (GTK_CLIST (widget)->selection->data), 0);
                        }
                     }
                  if (is_sys && !admin_user)
                     {
                     gtk_widget_set_sensitive (popup_menu[0].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[1].widget,
                                               FALSE);
                     gtk_widget_set_sensitive (popup_menu[3].widget,
                                               FALSE);
                     }
                  gnome_popup_menu_do_popup_modal (popup,
                                                   NULL,
                                                   NULL,
                                                   event,
                                                   main_win);
                  gtk_widget_destroy (popup);
                  break;
               default:
                  break;
               }
         default:
            break;
         }
      }

   return FALSE;
}


static GtkWidget *
create_query_dlg (GtkWidget *parent,
                  PGconn    *conn,
                  gboolean   is_system,
                  gboolean   user_is_admin)
{
  GtkWidget    *query_dlg;
  GtkWidget    *dialog_vbox3;
  GtkWidget    *vbox3;
  GtkWidget    *label43;
  GtkWidget    *scrolledwindow10;
  GtkWidget    *query_clist;
  GtkWidget    *label44;
  GtkWidget    *label45;
  GtkWidget    *label46;
  GtkTooltips  *tooltips;

  tooltips = gtk_object_get_data (GTK_OBJECT (parent), TOOLTIPS);
  g_assert (tooltips != NULL);

  if (is_system)
     {
     query_dlg = gnome_dialog_new (_("System Query List"), NULL);
     set_dlg_size (query_dlg, QRY_SYS_SEC, 720, 235);
     }
  else
     {
     query_dlg = gnome_dialog_new (_("User Query List"), NULL);
     set_dlg_size (query_dlg, QRY_USR_SEC, 720, 235);
     }
  gnome_dialog_set_parent (GNOME_DIALOG (query_dlg), GTK_WINDOW (parent));
  gtk_container_border_width (GTK_CONTAINER (query_dlg), 5);
  gtk_signal_connect (GTK_OBJECT (query_dlg), "show",
                      GTK_SIGNAL_FUNC (on_query_dlg_show),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (query_dlg), "destroy",
                      GTK_SIGNAL_FUNC (on_query_dlg_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (query_dlg), "clicked",
                      GTK_SIGNAL_FUNC (on_query_dlg_clicked),
                      NULL);
  gtk_window_set_policy (GTK_WINDOW (query_dlg), FALSE, TRUE, FALSE);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (query_dlg),
                                          _("New"),
                                          GNOME_STOCK_PIXMAP_NEW);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (query_dlg),
                                          _("New Raw"),
                                          GNOME_STOCK_PIXMAP_NEW);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (query_dlg),
                                          _("View/Modify"),
                                          GNOME_STOCK_PIXMAP_OPEN);
  gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (query_dlg),
                                          _("Delete"),
                                          GNOME_STOCK_PIXMAP_REMOVE);
  gnome_dialog_append_button (GNOME_DIALOG (query_dlg),
                              GNOME_STOCK_BUTTON_CLOSE);
  gnome_dialog_set_default (GNOME_DIALOG (query_dlg), 4);
  gnome_dialog_set_close (GNOME_DIALOG (query_dlg), FALSE);
  if (is_system && !user_is_admin)
     {
     gnome_dialog_set_sensitive (GNOME_DIALOG (query_dlg), 0, FALSE);
     gnome_dialog_set_sensitive (GNOME_DIALOG (query_dlg), 1, FALSE);
     gnome_dialog_set_sensitive (GNOME_DIALOG (query_dlg), 3, FALSE);
     }

  gtk_object_set_data (GTK_OBJECT (query_dlg), MY_PARENT, parent);
  gtk_object_set_data (GTK_OBJECT (query_dlg), MY_DB_CONN, conn);
  gtk_object_set_data (GTK_OBJECT (query_dlg), ADMIN_USER,
        GINT_TO_POINTER (user_is_admin ? 1 : 0));
  gtk_object_set_data (GTK_OBJECT (query_dlg), IS_SYS_QUERY_LIST,
        GINT_TO_POINTER (is_system ? 1 : 0));

  dialog_vbox3 = GNOME_DIALOG (query_dlg)->vbox;
  gtk_box_set_spacing (GTK_BOX (dialog_vbox3), 5);
  gtk_widget_show (dialog_vbox3);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), vbox3, TRUE, TRUE, 0);

  label43 = gtk_label_new (_("Queries:"));
  gtk_widget_show (label43);
  gtk_box_pack_start (GTK_BOX (vbox3), label43, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label43), 0, 0.5);

  scrolledwindow10 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow10);
  gtk_box_pack_start (GTK_BOX (vbox3), scrolledwindow10, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow10), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);

  query_clist = gtk_clist_new (3);
  gtk_clist_set_reorderable (GTK_CLIST (query_clist), 
                             user_is_admin || !is_system);
  gtk_object_set_data (GTK_OBJECT (query_dlg), QUERY_CLIST, query_clist);
  gtk_widget_show (query_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow10), query_clist);
  gtk_clist_set_column_width (GTK_CLIST (query_clist), 0, 150);
  gtk_clist_set_column_width (GTK_CLIST (query_clist), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (query_clist), 2, 200);
  gtk_clist_column_titles_show (GTK_CLIST (query_clist));
  gtk_signal_connect (GTK_OBJECT (query_clist), "select_row",
                      GTK_SIGNAL_FUNC (on_query_sel_row),
                      query_dlg);
  gtk_signal_connect (GTK_OBJECT (query_clist), "button_press_event",
                      GTK_SIGNAL_FUNC (on_button_press_event),
                      query_dlg);

  label44 = gtk_label_new (_("Name"));
  gtk_widget_show (label44);
  gtk_clist_set_column_widget (GTK_CLIST (query_clist), 0, label44);

  label45 = gtk_label_new (_("Is Raw SQL?"));
  gtk_widget_show (label45);
  gtk_clist_set_column_widget (GTK_CLIST (query_clist), 1, label45);

  label46 = gtk_label_new (_("Description"));
  gtk_widget_show (label46);
  gtk_clist_set_column_widget (GTK_CLIST (query_clist), 2, label46);

  gtk_object_set_data (GTK_OBJECT (query_dlg), TOOLTIPS, tooltips);

  return query_dlg;
}


void 
destroy_query_dlg (void)
{
   if (sys_dlg != NULL)
      {
      gtk_widget_destroy (sys_dlg);
      sys_dlg = NULL;
      }

   if (user_dlg != NULL)
      {
      gtk_widget_destroy (user_dlg);
      user_dlg = NULL;
      }
}


void
show_query_dlg (GtkWidget *parent,
                PGconn    *conn,
                gboolean   is_system,
                gboolean   user_is_admin)
{
   GtkWidget        **the_dlg;

   if (is_system)
      {
      the_dlg = &user_dlg;
      }
   else
      {
      the_dlg = &sys_dlg;
      }

   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (*the_dlg != NULL)
      {
      gdk_window_show ((*the_dlg)->window);
      gdk_window_raise ((*the_dlg)->window);
      }
   else
      {
      *the_dlg = create_query_dlg (parent, conn, is_system, user_is_admin);
 
      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(),
       * which will set the "user_data" (&person_dlg in this case) to
       * NULL.
       */
      gtk_signal_connect (GTK_OBJECT (*the_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (gtk_widget_destroyed),
                          the_dlg);
      gtk_widget_show (*the_dlg);
      }
}


gboolean
query_dlg_dirty (void)
{
   gboolean dirty = FALSE;

   if (sys_dlg != NULL)
      {
      dirty = list_dlg_has_dirty_child (sys_dlg, my_child_list);
      }

   if (!dirty && user_dlg != NULL)
      {
      dirty = list_dlg_has_dirty_child (user_dlg, my_child_list);
      }

   return dirty;
}
